SystemOrganization addCategory: 'Orbeon'!

AbstractApplication subclass: #OrbeonApplication
	instanceVariableNames: 'definitionRegistry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonApplication class methodsFor: 'application operations' stamp: 'BrunoBB 04/27/2016 14:52'!
applicationNamed: aString ifNone: aBlock
	"Answer an OrbeonApplication or BpmApplication named <aString> if none evaluates the zero block argument <aBlock>"

	^self getOrbeonCache detect: [:each | each applicationName = aString] ifNone: aBlock! !

!OrbeonApplication class methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
exampleRegistry
	| form app reg |

	app := self getOrbeonApplicationClass applicationNamed: 'asse'.
	reg := app registryForFormNamed: 'relevamiento_prestaciones_2015'.
	form := reg getLastVersion.

	^(SinglelineSimpleFormExport definition: form formsCollection: form instances) generatePrintArray.! !

!OrbeonApplication class methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
formDefinitionNamed: aString in: appName
	"The receiver answer an <OrbeonFormDefinition> named <aString> in OrbeonApplication named <appName>.
	If the form definition does not exist then answer <nil>"
	| app |

	app := self applicationNamed: appName ifNone: [^nil].

	^app definitionNamed: aString! !

!OrbeonApplication class methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
formsDefinitionNamed: aString
	| forms |

	forms := OrderedCollection new.
	self getOrbeonCache do:[:each | | form |
		form := each definitionNamed: aString.
		form ifNotNil: [forms add: form].].
	^forms! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllDefinitionsWithId: stringId ifNone: aBlock
	| definitions |

	definitions := OrderedCollection new.

	(self getDefinitionWithId: stringId ifNone: [nil]) ifNotNil: [:def | definitions add: def].

	(self getSavedDefinitionWithId: stringId ifNone: [nil]) ifNotNil: [:def | definitions add: def].

	definitions isEmpty ifTrue: [^aBlock value].

	^definitions! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllFormDefinitions
	| definitions |

	definitions := OrderedCollection new.
	self getOrbeonCache do: [:each | definitions addAll: each getAllDefinitions].

	^definitions! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllInstances
	| instances |

	instances := OrderedCollection new.
	self getOrbeonCache do: [:each | instances addAll: each getAllInstances].

	^instances! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllInstancesOf: appName definitionNamed: defName
	""
	| application registry |

	application := self getOrbeonApplicationClass applicationNamed: appName ifNone: [^#()].
	registry := application registryForFormNamed: defName ifNone: [^#()].

	^registry getInstancesOfAllVersions! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 05/04/2016 13:51'!
getAllPublishedDefinitionNamed: defName fromApplicationNamed: appName ifNone: aBlock
	"Answer <anOrderedCollection> with all <OrbeonFormDefinition> named <defName> from the OrbeonApplication named <appName> if none evaluate <aBlock>"
	| app |

	app := self applicationNamed: appName ifNone: [^aBlock value].

	(app hasDefinitionNamed: defName) ifFalse: [^aBlock value].

	^app getAllDefinitionsNamed: defName ifNone: aBlock! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 05/04/2016 13:58'!
getAllPublishedDefinitionNamed: defName fromApplicationNamed: appName withVersion: version ifNone: aBlock
	"Answer <anOrderedCollection> with all <OrbeonPublishedFormDefinition> named <defName> from the OrbeonApplication named <appName> wtih version <version> if none evaluate <aBlock>"
	| app |

	app := self applicationNamed: appName ifNone: [^aBlock value].

	(app hasDefinitionNamed: defName) ifFalse: [^aBlock value].

	^(app getAllDefinitionsNamed: defName ifNone: [^aBlock value]) select: [:each | each version = version]! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllSavedDefinitions
	| all |

	all := OrderedCollection new.
	self getOrbeonCache do: [:each | all addAll: each getAllSavedDefinitions].

	^all! !

!OrbeonApplication class methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAnyDefinitionWithId: stringId ifNone: aBlock
	"The receiver answer any form definition with id <stringId> if none evaluate the block <aBlock>.
	1- Search in all Published Forms.
	2- If there is no published form search in saved forms"

	^self getDefinitionWithId: stringId ifNone: [self getSavedDefinitionWithId: stringId ifNone: aBlock]! !

!OrbeonApplication class methodsFor: 'draft instance forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAnyDraftWith: id formName: formNamed in: appName ifNone: aBlock
	| application definitions |

	application := self applicationNamed: appName ifNone: [^aBlock value].
	definitions := application getAllDefinitionsNamed: formNamed ifNone: [^aBlock value].

	definitions do: [:each | (each getDraftWithId: id) ifNotNil: [:form | ^form]].

	^aBlock value! !

!OrbeonApplication class methodsFor: 'instance forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAnyInstanceWith: id formName: formNamed in: appName ifNone: aBlock
	| application definitions |

	application := self applicationNamed: appName ifNone: [^aBlock value].
	definitions := application getAllDefinitionsNamed: formNamed ifNone: [^aBlock value].

	definitions do: [:each | (each getInstanceWithId: id) ifNotNil: [:form | ^form]].

	^aBlock value! !

!OrbeonApplication class methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
getDefinitionWithId: stringId ifNone: aBlock
	| application |

	application := self getOrbeonCache detect: [:eachApp | (eachApp getDefinitionWithId: stringId ifNone: [nil]) notNil] ifNone: [^aBlock value].

	^application getDefinitionWithId: stringId ifNone: aBlock! !

!OrbeonApplication class methodsFor: 'application operations' stamp: 'BrunoBB 04/27/2016 15:20'!
getExistingApplicationOrCreateNewNamed: aString
	"The receiver answer a instance of it self named <aString>. 
	If it is present in the cache --> answer the existing one.
	If absent --> create a new one"

	^self cache 
		at: aString 
		ifPresent: [:application | application]
		ifAbsent: [super new initialize
						applicationName: aString;
						addToCache;
						yourself]! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/01/2016 10:04'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	(self cache select: [:each | each class == self]) do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/01/2016 09:58'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger
	| distribution |

	distribution := Dictionary new.
	(self cache select: [:each | each class == self]) do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 11:55'!
getInstancesCreationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	(self cache select: [:each | each class == self]) do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDistributionByMonthFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 12:01'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	(self cache select: [:each | each class == self]) do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/01/2016 10:39'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger
	| distribution |

	distribution := Dictionary new.
	(self cache select: [:each | each class == self]) do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 11:55'!
getInstancesModificationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	(self cache select: [:each | each class == self]) do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDistributionByMonthFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 07:03'!
getInstancesQuantityFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	(self cache select: [:each | each class == self]) do: [:each | 
		distribution at: each applicationName put: (each getInstancesQuantityFrom: startDate to: endDate).
	].
		
	^distribution! !

!OrbeonApplication class methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getLastDefinitions
	| lasts |

	lasts := OrderedCollection new.
	self getOrbeonCache do: [:each | lasts addAll: each getLastDefinitions].

	^lasts! !

!OrbeonApplication class methodsFor: 'all forms' stamp: 'BrunoBB 05/04/2016 13:52'!
getLastPublishedDefinitionNamed: defName fromApplicationNamed: appName withVersion: version ifNone: aBlock
	"Answer the OrbeonFormDefinition named <defName> and version equal to <version> from the OrbeonApplication named <appName> if none evaluate <aBlock>"
	| app |

	app := self applicationNamed: appName ifNone: [^aBlock value].

	(app hasDefinitionNamed: defName) ifFalse: [^aBlock value].

	^(app getAllDefinitionsNamed: defName) detect: [:each | each version = version] ifNone: [aBlock value]! !

!OrbeonApplication class methodsFor: 'saved forms' stamp: 'BrunoBB 04/30/2016 16:18'!
getLatestSavedDefinition: formName app: app 
	"Answer the lastest saved definition named <formName> in orbeon application named <app>.
	If any search fail then rise an exception"

	^self getLatestSavedDefinition: formName app: app ifNone: [^NoApplicationException signal: 'No application named [', app,'] was found']! !

!OrbeonApplication class methodsFor: 'saved forms' stamp: 'BrunoBB 04/30/2016 17:15'!
getLatestSavedDefinition: formName app: app ifNone: aBlock
	"Answer the lastest saved definition named <formName> in orbeon application named <app>.
	If any search fail then rise an exception"
	| application |

	application := self applicationNamed: app ifNone: [^aBlock value].

	^application getLatestSavedDefinition: formName ifNone: [^aBlock value]! !

!OrbeonApplication class methodsFor: 'other' stamp: 'BrunoBB 10/01/2016 10:11'!
getNumberOfFormInstances

	^(self cache select: [:each | each class == self]) inject: 0 into:  [:sum :each | sum + each getNumberOfInstances]! !

!OrbeonApplication class methodsFor: 'saved forms' stamp: 'BrunoBB 12/06/2016 13:55'!
getSavedDefinition: formName app: app withId: id
	"Answer the saved definition named <formName> with id <id> in orbeon application named <app>.
	If any search fail then rise an exception"
	| application |

	application := self applicationNamed: app ifNone: [^NoApplicationException signal: 'No application named [', app,'] was found'].

	^application 
		getSavedDefinitionFor: formName 
		withId: id 
		ifNone: [^NoOrbeonFormException signal: ('For form named [', formName, '] was found in application [', app, '] with id [', id, ']')].! !

!OrbeonApplication class methodsFor: 'saved forms' stamp: 'BrunoBB 04/30/2016 09:25'!
getSavedDefinitionWithId: stringId ifNone: aBlock
	"The receiver answer <anOrbeonSavedFormDefinition> with id <stringId> if none evaluate the block <aBlock> "
	| appCollection |

	appCollection := (self getOrbeonCache select: [:eachApp | (eachApp getSavedDefinitionWithId: stringId ifNone: [nil]) notNil]) asArray.

	^appCollection isEmpty
		ifFalse: [appCollection last getSavedDefinitionWithId: stringId ifNone: aBlock]
		ifTrue: aBlock! !

!OrbeonApplication class methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
hasFormDefinitionNamed: aString


	^(self definitionNamed: aString) notEmpty! !

!OrbeonApplication class methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
includesDefinitionNamed: aString in: appName
	| forms app |

	app := self applicationNamed: appName ifNone: [^false].

	^(app definitionNamed: aString) ifNil: [false] ifNotNil: [true]! !

!OrbeonApplication class methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:20'!
initializeAllCache
	"self initializeCache"

	self initializeCache.
	OrbeonPublishedFormDefinition initializeCache.
	OrbeonFormInstance initializeCache.
	OrbeonSavedFormDefinition initializeCache.! !

!OrbeonApplication class methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
logicalRemoveDefinitionWithId: stringId ifNone: aBlock
	| instanceDefinition |

	instanceDefinition := self getSavedDefinitionWithId: stringId ifNone: [^aBlock value].

	instanceDefinition logicalDelete. "delete the OrbeonFormInstanceDefinition"

	instanceDefinition definition logicalDelete.  "delete the OrbeonFormDefinition"! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 05:50'!
reportCreationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate.

	^(OrbeonGlobalDistributionReport distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			yourself! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:42'!
reportCreationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesCreationDistributionByMonthFrom: startDate to: endDate.

	^(OrbeonGlobalDistributionReport distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			yourself! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 05:50'!
reportModificationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate.

	^(OrbeonGlobalDistributionReport distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			yourself! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:42'!
reportModificationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesModificationDistributionByMonthFrom: startDate to: endDate.

	^(OrbeonGlobalDistributionReport distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			yourself! !

!OrbeonApplication class methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 04:18'!
reportQuantityDistributionFrom: startDate to: endDate
	| secondsToRun distribution |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesQuantityFrom: startDate to: endDate].

	^(OrbeonGlobalDistributionReport distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			sumSecondsToConstruct: secondsToRun;
			yourself! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
addPublishedDefinition: anOrbeonFormDefinition
	"Add a new form definition <anOrbeonFormDefinition> to receiver's registry"

	(self hasRegistryFor: anOrbeonFormDefinition) "check if the registry already exist, if not it creates it"
	ifFalse: [self createRegistryFor: anOrbeonFormDefinition].

	"add the definition to the registry"
	^(self registryFor: anOrbeonFormDefinition) addCurrentVersion: anOrbeonFormDefinition! !

!OrbeonApplication methodsFor: 'saved forms' stamp: 'BrunoBB 04/27/2016 14:52'!
addSavedDefinition: anOrbeonFormInstanceDefinition
	"Add a new saved definition <anOrbeonFormInstanceDefinition> to receiver's registry"

	(self hasRegistryFor: anOrbeonFormInstanceDefinition) "check if the registry already exist, if not it creates it"
	ifFalse: [self createRegistryFor: anOrbeonFormInstanceDefinition].

	"add the definition to the registry"
	^(self registryFor: anOrbeonFormInstanceDefinition) addSavedDefinition: anOrbeonFormInstanceDefinition! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
addTestPermissions

	self getLastDefinitions do: [:each | each addTestPermissions].! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
canDelete
	"Answer whether or not the receiver can be deleted from the system"

	Smalltalk at: #BpmApplication ifAbsent: [^definitionRegistry isEmpty]. "the BPM application is NOT installed in the system"

	^definitionRegistry isEmpty and: [super canDelete] "the BPM application is installed in the system --> check BPM dependencies"! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 07/28/2016 10:35'!
createRegistryFor: anOrbeonFormDefinition
	"The receiver creates a definition registry for <anOrbeonFormDefinition>"

	^self setRegistryFormNamed: anOrbeonFormDefinition formName with: (FormDefinitionRegistry newFor: self)! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
definitionNamed: aString
	"Answer the Last Available version of anOrbeonFormDefinition named <aString> if none answer <nil>."

	^self definitionNamed: aString ifNone: [nil]! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
definitionNamed: aString ifNone: aBlock
	"Answer the latest version of anOrbeonFormDefinition named <aString> if none evaluate <aBlock>."

	^(self registryForFormNamed: aString ifNone: [^aBlock value]) getLastVersion! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 12/09/2016 08:37'!
definitionNamed: aString version: versionNumber ifNone: aBlock
	"Answer the latest version of anOrbeonFormDefinition named <aString> if none evaluate <aBlock>."

	^(self registryForFormNamed: aString ifNone: [^aBlock value]) getDefinitionVersion: versionNumber ifNone: aBlock! !

!OrbeonApplication methodsFor: 'Accessing' stamp: 'BrunoBB 04/27/2016 14:52'!
definitionRegistry

   "Return the value of the instance variable 'definitionRegistry'."
   ^definitionRegistry! !

!OrbeonApplication methodsFor: 'Updating' stamp: 'BrunoBB 04/27/2016 14:52'!
definitionRegistry: newValue

   "Modify the value of the instance variable 'definitionRegistry'."
   definitionRegistry := newValue! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllDefinitions
	"The receiver answer anOrderedCollection with all OrbeonFormDefinition.
	Iterate over all definition registry asking all versions"
	| all |

	all := OrderedCollection new.
	(definitionRegistry select: [:each | each hasVersions]) do: [:each | all addAll: each getAllVersion].

	^all! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 05/04/2016 13:35'!
getAllDefinitionsNamed: aString
	"Answer a collection with all versions of OrbeonFormDefinition named <aString> if none answer an empty collection.
	It can be more than one definition with version 1 --> answer the latest version 1"

	^self getAllDefinitionsNamed: aString ifNone: [OrderedCollection new].! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllDefinitionsNamed: aString ifNone: aBlock
	"Answer a collection with all latest versions of OrbeonFormDefinition named <aString> if none then evaluate <aBlock>.
	It can be more than one definition with version 1 --> answer the latest version 1"
	| registry |

	(self hasRegistryForFormNamed: aString) ifFalse: [^aBlock value].

	registry := self registryForFormNamed: aString.

	^registry hasVersions 
		ifTrue: [registry oldVersions, registry currentVersions] 
		ifFalse: [aBlock value]! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 05/04/2016 13:48'!
getAllDefinitionsNamed: aString withVersion: versionNumber ifNone: aBlock
	"Answer a collection with all latest versions of OrbeonFormDefinition named <aString> if none then evaluate <aBlock>.
	It can be more than one definition with version 1 --> answer the latest version 1"

	^(self getAllDefinitionsNamed: aString ifNone: [^aBlock value]) select: [:each | each version = versionNumber]! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllInstances
	"Answer a collection with all form instances <OrbeonFormInstance> of the receiver.
	Collect instances from all receiver's registries <FormDefinitionRegistry>"
	| all |

	all := OrderedCollection new.
	definitionRegistry do: [:each | all addAll: each getAllInstances].

	^all! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllInstancesKeysOfFormNamed: formName
	"The receiver answer all keys of form instances of form definition named <formName>"

	^(self registryForFormNamed: formName ifNone: [^#()]) getAllInstancesKeys! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllSavedDefinitions
	| all |

	all := OrderedCollection new.
	definitionRegistry do: [:each | all addAll: each getSavedDefinitions].

	^all! !

!OrbeonApplication methodsFor: 'all forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAllSearchFieldsOf: formName
	"The receiver answer a collection with all search fields of the form definition named <formName>"

	^(self registryForFormNamed: formName ifNone: [^#()]) getAllSearchFields! !

!OrbeonApplication methodsFor: 'drafts forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAnyDraftWithId: stringId of: formName
	"The receiver answer anOrbeonFormInstance with id <stringId> if none evaluates the block <aBlock>"

	^(self registryForFormNamed: formName ifNone: [^nil]) getAnyDraftWithId: stringId ifNone: [nil]! !

!OrbeonApplication methodsFor: 'instance forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAnyInstanceWithId: stringId of: formName
	"The receiver answer anOrbeonFormInstance with id <stringId> if none evaluates the block <aBlock>"

	^self getAnyInstanceWithId: stringId of: formName ifNone: [nil]! !

!OrbeonApplication methodsFor: 'instance forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getAnyInstanceWithId: stringId of: formName ifNone: aBlock
	"The receiver answer anOrbeonFormInstance with id <stringId> if none evaluates the block <aBlock>"

	^(self registryForFormNamed: formName ifNone: [^aBlock value]) getAnyInstanceWithId: stringId ifNone: aBlock! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
getApplicationNameForCombo

	^applicationName , ' [Orbeon]'! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getDefinitionNamed: aString withVersion: version ifNone: aBlock
	"Answer the latest version of anOrbeonFormDefinition named <aString> with version <version> if none evaluate <aBlock>."

	^(self registryForFormNamed: aString ifNone: [^aBlock value]) getDefinitionVersion: version ifNone: aBlock! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getDefinitionWithId: stringId ifNone: aBlock
	"The receiver answer anOrbeonFormDefinition with id <stringId> if none evaluates the block <aBlock>.
	1- Goes over all FormDefinitionRegistry of the receiver asking if contains the id <stringId>.
	2- Use the registry the get the form definition"
	| registry |

	registry := definitionRegistry detect: [:eachReg | (eachReg getDefinitionWithId: stringId ifNone: [nil]) notNil] ifNone: [^aBlock value].

	^registry getDefinitionWithId: stringId ifNone: aBlock! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 09/30/2016 09:19'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 09/30/2016 10:51'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 11:46'!
getInstancesCreationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDistributionByMonthFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 17:58'!
getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 09/23/2016 05:25'!
getInstancesDistributionPercentage
	"Answer aDictionary where each key is the name of the form definition and value is the number of instances of that form"
	| distribution total orderedPercentage |

	distribution := Dictionary new.
	orderedPercentage := OrderPreservingDictionary new.
	total := 0.

	definitionRegistry keysAndValuesDo: [:formName :registry | 
		distribution at: formName put: registry getAllInstancesSize.
		total := total + (distribution at: formName)].

	distribution keysAndValuesDo: [:formName :value |  | instanceNumber |
		instanceNumber := value * 100 / total.
		distribution at: formName put: instanceNumber.
	].

	distribution asSortedCollection reverse do: [:each | | key |
		key := distribution keyAtValue: each ifAbsent: [].
		orderedPercentage at: key put: each.
		distribution removeKey: key ifAbsent: [].
		
	].

	^orderedPercentage! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 09/24/2016 08:51'!
getInstancesDistributionQuantity
	"Answer aDictionary where each key is the name of the form definition and value is the number of instances of that form"
	| distribution total orderedQuantity |

	distribution := Dictionary new.
	orderedQuantity := OrderPreservingDictionary new.
	total := 0.

	definitionRegistry keysAndValuesDo: [:formName :registry | 
		distribution at: formName put: registry getAllInstancesSize].

	distribution asSortedCollection reverse do: [:each | | key |
		key := distribution keyAtValue: each ifAbsent: [].
		orderedQuantity at: key put: each.
		distribution removeKey: key ifAbsent: [].
		
	].

	^orderedQuantity! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 09:46'!
getInstancesDistributionQuantityFrom: startDate to: endDate
	"Answer aDictionary where each key is the name of the form definition and value is the number of instances of that form"
	| distribution orderedQuantity |

	distribution := Dictionary new.
	orderedQuantity := OrderPreservingDictionary new.

	definitionRegistry keysAndValuesDo: [:formName :registry | | registryDistribution total |
		registryDistribution := registry getInstancesDistributionQuantityFrom: startDate to: endDate."answer a dictionary key->value"
		total := registryDistribution inject: 0 into: [:sum :each | sum + each].
		distribution at: formName put: total].

	distribution asSortedCollection reverse do: [:each | | key |
		key := distribution keyAtValue: each ifAbsent: [].
		orderedQuantity at: key put: each.
		distribution removeKey: key ifAbsent: [].
		
	].

	^orderedQuantity! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/18/2016 11:38'!
getInstancesDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock
	"Answer aDictionary where each key is the name of the form definition and value is the number of instances of that form"
	| distribution total orderedQuantity |

	distribution := Dictionary new.
	orderedQuantity := OrderPreservingDictionary new.
	total := 0.

	definitionRegistry keysAndValuesDo: [:formName :registry | 
		distribution at: formName put: (registry getInstancesDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock)].

	distribution asSortedCollection reverse do: [:each | | key |
		key := distribution keyAtValue: each ifAbsent: [].
		orderedQuantity at: key put: each.
		distribution removeKey: key ifAbsent: [].
		
	].

	^orderedQuantity! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 11:41'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 09/30/2016 10:54'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 11:33'!
getInstancesModificationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDistributionByMonthFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 17:59'!
getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := Dictionary new.
	self getAllDefinitions do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 10/07/2016 06:57'!
getInstancesQuantityFrom: startDate to: endDate
	""
	| total |

	total := 0.

	definitionRegistry keysAndValuesDo: [:formName :registry | 
		total := total + (registry getInstancesQuantityFrom: startDate to: endDate)].

	^total! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getLastDefinitions
	"The receiver answer a collection with the latest version of all registered form definitions"

	^((definitionRegistry select: [:each | each hasVersions]) collect: [:each | each getLastVersion]) select: [:each | each deleted = 'N']! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getLastFormDefinitionVersionOf: formName
	"The receiver answer the last version number of the form definition named <formName>"

	^(self registryForFormNamed: formName ifNone: [^0]) getLastVersionNumber! !

!OrbeonApplication methodsFor: 'saved forms' stamp: 'BrunoBB 12/06/2016 13:54'!
getLatestSavedDefinition: formName
	"The receiver answer <anOrbeonFormInstanceDefinition> with id <stringId> if none evaluate the block <aBlock>"

	^self getLatestSavedDefinition: formName ifNone: [^NoOrbeonFormException signal: ('No form saved definition named [', formName, '] was found in application [', applicationName,']')]! !

!OrbeonApplication methodsFor: 'saved forms' stamp: 'BrunoBB 04/30/2016 17:14'!
getLatestSavedDefinition: formName ifNone: aBlock
	"The receiver answer <anOrbeonFormInstanceDefinition> with id <stringId> if none evaluate the block <aBlock>"
	| registry |

	registry := (self registryForFormNamed: formName ifNone: [^aBlock value]).

	^registry getLastSavedDefinition! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 10/01/2016 10:11'!
getNumberOfInstances
	""
	| total |

	total := 0.

	definitionRegistry keysAndValuesDo: [:formName :registry | 
		total := total + registry getAllInstancesSize].

	^total! !

!OrbeonApplication methodsFor: 'saved forms' stamp: 'BrunoBB 04/30/2016 09:21'!
getSavedDefinitionFor: formName withId: stringId ifNone: aBlock
	"The receiver answer <anOrbeonFormInstanceDefinition> with id <stringId> if none evaluate the block <aBlock>.
	Answer the last save definition"

	^(self registryForFormNamed: formName ifNone: [^aBlock value]) getSavedDefinitionWithId: stringId ifNone: aBlock! !

!OrbeonApplication methodsFor: 'saved forms' stamp: 'BrunoBB 04/27/2016 14:52'!
getSavedDefinitionWithId: stringId ifNone: aBlock
	"The receiver answer <anOrbeonFormInstanceDefinition> with id <stringId> if none evaluate the block <aBlock>"
	| registry |

	registry := definitionRegistry detect: [:eachReg | eachReg includesSavedDefinitionWithId: stringId] ifNone: [^aBlock value].

	^registry getSavedDefinitionWithId: stringId ifNone: aBlock! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 04/27/2016 14:52'!
hasDefinitionNamed: aString
	"Answer whether or not the receiver has a form definiton named <aString>"

	^(self hasRegistryForFormNamed: aString) and:[(self registryForFormNamed: aString) hasVersions]! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
hasReadAccess: anOrbeonUser
	"Answer whether or not the user <anOrbeonUser> has read permission over the receiver.
	If an user has read permission over a definition then has read access over the application (the receiver)"

	^self getLastDefinitions anySatisfy: [:each | anOrbeonUser hasReadPermissionOn: each]! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/28/2016 11:03'!
hasRegistryFor: anOrbeonFormDefinition
	"Answer whether or not the receiver has a registry for form definition <anOrbeonFormDefinition>"

	^self hasRegistryForFormNamed: anOrbeonFormDefinition formName! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/27/2016 14:52'!
hasRegistryForFormNamed: aString
	"Answer whether or not the receiver has a registry for form definition named <aString>"

	^definitionRegistry includesKey: aString! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
initialize

	super initialize.

	applicationName := ''.

	definitionRegistry := RcKeyValueDictionary new. "aDictionary of Collections (each Collection with OrbeonFormDefinition objects)"! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 08/31/2016 11:49'!
isObeonApplication

	^true! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
numberOfInstances
	"Answer the number of instances of all form definitions combined"

	^self getAllDefinitions inject: 0 into: [:sum :each | sum + each numberOfInstances]! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: ' (', applicationName, ')'.! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 08/31/2016 12:02'!
printType

	^'Orbeon'! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/28/2016 11:04'!
registryFor: anOrbeonFormDefinition
	"Answer the registry <FormDefinitionRegistry> for the form definition <anOrbeonFormDefinition> if none raise an Exception"

	^self registryForFormNamed: anOrbeonFormDefinition formName
			ifNone: [FormRegistryNotExistException signal: 'No registry available for :', anOrbeonFormDefinition formName]! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/27/2016 14:52'!
registryForFormNamed: aString
	"Answer the registry <FormDefinitionRegistry> for the form definition named <aString> if none the raise an Exception"

	^self registryForFormNamed: aString ifNone: [FormRegistryNotExistException signal: 'No registry available for :', aString]! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/27/2016 14:52'!
registryForFormNamed: aString ifNone: aBlock
	"Answer the registry <FormDefinitionRegistry> for the definition named <aString> if none then evaluate <aBlock>"

	^definitionRegistry at: aString ifAbsent: aBlock! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
removeDefinition: anOrbeonFormDefinition
	"The receiver remove the form definition <anOrbeonFormDefinition> from the registry"

	(self hasRegistryFor: anOrbeonFormDefinition) ifFalse: [^anOrbeonFormDefinition].

	^(self registryFor: anOrbeonFormDefinition) removeDefinition: anOrbeonFormDefinition! !

!OrbeonApplication methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 14:52'!
removeInstanceDefinition: anOrbeonFormInstanceDefinition
	"The receiver remove the form definition <anOrbeonFormInstanceDefinition> from the registry"

	(self hasRegistryFor: anOrbeonFormInstanceDefinition) ifFalse: [^anOrbeonFormInstanceDefinition].

	^(self registryFor: anOrbeonFormInstanceDefinition) removeInstanceDefinition: anOrbeonFormInstanceDefinition! !

!OrbeonApplication methodsFor: 'published forms' stamp: 'BrunoBB 12/09/2016 09:35'!
replaceCurrentDefinition: currentDefinition with: newDefinition

	^(self registryForFormNamed: currentDefinition formName ifNone: [^nil]) replaceCurrentDefinition: currentDefinition with: newDefinition! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 04:40'!
reportCreationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate.

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			distributionMethod: #createdTime;
			yourself! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/18/2016 11:47'!
reportCreationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesCreationDistributionByMonthFrom: startDate to: endDate.

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			distributionMethod: #createdTime;
			yourself! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/18/2016 11:43'!
reportCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := self getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock.

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			conditionStringBlock: formConditionBlock blockSource;
			distributionMethod: #createdTime;
			yourself! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:59'!
reportModificationDayDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate.

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			distributionMethod: #modifiedTime;
			yourself! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/18/2016 11:47'!
reportModificationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := self getInstancesModificationDistributionByMonthFrom: startDate to: endDate.

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			distributionMethod: #modifiedTime;
			yourself! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 07:00'!
reportModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| secondsToRun distribution |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock].

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			conditionStringBlock: formConditionBlock blockSource;
			distributionMethod: #modifiedTime;
			sumSecondsToConstruct: secondsToRun;
			yourself! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/18/2016 11:31'!
reportQuantityDistributionFrom: startDate to: endDate
	| secondsToRun distribution |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesDistributionQuantityFrom: startDate to: endDate].

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			sumSecondsToConstruct: secondsToRun;
			yourself! !

!OrbeonApplication methodsFor: 'distribution' stamp: 'BrunoBB 10/18/2016 11:55'!
reportQuantityDistributionFrom: startDate to: endDate satisfying: formConditionBlock
	| secondsToRun distribution |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock].

	^(OrbeonApplicationDistributionReport applicationName: applicationName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			sumSecondsToConstruct: secondsToRun;
			yourself! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/30/2016 15:24'!
setRegistryFormNamed: formName with: aFormDefinitionRegistry
	"The receiver set/put at <formName> the registry <aFormDefinitionRegistry>"

	^definitionRegistry 
		at: formName 
		put: aFormDefinitionRegistry! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/27/2016 14:52'!
unregisterDefinition: anOrbeonFormDefinition
	"Unregister from registry <FormDefinitionRegistry> the definition <anOrbeonFormDefinition> if absent answer nil"

	^self unregisterDefinitionNamed: anOrbeonFormDefinition name! !

!OrbeonApplication methodsFor: 'registry operations' stamp: 'BrunoBB 04/27/2016 14:52'!
unregisterDefinitionNamed: aString
	"Unregister from registry <FormDefinitionRegistry> the definition named <aString> if absent answer nil"

	^definitionRegistry removeKey: aString ifAbsent: [nil]! !

!OrbeonWebLocalLogin methodsFor: '*orbeon' stamp: 'BrunoBB 07/15/2016 10:23'!
validateUser: username password: password

	^OrbeonUser cache anySatisfy: [:each | each name = username and:[each password = password]]! !

Exception subclass: #OrbeonException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #FormIdNotExistException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #FormRegistryNotExistException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #GridValueNotFoundException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #NoOrbeonAppException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!NoOrbeonAppException commentStamp: 'BrunoBB 10/26/2014 16:50' prior: 0!
When OrbeonFormProcessDefinition has NO OrbeonApplication this exception arise.!

OrbeonException subclass: #NoOrbeonFormException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #NoSqlDataRetrievedException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #NoSuchObjectInCacheException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #OrbeonControlSizeException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #OrbeonDateMaskException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #OrbeonDefinitionNotFoundException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #OrbeonInvalidXMLException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonException subclass: #OrbeonTimeStampMaskException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!TrueClass methodsFor: '*orbeon' stamp: 'BrunoBB 04/09/2016 13:40'!
printOrbeonString

	^'true'! !

AppLocalDistributionReport subclass: #OrbeonApplicationDistributionReport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

AppLocalDistributionReport subclass: #OrbeonFieldDistributionReport
	instanceVariableNames: 'formName version fieldName startValue endValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonFieldDistributionReport class methodsFor: 'other' stamp: 'BrunoBB 10/29/2016 14:17'!
applicationName: applicationName formName: formName version: version fieldName: fieldName distribution: aDictionary

	^(super distribution: aDictionary)
		applicationName: applicationName;
		formName: formName;
		version: version;
		fieldName: fieldName;
		yourself! !

!OrbeonFieldDistributionReport class methodsFor: 'other' stamp: 'BrunoBB 10/29/2016 14:17'!
fieldName: fieldName distribution: aDictionary

	^(super distribution: aDictionary)
		fieldName: fieldName;
		yourself! !

!OrbeonFieldDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/29/2016 14:17'!
endValue
	^endValue! !

!OrbeonFieldDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/29/2016 14:17'!
endValue: newValue
	endValue := newValue! !

!OrbeonFieldDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/29/2016 14:17'!
fieldName
	^fieldName! !

!OrbeonFieldDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/29/2016 14:17'!
fieldName: newValue
	fieldName := newValue! !

!OrbeonFieldDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/29/2016 14:17'!
formName
	^formName! !

!OrbeonFieldDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/29/2016 14:17'!
formName: newValue
	formName := newValue! !

!OrbeonFieldDistributionReport methodsFor: 'other' stamp: 'BrunoBB 10/29/2016 14:17'!
isAllVersions
	"This report is about all versions published versions ?"

	^version isNil! !

!OrbeonFieldDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/29/2016 14:17'!
startValue
	^startValue! !

!OrbeonFieldDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/29/2016 14:17'!
startValue: newValue
	startValue := newValue! !

!OrbeonFieldDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/29/2016 14:17'!
version
	^version! !

!OrbeonFieldDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/29/2016 14:17'!
version: newValue
	version := newValue! !

OrbeonFieldDistributionReport subclass: #OrbeonRecursiveFieldDistributionReport
	instanceVariableNames: 'subDistributionMethods subDistributions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonRecursiveFieldDistributionReport methodsFor: 'other' stamp: 'BrunoBB 10/25/2016 08:24'!
addSubDistribution: aDictionary

	^subDistributions add: aDictionary! !

!OrbeonRecursiveFieldDistributionReport methodsFor: 'other' stamp: 'BrunoBB 10/25/2016 08:43'!
getFirstSubDistribution

	^subDistributions first! !

!OrbeonRecursiveFieldDistributionReport methodsFor: 'other' stamp: 'BrunoBB 10/25/2016 08:23'!
initialize

	super initialize.

	subDistributionMethods := OrderedCollection new.
	subDistributions := OrderedCollection new.! !

!OrbeonRecursiveFieldDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/25/2016 08:23'!
subDistributionMethods
	^subDistributionMethods! !

!OrbeonRecursiveFieldDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/25/2016 08:23'!
subDistributionMethods: newValue
	subDistributionMethods := newValue! !

!OrbeonRecursiveFieldDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/25/2016 08:23'!
subDistributions
	^subDistributions! !

!OrbeonRecursiveFieldDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/25/2016 08:23'!
subDistributions: newValue
	subDistributions := newValue! !

AppLocalDistributionReport subclass: #OrbeonFormDistributionReport
	instanceVariableNames: 'formName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonFormDistributionReport subclass: #OrbeonFormDefRegistryDistributionReport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonFormDefRegistryDistributionReport class methodsFor: 'other' stamp: 'BrunoBB 10/17/2016 05:06'!
applicationName: applicationName formName: formName distribution: aDictionary

	^(super distribution: aDictionary)
		applicationName: applicationName;
		formName: formName;
		yourself! !

!OrbeonFormDistributionReport class methodsFor: 'other' stamp: 'BrunoBB 10/29/2016 14:18'!
applicationName: applicationName formName: formName distribution: aDictionary

	^(super distribution: aDictionary)
		applicationName: applicationName;
		formName: formName;
		yourself! !

!OrbeonFormDistributionReport class methodsFor: 'other' stamp: 'BrunoBB 10/29/2016 14:18'!
applicationName: applicationName formName: formName version: version distribution: aDictionary

	^(super distribution: aDictionary)
		applicationName: applicationName;
		formName: formName;
		version: version;
		yourself! !

!OrbeonFormDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/29/2016 14:18'!
applicationName
	^applicationName! !

!OrbeonFormDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/29/2016 14:18'!
applicationName: newValue
	applicationName := newValue! !

!OrbeonFormDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/29/2016 14:18'!
formName
	^formName! !

!OrbeonFormDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/29/2016 14:18'!
formName: newValue
	formName := newValue! !

OrbeonFormDistributionReport subclass: #OrbeonPublishedFormDistributionReport
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonPublishedFormDistributionReport class methodsFor: 'other' stamp: 'BrunoBB 10/17/2016 05:05'!
applicationName: applicationName formName: formName version: version distribution: aDictionary

	^(super distribution: aDictionary)
		applicationName: applicationName;
		formName: formName;
		version: version;
		yourself! !

!OrbeonPublishedFormDistributionReport methodsFor: 'Accessing' stamp: 'BrunoBB 10/17/2016 05:04'!
version
	^version! !

!OrbeonPublishedFormDistributionReport methodsFor: 'Updating' stamp: 'BrunoBB 10/17/2016 05:04'!
version: newValue
	version := newValue! !

!TimeStamp class methodsFor: '*orbeon' stamp: 'BrunoBB 04/07/2016 10:34'!
fromOrbeonString: aString
	"Answer an instance of the receiver based on time stamp string <aString>.
	The supported TimeStamp format is: Date + Time. Where Date format is defined in Date fromOrbeonString:.
	Time part support p.m. or pm"

	^self fromOrbeonString: aString 
			dateFormat: self getOrbeonWebServerConfiguration dateFormat asUppercase! !

!TimeStamp class methodsFor: '*orbeon' stamp: 'BrunoBB 04/07/2016 11:37'!
fromOrbeonString: aString dateFormat: stringFormat
	"Answer an instance of the receiver based on time stamp string <aString>.
	The supported TimeStamp format is: Date + Time. Where Date format is defined in Date fromOrbeonString:.
	Time part support p.m. or pm"
	| timeStampArray dateString timeString |

	aString isEmpty ifTrue: [^nil].
	timeStampArray := ((aString findString: 'T') = 0) ifTrue: [aString subStrings: ' '] ifFalse: [aString subStrings: 'T'].

	((timeStampArray size = 2) or: [timeStampArray size = 3]) 
	ifFalse: [OrbeonTimeStampMaskException signal: 'Incorrect time stamp format'].
	"size = 3 --> must includes pm or p.m. or am or a.m."
	(timeStampArray size = 3 and: [(timeStampArray third asUppercase findString: 'M') = 0])
	ifTrue:  [OrbeonTimeStampMaskException signal: 'Incorrect time stamp format'].

	dateString := timeStampArray first.
	timeString := timeStampArray second.
	(timeStampArray size = 3) ifTrue: [timeString := timeString, timeStampArray third].

	^self date: (Date fromOrbeonString: dateString format: stringFormat) time: (Time fromString: timeString)! !

!TimeStamp methodsFor: '*orbeon' stamp: 'BrunoBB 10/15/2015 14:24'!
asOrbeonDateTimeString
	"yyyy-mm-ddThh:mm:ss+zzzz"
	| string offsetHours offsetMinutes |
	string := self asStringUsingFormat: #(3 2 1 $- 1 1 $: true true false).
	string at: 11 put: $T.
	string add: $..
	offset := offset abs // 60.
	offsetHours := offset // 60.
	offsetMinutes := offset \\ 60.
	offsetHours < 10 ifTrue: [string add: $0].
	string addAll: offsetHours printString.
	offsetMinutes < 10 ifTrue: [string add: $0].
	string addAll: offsetMinutes printString.
	^string.! !

!TimeStamp methodsFor: '*orbeon' stamp: 'BrunoBB 07/02/2014 18:16'!
printMySqlString

	^self date year printString, '-', 
		self date monthIndex printString, '-', 
		self date day printString, ' ', 
		self time printString! !

AppGlobalDistributionReport subclass: #OrbeonGlobalDistributionReport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!UndefinedObject methodsFor: '*orbeon' stamp: 'BrunoBB 07/02/2014 18:16'!
printMySqlString

	^'nil'! !

!Date class methodsFor: '*orbeon' stamp: 'BrunoBB 04/09/2016 13:38'!
fromOrbeonString: aString
	"Create an instance of the receiver from Orbeon formated string date <aString>.
	The Orbeon <oxf.xforms.format.input.date> configuration in XML file properties-loca.xml MUST be equal to: 
	self getOrbeonWebServerConfiguration dateFormat"

	^self fromOrbeonString: aString 
			format: self getOrbeonWebServerConfiguration dateFormat! !

!Date class methodsFor: '*orbeon' stamp: 'BrunoBB 04/22/2016 11:13'!
fromOrbeonString: aString format: stringFormat
	"Create an instance of the receiver from Orbeon formated string date <aString>.
	The Orbeon <oxf.xforms.format.input.date> configuration in XML file properties-loca.xml MUST be equal to: 
	self getOrbeonWebServerConfiguration dateFormat"
	| dateFormat dateSeparator dateArray dictIndex sortedIndex dayValue monthValue yearValue pivotKey convertedString |

	convertedString := (aString includes: $:) ifFalse: [aString] ifTrue: [aString allButLast: 6]. "issue #198 (timezone included in date)"
	dateFormat := stringFormat asUppercase.
	dateSeparator := ((dateFormat findString: '/') = 0)
		ifFalse: ['/']
		ifTrue: [((dateFormat findString: '-') = 0)
				ifFalse:['-'] 
				ifTrue: [((dateFormat findString: '.') = 0)
						ifFalse: ['.']
						ifTrue: [OrbeonDateMaskException signal: 'Not supported mask date separator exeception. It must be (/) or (-) or (.)']]].
	dateArray := convertedString subStrings: dateSeparator.
	(((convertedString findString: dateSeparator) = 0) or:[dateArray size ~= 3]) ifTrue: [OrbeonDateMaskException signal: 'Incorrect date format'].
	dictIndex := Dictionary new.
	dictIndex at: 'd' put: (dateFormat findString: 'D'). 
	dictIndex at: 'm' put: (dateFormat findString: 'M').
	dictIndex at: 'y' put: (dateFormat findString: 'Y').
	(dictIndex anySatisfy: [:each | each = 0]) ifTrue: [OrbeonDateMaskException signal: 'Bad date format configuration'].

	sortedIndex := dictIndex asSortedCollection.
	
	pivotKey := dictIndex keyAtValue: (sortedIndex  at: 1).
	dictIndex at: pivotKey put: 1.
	pivotKey := dictIndex keyAtValue: (sortedIndex  at: 2).
	dictIndex at: pivotKey put: 2.
	pivotKey := dictIndex keyAtValue: (sortedIndex  at: 3).
	dictIndex at: pivotKey put: 3.
	dayValue := dateArray at: (dictIndex at: 'd').
	monthValue := dateArray at: (dictIndex at: 'm').
	yearValue := dateArray at: (dictIndex at: 'y').

	^self 
		year: (Integer fromString: yearValue)
		month: (Integer fromString: monthValue)
		day: (Integer fromString: dayValue)! !

!Date methodsFor: '*orbeon' stamp: 'BrunoBB 07/02/2014 18:14'!
printMySqlString

	^self year printString, '-', self monthIndex printString , '-', self dayOfMonth printString! !

!Date methodsFor: '*orbeon' stamp: 'BrunoBB 04/09/2016 13:38'!
printOrbeonString
	"Create an instance of the receiver from Orbeon formated string date <aString>.
	The Orbeon <oxf.xforms.format.input.date> configuration in XML file properties-loca.xml MUST be equal to: 
	self getOrbeonWebServerConfiguration dateFormat"

	^self printOrbeonStringFormat: self getOrbeonWebServerConfiguration dateFormat! !

!Date methodsFor: '*orbeon' stamp: 'BrunoBB 04/09/2016 13:36'!
printOrbeonStringFormat: stringFormat
	"Create an instance of the receiver from Orbeon formated string date <aString>.
	The Orbeon <oxf.xforms.format.input.date> configuration in XML file properties-loca.xml MUST be equal to: 
	self getOrbeonWebServerConfiguration dateFormat"
	| dateFormat dateSeparator dictIndex sortedIndex printStream |

	dateFormat := stringFormat asUppercase.
	dateSeparator := ((dateFormat findString: '/') = 0)
		ifFalse: ['/']
		ifTrue: [((dateFormat findString: '-') = 0)
				ifFalse:['-'] 
				ifTrue: [((dateFormat findString: '.') = 0)
						ifFalse: ['.']
						ifTrue: [OrbeonDateMaskException signal: 'Not supported mask date separator exeception. It must be (/) or (-) or (.)']]].

	dictIndex := Dictionary new.
	dictIndex at: 'day' put: (dateFormat findString: 'D'). 
	dictIndex at: 'month' put: (dateFormat findString: 'M').
	dictIndex at: 'year' put: (dateFormat findString: 'Y').
	(dictIndex anySatisfy: [:each | each = 0]) ifTrue: [OrbeonDateMaskException signal: 'Bad date format configuration'].

	sortedIndex := dictIndex asSortedCollection.

	printStream := PrintStream on: String new.

	printStream nextPutAll: (self perform: (dictIndex keyAtValue: (sortedIndex at: 1))) printString.
	printStream nextPutAll: dateSeparator.
	printStream nextPutAll: (self perform: (dictIndex keyAtValue: (sortedIndex at: 2))) printString.
	printStream nextPutAll: dateSeparator.
	printStream nextPutAll: (self perform: (dictIndex keyAtValue: (sortedIndex at: 3))) printString.
	
	^printStream contents! !

!String methodsFor: '*orbeon' stamp: 'BrunoBB 07/02/2014 18:15'!
printMySqlString

	^self! !

AppFieldValueHolder subclass: #OrbeonFieldValueHolder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

Object subclass: #FormComparator
	instanceVariableNames: 'formOne formTwo comparisionResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormComparator class methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:36'!
formOne: anOrbeonFormInstance

	^super new initialize
		formOne: anOrbeonFormInstance;
		yourself! !

!FormComparator class methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:36'!
formOne: anOrbeonFormInstance formTwo: formInstance

	^super new initialize
		formOne: anOrbeonFormInstance;
		formTwo: formInstance;
		yourself! !

!FormComparator methodsFor: 'Accessing' stamp: 'BrunoBB 04/18/2016 15:36'!
comparisionResult
	^comparisionResult! !

!FormComparator methodsFor: 'Updating' stamp: 'BrunoBB 04/18/2016 15:36'!
comparisionResult: newValue
	comparisionResult := newValue! !

!FormComparator methodsFor: 'Accessing' stamp: 'BrunoBB 04/18/2016 15:36'!
formOne

   "Return the value of the instance variable 'formOne'."
   ^formOne! !

!FormComparator methodsFor: 'Updating' stamp: 'BrunoBB 04/18/2016 15:36'!
formOne: newValue

   "Modify the value of the instance variable 'formOne'."
   formOne := newValue! !

!FormComparator methodsFor: 'Accessing' stamp: 'BrunoBB 04/18/2016 15:36'!
formTwo

   "Return the value of the instance variable 'formTwo'."
   ^formTwo! !

!FormComparator methodsFor: 'Updating' stamp: 'BrunoBB 04/18/2016 15:36'!
formTwo: newValue

   "Modify the value of the instance variable 'formTwo'."
   formTwo := newValue! !

FormComparator subclass: #FormDefinitionComparator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormDefinitionComparator commentStamp: '' prior: 0!
This class compare two OrbeonFormDefinition and answer a collection with the differences.

It is used when an Orbeon Definition in the database (MySQL) has been updated. 

If the comparison IS Empty then the forms are equal and there is NO need to create another version.

If the comparison is NOT Empty then the forms are NOT equal and we should create another version.

This is an automatic process.!

!FormDefinitionComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 16:15'!
compare
	"The receiver compare <formOne> and <formTwo> and answer a collection with the differences"
	| difference sectionsOne sectionsTwo |

	difference := OrderedCollection new.

	sectionsOne := formOne sectionDefinitions. 
	sectionsTwo := formTwo sectionDefinitions.

	sectionsOne do: [:each | | sectionTwo | "compare sections one by one in formOne and formTwo"
		sectionTwo := sectionsTwo at: each name 
			ifAbsent: [difference add: (SectionDoesNotExist form: formOne object: each). nil].
		"here we compare the tree structure between the two forms"
		sectionTwo ifNotNil: [self compareSectionOf: each with: sectionTwo on: difference]]. 

	sectionsTwo do: [:each | | sectionOne | "compare sections one by one in formOne and formTwo"
		sectionOne := sectionsOne at: each name 
			ifAbsent: [difference add: (SectionDoesNotExist form: formTwo object: each). nil].]. 

	comparisionResult := difference! !

!FormDefinitionComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 12:47'!
compareFieldTypesOf: aSectionDefinitionOne with: aSectionDefinitionTwo on: anOrderedCollection
	"The receiver compare the field type's of section <aSectionDefinitionOne> with the field type's of section <aSectionDefinitionTwo>"

	aSectionDefinitionOne getAllFieldDefinitions do: [:fieldDefinitionOne |
		(aSectionDefinitionTwo getAllFieldDefinitions detect: [:each | each name =  fieldDefinitionOne name] ifNone: [nil]) 
		ifNotNil: [:fieldDefinitionTwo | | typeOne typeTwo |
			typeOne := aSectionDefinitionOne classOf: fieldDefinitionOne name.
			typeTwo := aSectionDefinitionTwo classOf: fieldDefinitionTwo name.
			(typeOne == typeTwo)
			ifFalse: [
			anOrderedCollection add: 
				(FieldTypeNotEqual new 
					form: aSectionDefinitionOne formDefinition; 
					field: fieldDefinitionOne name; 
					typeOne: fieldDefinitionOne type; 
					typeTwo: fieldDefinitionTwo type; 
					yourself)].
		]
	].! !

!FormDefinitionComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 16:10'!
compareGridsOf: aSectionDefinitionOne with: aSectionDefinitionTwo on: anOrderedCollection
	"The receiver compare the Grid's of section <aSectionDefinitionOne> and section <aSectionDefinitionTwo> and store difference on <anOrderedCollection>"

	aSectionDefinitionOne gridDefinitions do: [:each | 
		aSectionDefinitionTwo 
			getGridDefinitionNamed: each name 
			ifNone: [anOrderedCollection 
							add: (FormGridDoesNotExist new 
										form: aSectionDefinitionOne formDefinition;
										grid: each;
										yourself)].
	].

	aSectionDefinitionTwo gridDefinitions do: [:each | 
		aSectionDefinitionOne 
			getGridDefinitionNamed: each name 
			ifNone: [anOrderedCollection 
							add: (FormGridDoesNotExist new 
										form: aSectionDefinitionTwo formDefinition;
										grid: each;
										yourself)].
	].! !

!FormDefinitionComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 13:42'!
compareNames: formOneNames with: formTwoNames on: anOrderedCollection
	"Compares the names contained in <formOneNames> with the names in <formTwoNames>.
	If a difference is found then store it in <anOrderedCollection> using the class <difClass> to create the difference object"

	(formOneNames difference: formTwoNames) do: [:each | anOrderedCollection add: (FieldDoesNotExist form: formTwo object: each)].

	(formTwoNames difference: formOneNames) do: [:each | anOrderedCollection add: (FieldDoesNotExist form: formOne object: each)].
	
	^anOrderedCollection! !

!FormDefinitionComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 16:04'!
compareSectionOf: aSectionDefinitionOne with: aSectionDefinitionTwo on: anOrderedCollection
	"The receiver recursively compare section <aSectionDefinitionOne> with section <aSectionDefinitionTwo> structures.
	The comparison also includes the fields of each section. Append differences to the argument <anOrderedCollection>"

	"comparing fields contained in sections <aSectionDefinitionOne> and <aSectionDefinitionTwo>"
	self compareNames: aSectionDefinitionOne getAllFieldNames with: aSectionDefinitionTwo getAllFieldNames on: anOrderedCollection.

	"comparing field types"
	self compareFieldTypesOf: aSectionDefinitionOne with: aSectionDefinitionTwo on: anOrderedCollection.

	"comparing section's grid"
	self compareGridsOf: aSectionDefinitionOne with: aSectionDefinitionTwo on: anOrderedCollection.

	"comparing the tree structure of the sections <aSectionDefinitionOne> and <aSectionDefinitionTwo>"
	"also comparing subsection in <subSectionTwo> not present in <aSectionDefinitionTwo>"
	aSectionDefinitionOne subSections do: [:each | | subSectionTwo |
		subSectionTwo := aSectionDefinitionTwo getSubSectionNamed: each name 
									ifNone: [anOrderedCollection add: (SectionDoesNotExist form: aSectionDefinitionTwo formDefinition object: each). nil].

		subSectionTwo ifNotNil: [self compareSectionOf: each with: subSectionTwo on: anOrderedCollection]]. "recursive CALL"

	"comparing subsection in <aSectionDefinitionTwo> not present in <aSectionDefinitionOne>"
	aSectionDefinitionTwo subSections do: [:each |
		aSectionDefinitionOne getSubSectionNamed: each name 
										ifNone: [anOrderedCollection add: (SectionDoesNotExist form: aSectionDefinitionTwo formDefinition object: each)]].

	^anOrderedCollection! !

!FormDefinitionComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:48'!
hasFieldTypeChanged
	"Answer whether or not at least on field definition has changed it's type"

	^comparisionResult anySatisfy: [:each | each isFieldTypeChanged]! !

!FormDefinitionComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:47'!
shouldCreateAnotherVersion
	"Answer wheter or not <formOne> and <formTwo> should be different versions"

	^comparisionResult notEmpty! !

FormComparator subclass: #FormInstanceComparator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 02/06/2015 14:14'!
buildSummaryWith: aDictionary
	"Compare the receiver <formOne> with the dictionary of fields <aDictionary>. Answer matching and mismatching fields values.
	Check each key of <aDictionary> with the field names of <formOne>. 
	If the key and field name match then compare the value of the <aDictionary> for that key with the field value of <formOne>"
	| comparision |

	comparision := OrderedCollection new.
	aDictionary keysAndValuesDo: [:key :value |
		(formOne fields includesKey: key) "<formOne> has this field named <key> ???"
		ifFalse: [comparision add: (FieldDoesNotExist form: formOne fieldName: key)] "field in <formOne> doed NOT exist"
		ifTrue: [(value = (formOne valueAt: key)) 
					ifFalse: [comparision 
									add: ((FieldValueNotEqual form: formOne fieldName: key) "field in <formOne> doed exist but NOT equal value"
											valueOne: value;
											valueTwo: (formOne valueAt: key);
											yourself).]
					ifTrue: [comparision 
									add: ((FieldValueEqual form: formOne fieldName: key) "field in <formOne> doed exist with equal value"
											valueOne: value;
											yourself)] ]].
	^comparision! !

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 16:15'!
compare
	"Compare the receiver <formOne> with <formTwo>. ONLY answer differences at field value level.
	Check each field name of <formOne> is included in <formTwo>. 
	If the field name exist then compare the value of the <formOne> field with <formTwo> field value"
	| difference |

	difference := OrderedCollection new.
	formOne fields keysAndValuesDo: [:key :value |
		(formTwo hasFieldNamed: key) "<formTwo> has the field name <key> of <formOne> ???"
		ifFalse: [difference add: (FieldDoesNotExist form: formTwo fieldName: key)] "field does NOT exist in <formTwo> "
		ifTrue: [(value = (formTwo valueAt: key)) 
					ifFalse: [difference 
									add: ((FieldValueNotEqual form: formTwo fieldName: key) "field exist but value is different"
											valueOne: value;
											valueTwo: (formTwo valueAt: key);
											yourself).
					] 	]  ].

	comparisionResult := difference! !

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 02/06/2015 14:14'!
compareWith: aDictionary
	"Compare the receiver <formOne> with the dictionary of fields <aDictionary>.ONLY answer mismatching fields values.
	Check field names of <formOne> with each key of <aDictionary>. 
	If the key and field name match then compare the value of the <aDictionary> for that key with the field value of <formOne>"
	| difference |

	difference := OrderedCollection new.
	formOne fields keysAndValuesDo: [:key :value |
		(aDictionary includesKey: key)
		ifFalse: [difference add: (FieldDoesNotExist form: nil fieldName: key)] "<aDictionary> does NOT include <formOne> field"
		ifTrue: [(value = (aDictionary at: key)) "<aDictionary> VALUE does is equal to <formOne> field value ???"
					ifFalse: [difference 
									add: ((FieldValueNotEqual form: nil fieldName: key) "value is different"
											valueOne: value;
											valueTwo: (aDictionary at: key);
											yourself).
					] 	]  ].
	^difference! !

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:48'!
showFieldDoesNotExist
	"The receiver answer a collection of difference of type <FieldDoesNotExist>"

	^comparisionResult select: [:each | each class == FieldDoesNotExist]! !

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 03/14/2015 11:49'!
showFieldDoesNotExistWith: aDictionary
	"The receiver answer a collection of difference with <aDictionary> of type <FieldDoesNotExist>"

	^(self compareWith: aDictionary) select: [:each | each class == FieldDoesNotExist]! !

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 02/06/2015 14:14'!
showSummaryWith: aDictionary
	| summary |

	summary := ''.

	(self buildSummaryWith: aDictionary) do: [:each | summary := summary, each printResume, String lf].

	^summary! !

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:49'!
showValueNotEqual

	^comparisionResult select: [:each | each class == FieldValueNotEqual]! !

!FormInstanceComparator methodsFor: 'other' stamp: 'BrunoBB 02/06/2015 14:14'!
showValueNotEqualWith: aDictionary

	^(self compareWith: aDictionary) select: [:each | each class == FieldValueNotEqual]! !

Object subclass: #FormDifference
	instanceVariableNames: 'form'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

FormDifference subclass: #FieldDifference
	instanceVariableNames: 'field'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FieldDifference class methodsFor: 'other' stamp: 'BrunoBB 09/27/2014 12:33'!
form: anOrbeonFormInstance

	^super new initialize
		form: anOrbeonFormInstance;
		yourself! !

!FieldDifference class methodsFor: 'other' stamp: 'BrunoBB 02/12/2015 03:21'!
form: anOrbeonForm fieldName: fieldName

	^super new initialize
		form: anOrbeonForm;
		field: fieldName;
		yourself! !

!FieldDifference class methodsFor: 'other' stamp: 'BrunoBB 02/12/2015 03:20'!
form: anOrbeonForm object: anObject

	^self form: anOrbeonForm fieldName: anObject! !

!FieldDifference methodsFor: 'Accessing' stamp: 'BrunoBB 09/27/2014 12:33'!
field

   "Return the value of the instance variable 'field'."
   ^field! !

!FieldDifference methodsFor: 'Updating' stamp: 'BrunoBB 09/27/2014 12:33'!
field: newValue

   "Modify the value of the instance variable 'field'."
   field := newValue! !

!FieldDifference methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 13:46'!
isEqual
	^false! !

!FieldDifference methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 13:46'!
isNotEqual
	^false! !

!FieldDifference methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 12:14'!
printResume

	^self subclassResponsibility! !

FieldDifference subclass: #FieldDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FieldDoesNotExist methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:38'!
isFieldDoesNotExist

	^true! !

!FieldDoesNotExist methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:03'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: '[', form formName, '.', field, ']'! !

!FieldDoesNotExist methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 12:15'!
printResume

	^field, ' does not exist'! !

FieldDifference subclass: #FieldTypeNotEqual
	instanceVariableNames: 'typeOne typeTwo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FieldTypeNotEqual class methodsFor: 'other' stamp: 'BrunoBB 02/09/2015 11:59'!
form: anOrbeonFormDefinition typeOne: typeOne typeTwo: typeTwo

	^super new initialize
		form: anOrbeonFormDefinition;
		typeOne: typeOne;
		typeTwo: typeTwo;
		yourself! !

!FieldTypeNotEqual methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:38'!
isFieldTypeChanged

	^true! !

!FieldTypeNotEqual methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:22'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: ' (' ,field, ').[', typeOne, ']-[', typeTwo, ']'! !

!FieldTypeNotEqual methodsFor: 'Accessing' stamp: 'BrunoBB 02/09/2015 11:47'!
typeOne

   "Return the value of the instance variable 'typeOne'."
   ^typeOne! !

!FieldTypeNotEqual methodsFor: 'Updating' stamp: 'BrunoBB 02/09/2015 11:47'!
typeOne: newValue

   "Modify the value of the instance variable 'typeOne'."
   typeOne := newValue! !

!FieldTypeNotEqual methodsFor: 'Accessing' stamp: 'BrunoBB 02/09/2015 11:47'!
typeTwo

   "Return the value of the instance variable 'typeTwo'."
   ^typeTwo! !

!FieldTypeNotEqual methodsFor: 'Updating' stamp: 'BrunoBB 02/09/2015 11:47'!
typeTwo: newValue

   "Modify the value of the instance variable 'typeTwo'."
   typeTwo := newValue! !

FieldDifference subclass: #FieldValueEqual
	instanceVariableNames: 'valueOne'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FieldValueEqual methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 13:46'!
isEqual
	^true! !

!FieldValueEqual methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:39'!
isFieldValueEqual

	^true! !

!FieldValueEqual methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 14:01'!
printResume

	^(form labelAt: field), String lf, valueOne, ', [OK]'! !

!FieldValueEqual methodsFor: 'Accessing' stamp: 'BrunoBB 09/28/2014 12:10'!
valueOne

   "Return the value of the instance variable 'valueOne'."
   ^valueOne! !

!FieldValueEqual methodsFor: 'Updating' stamp: 'BrunoBB 09/28/2014 12:10'!
valueOne: newValue

   "Modify the value of the instance variable 'valueOne'."
   valueOne := newValue! !

FieldDifference subclass: #FieldValueNotEqual
	instanceVariableNames: 'valueOne valueTwo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FieldValueNotEqual methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:39'!
isFieldValueNotEqual

	^true! !

!FieldValueNotEqual methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 13:46'!
isNotEqual
	^true! !

!FieldValueNotEqual methodsFor: 'other' stamp: 'BrunoBB 09/28/2014 13:29'!
printResume

	^field, String lf, valueOne, ' <> ' , valueTwo, ', [Error]'! !

!FieldValueNotEqual methodsFor: 'Accessing' stamp: 'BrunoBB 09/27/2014 12:43'!
valueOne

   "Return the value of the instance variable 'valueOne'."
   ^valueOne! !

!FieldValueNotEqual methodsFor: 'Updating' stamp: 'BrunoBB 09/27/2014 12:43'!
valueOne: newValue

   "Modify the value of the instance variable 'valueOne'."
   valueOne := newValue! !

!FieldValueNotEqual methodsFor: 'Accessing' stamp: 'BrunoBB 09/27/2014 12:43'!
valueTwo

   "Return the value of the instance variable 'valueTwo'."
   ^valueTwo! !

!FieldValueNotEqual methodsFor: 'Updating' stamp: 'BrunoBB 09/27/2014 12:43'!
valueTwo: newValue

   "Modify the value of the instance variable 'valueTwo'."
   valueTwo := newValue! !

!FormDifference class methodsFor: 'other' stamp: 'BrunoBB 02/12/2015 03:20'!
form: anOrbeonFormInstance object: anObject

	self subclassResponsibility! !

!FormDifference methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2015 05:54'!
form

   "Return the value of the instance variable 'form'."
   ^form! !

!FormDifference methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2015 05:54'!
form: newValue

   "Modify the value of the instance variable 'form'."
   form := newValue! !

!FormDifference methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:39'!
isFieldDoesNotExist

	^false! !

!FormDifference methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:38'!
isFieldTypeChanged

	^false! !

!FormDifference methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:39'!
isFieldValueEqual

	^false! !

!FormDifference methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:39'!
isFieldValueNotEqual

	^false! !

!FormDifference methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 16:00'!
isGridDoesNotExist

	^false! !

!FormDifference methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:50'!
isSectionDoesNotExist

	^false! !

FormDifference subclass: #FormGridDifference
	instanceVariableNames: 'grid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormGridDifference methodsFor: 'Accessing' stamp: 'BrunoBB 04/18/2016 16:11'!
grid
	^grid! !

!FormGridDifference methodsFor: 'Updating' stamp: 'BrunoBB 04/18/2016 16:11'!
grid: newValue
	grid := newValue! !

FormGridDifference subclass: #FormGridDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormGridDoesNotExist methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 16:00'!
isGridDoesNotExist

	^true! !

!FormGridDoesNotExist methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:03'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: ' [' ,form formName, '].[', grid name, ']'! !

FormDifference subclass: #FormSectionDifference
	instanceVariableNames: 'section'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormSectionDifference class methodsFor: 'other' stamp: 'BrunoBB 02/12/2015 03:21'!
form: anOrbeonForm object: anObject

	^self form: anOrbeonForm sectionName: anObject! !

!FormSectionDifference class methodsFor: 'other' stamp: 'BrunoBB 02/09/2015 11:46'!
form: anOrbeonFormDefinition sectionName: sectionName

	^super new initialize
		form: anOrbeonFormDefinition;
		section: sectionName;
		yourself! !

!FormSectionDifference methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 13:07'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: '[',section name, ']'! !

!FormSectionDifference methodsFor: 'Accessing' stamp: 'BrunoBB 02/09/2015 11:48'!
section

   "Return the value of the instance variable 'section'."
   ^section! !

!FormSectionDifference methodsFor: 'Updating' stamp: 'BrunoBB 02/09/2015 11:48'!
section: newValue

   "Modify the value of the instance variable 'section'."
   section := newValue! !

FormSectionDifference subclass: #SectionDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!SectionDoesNotExist methodsFor: 'other' stamp: 'BrunoBB 04/18/2016 15:50'!
isSectionDoesNotExist

	^true! !

FormSectionDifference subclass: #SectionStructureMismatch
	instanceVariableNames: 'structureDifferences'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!SectionStructureMismatch methodsFor: 'other' stamp: 'BrunoBB 02/07/2015 15:09'!
addMismatch: aSectionDoesNotExist

	^structureDifferences add: aSectionDoesNotExist! !

!SectionStructureMismatch methodsFor: 'other' stamp: 'BrunoBB 02/07/2015 15:08'!
initialize

	super initialize.

	structureDifferences := OrderedCollection new.! !

!SectionStructureMismatch methodsFor: 'Accessing' stamp: 'BrunoBB 02/07/2015 15:08'!
structureDifferences

   "Return the value of the instance variable 'structureDifferences'."
   ^structureDifferences! !

!SectionStructureMismatch methodsFor: 'Updating' stamp: 'BrunoBB 02/07/2015 15:08'!
structureDifferences: newValue

   "Modify the value of the instance variable 'structureDifferences'."
   structureDifferences := newValue! !

Object subclass: #FormExportProcess
	instanceVariableNames: 'definition formsCollection printArray lineNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormExportProcess class methodsFor: 'other' stamp: 'BrunoBB 04/14/2015 16:02'!
definition: anOrbeonFormDefinition formsCollection: anOrderedCollection

	^super new initialize
		definition: anOrbeonFormDefinition;
		formsCollection: anOrderedCollection;
		yourself! !

!FormExportProcess methodsFor: 'export' stamp: 'BrunoBB 04/14/2015 16:05'!
currentInfo
	| info |

	info := (printArray at: lineNumber ifAbsent: [^'']) at: 2 "info" ifAbsent: [''].

	info ifNil: [^''].

	^info! !

!FormExportProcess methodsFor: 'other' stamp: 'BrunoBB 04/14/2015 16:02'!
definition

   "Return the value of the instance variable 'definition'."
   ^definition! !

!FormExportProcess methodsFor: 'other' stamp: 'BrunoBB 04/14/2015 16:02'!
definition: newValue

   "Modify the value of the instance variable 'definition'."
   definition := newValue! !

!FormExportProcess methodsFor: 'other' stamp: 'BrunoBB 04/14/2015 16:02'!
formsCollection

   "Return the value of the instance variable 'formsCollection'."
   ^formsCollection! !

!FormExportProcess methodsFor: 'other' stamp: 'BrunoBB 04/14/2015 16:02'!
formsCollection: newValue

   "Modify the value of the instance variable 'formsCollection'."
   formsCollection := newValue! !

!FormExportProcess methodsFor: 'other' stamp: 'BrunoBB 07/29/2016 10:09'!
getLabelOf: fieldName
	| label |

	label := [((GRCodec forEncoding: 'utf-8') decode: (definition labelAt: fieldName)) asString] on: Error do: [:err | definition labelAt: fieldName].

	label := label copyWithoutAll: String lf.

	^label! !

!FormExportProcess methodsFor: 'other' stamp: 'BrunoBB 04/14/2015 16:04'!
initialize

	super initialize.

	printArray := Array new.
	lineNumber := 0.! !

!FormExportProcess methodsFor: 'Accessing' stamp: 'BrunoBB 04/14/2015 16:02'!
lineNumber
	^lineNumber! !

!FormExportProcess methodsFor: 'Updating' stamp: 'BrunoBB 04/14/2015 16:02'!
lineNumber: newValue
	lineNumber := newValue! !

!FormExportProcess methodsFor: 'other' stamp: 'BrunoBB 04/14/2015 16:02'!
numberOfFields
	"Answer the number of fields of the OrbeonFormDefinition <definition>.
	Plus 2 is used to add 2 extra columns, one for form <id> and another to insert extra data"

	^definition fieldDefinitions size + 2! !

!FormExportProcess methodsFor: 'Accessing' stamp: 'BrunoBB 04/14/2015 16:02'!
printArray
	^printArray! !

!FormExportProcess methodsFor: 'Updating' stamp: 'BrunoBB 04/14/2015 16:02'!
printArray: newValue
	printArray := newValue! !

!FormExportProcess methodsFor: 'export' stamp: 'BrunoBB 04/14/2015 16:05'!
printMatrix
	"Print the export structure to a PrintStream"
	| stream |

	stream := PrintStream on: String new.

	self printMatrixToStream: stream.

	^stream! !

!FormExportProcess methodsFor: 'export' stamp: 'BrunoBB 04/15/2015 05:46'!
printMatrixToStream: aStream
	"The receiver iterate over structure <printArray> and put it's content on <aStream>"

	printArray do: [:columnArray | "row iteration"
		columnArray do: [:columnValue |  "column iteration"
			columnValue 
			ifNotNil: [aStream nextPutAll: columnValue; nextPutAll: String tab] 
			ifNil: [aStream nextPutAll: String tab].
		].
		aStream nextPutAll: String lf.
	].

	^aStream! !

!FormExportProcess methodsFor: 'export' stamp: 'BrunoBB 04/14/2015 16:06'!
printMatrixToWriteStream
	| stream |

	stream := WriteStream on: String new.

	self printMatrixToStream: stream.

	^stream! !

!FormExportProcess methodsFor: 'export' stamp: 'BrunoBB 04/14/2015 16:06'!
printToFile: pathFile
	| stream |

	stream := FileStream write: pathFile.

	self printMatrixToStream: stream.

	stream flush; close.! !

FormExportProcess subclass: #RecursiveFormExport
	instanceVariableNames: 'fieldPositions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

RecursiveFormExport subclass: #MultilineRecursiveFormExport
	instanceVariableNames: 'columnNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!MultilineRecursiveFormExport methodsFor: 'Accessing' stamp: 'BrunoBB 05/10/2015 15:47'!
columnNumber

   "Return the value of the instance variable 'columnNumber'."
   ^columnNumber! !

!MultilineRecursiveFormExport methodsFor: 'Updating' stamp: 'BrunoBB 05/10/2015 15:47'!
columnNumber: newValue

   "Modify the value of the instance variable 'columnNumber'."
   columnNumber := newValue! !

!MultilineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 02/10/2016 10:58'!
generatePrintArray
	"The receiver creates anArray with values to export to a CSV file"

	fieldPositions at: 'id' put: 1. "the first column is the <id> of the form"
	fieldPositions at: 'info' put: 2. "the secon column is for extra information"

	definition getFieldsNames keysAndValuesDo: [:index :fieldName | "from 2nd column we put the name of the fields"
		fieldPositions at: fieldName put: (index + 2). 
		printArray at: index put: Array new. "<printArray> is an Array of Arrays."].

	"at: 1 put the names of the fields"
	printArray at: 1 put: (fieldPositions asSortedArray collect: [:index | fieldPositions keyAtValue: index ifAbsent: [nil]]).
	"at: 2 put the labels of the fields"

	printArray at: 2 put: Array new. "in the row below of field names we set the fields label"
	(printArray at: 1) keysAndValuesDo: [:index :fieldName | | label |
		label := (definition labelAt: fieldName) copyWithoutAll: String lf.
		(printArray at: 2) at: index put: label].

	lineNumber := 3. "the actual data is put from row 3 ..."

	formsCollection do: [:each | self printInstance: each.]! !

!MultilineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 05/10/2015 15:50'!
initialize

	super initialize.

	columnNumber := 0.! !

!MultilineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 05/10/2015 15:51'!
insert: fieldName value: fieldValue
	"The receiver insert in Array <valueArray> at <column> the value <fieldValue>. 
	<column> is the row index of the field <fieldName>
	<valueArray>  is <printArray> at <lineNumber>"
	| column valueArray |

	column := fieldPositions at: fieldName. "get the column of <fieldName>"
	valueArray := printArray at: lineNumber. "get the current row"
	valueArray at: column put: fieldValue. "insert the value"! !

!MultilineRecursiveFormExport methodsFor: 'export' stamp: 'BrunoBB 05/10/2015 15:47'!
printFormSectionInstance: aFormSectionInstance
	"The receiver map the section instance <aFormSectionInstance> to <printArray>"

	aFormSectionInstance isRepeatable  "if repeatable then print all iterations in <printArray>"
	ifTrue: [^aFormSectionInstance iterationsDo: [:each | self printFormSectionInstance: each]].

	"initialize all Row Array"
	1 to: self numberOfFields do: [:each | printArray at: lineNumber put: (Array new: self numberOfFields)].

	(printArray at: lineNumber) at: 1 "id" put: aFormSectionInstance id.

	self printInstancesFieldsOf: aFormSectionInstance. "print top fields of <aFormSectionInstance>"

	self printInstancesGridsOf: aFormSectionInstance. "print top grids of <aFormSectionInstance>"

	"print subsections of <aFormSectionInstance>"
	^aFormSectionInstance getSectionsToExport do: [:each | self printFormSectionInstance: each].! !

!MultilineRecursiveFormExport methodsFor: 'export' stamp: 'BrunoBB 05/10/2015 15:47'!
printInfoForFormFieldsOf: aFormSectionInstance

	(printArray at: lineNumber) at: 2 "info" put: self currentInfo, '-', aFormSectionInstance name, '-', aFormSectionInstance printExportType.! !

!MultilineRecursiveFormExport methodsFor: 'export' stamp: 'BrunoBB 05/10/2015 15:47'!
printInfoForGrid: aFormGridInstance

	(printArray at: lineNumber) at: 2 "info" put: self currentInfo, '-', aFormGridInstance sectionName, '-',aFormGridInstance name.! !

!MultilineRecursiveFormExport methodsFor: 'export' stamp: 'BrunoBB 05/10/2015 15:47'!
printInstance: anOrbeonFormInstance

	anOrbeonFormInstance getSectionsToExport do: [:each | self printFormSectionInstance: each].! !

!MultilineRecursiveFormExport methodsFor: 'export' stamp: 'BrunoBB 05/10/2015 15:47'!
printInstancesFieldsOf: aFormSectionInstance
	"The receiver print the top fields of <aFormSectionInstance> in <printArray>"

	self printInfoForFormFieldsOf: aFormSectionInstance. "put info of the section <aFormSectionInstance> in the second column"
	
	aFormSectionInstance fields keysAndValuesDo: [:fieldName :fieldValue |
		
		self insert: fieldName value: fieldValue. "for each field insert in <printArray> field name and value"
	].

	(aFormSectionInstance fields notEmpty) ifTrue: [lineNumber := lineNumber + 1.]. "new row if the fields where not empty"! !

!MultilineRecursiveFormExport methodsFor: 'export' stamp: 'BrunoBB 05/10/2015 15:47'!
printInstancesGridsOf: aFormSectionInstance
	"The receiver print the top grids of <aFormSectionInstance> in <printArray>"

	aFormSectionInstance grids do: [:eachGrid | 
		eachGrid gridIterations do: [:eachDictionary | "initialize array for each grid iteration"
			1 to: self numberOfFields do: [:each | printArray at: lineNumber put: (Array new: self numberOfFields)].

			self printInfoForGrid: eachGrid. "print in the 2 column info about the grid"

			(printArray at: lineNumber) at: 1 "id" put: aFormSectionInstance id.

			eachDictionary keysAndValuesDo: [:fieldName :fieldValue |
				
				self insert: fieldName value: fieldValue]. "for each field insert in <printArray> field name and value"

			lineNumber := lineNumber + 1.
		]
	].! !

!RecursiveFormExport methodsFor: 'Accessing' stamp: 'BrunoBB 05/10/2015 15:51'!
fieldPositions
	^fieldPositions! !

!RecursiveFormExport methodsFor: 'Updating' stamp: 'BrunoBB 05/10/2015 15:51'!
fieldPositions: newValue
	fieldPositions := newValue! !

!RecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 05/10/2015 15:50'!
initialize

	super initialize.

	fieldPositions := Dictionary new.! !

RecursiveFormExport subclass: #SinglelineRecursiveFormExport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!SinglelineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 07/29/2016 10:38'!
generatePrintArray
	"The receiver creates anArray with values to export to a CSV file"
	| fieldSpace previousColumunShift firsElement |

	fieldSpace := self getFieldsNameSpace2. "aDictionary fieldName-->columunShift"
	lineNumber := 3.
	previousColumunShift := 2.
	printArray at: 1 put: (Array new: 500).
	printArray at: 2 put: (Array new: 500).
	(printArray at: 1) at: 1 put: 'ID'.
	(printArray at: 2) at: 1 put: 'ID'.
	printArray at: 3 put: Array new.
	firsElement := formsCollection isEmpty ifTrue: [nil] ifFalse: [formsCollection first].

	(fieldSpace orderedKeys reject: [:each | each isNil]) do: [:fieldName | | label columunShift |
		columunShift := fieldSpace at: fieldName.
		(printArray at: 1) at: previousColumunShift put: fieldName.
		label := self getLabelOf: fieldName.
		(printArray at: 2) at: previousColumunShift put: label.
		previousColumunShift := previousColumunShift + columunShift
	].

	formsCollection do: [:eachForm | | newArray columnNumber |
		newArray := Array new: 501.
		newArray at: 1 put: eachForm id.
		columnNumber := 1.
		(fieldSpace orderedKeys reject: [:each | each isNil]) do: [:fieldName | | valueHolder |
			valueHolder := eachForm holderValueAt: fieldName.
			newArray addAll: (valueHolder currentValues collect: [:e | e originalValue copyWithoutAll: String lf])
							afterIndex: columnNumber.
			columnNumber := columnNumber + (fieldSpace at: fieldName).
		].
		printArray at: lineNumber put: newArray.	
		lineNumber := 	lineNumber + 1.	
	].! !

!SinglelineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 10:11'!
generatePrintArray_new
	"The receiver creates anArray with values to export to a CSV file"
	| fieldSpace previousColumunShift firsElement |

	fieldSpace := self getFieldsNameSpace. "aDictionary fieldName-->columunShift"
	lineNumber := 3.
	previousColumunShift := 1.
	printArray at: 1 put: (Array new: 500).
	printArray at: 2 put: (Array new: 500).
	(printArray at: 2) at: 1 put: 'ID'.
	printArray at: 3 put: Array new.
	firsElement := formsCollection isEmpty ifTrue: [nil] ifFalse: [formsCollection first].
	fieldSpace keysAndValuesDo: [:fieldName :columunShift | | label |
		(printArray at: 1) at: (previousColumunShift + 1) put: fieldName.
		label := self getLabelOf: fieldName.
		(printArray at: 2) at: (previousColumunShift + 1) put: label.
		previousColumunShift := previousColumunShift + columunShift].

	formsCollection do: [:eachForm | | newArray columnNumber |
		newArray := Array new: 501.
		columnNumber := 1.
		newArray at: 1 put: eachForm id.
		eachForm getAllFields keysAndValuesDo: [:fieldName :valueCollection | 
			newArray addAll: (valueCollection collect: [:e | e copyWithoutAll: String lf]) afterIndex: columnNumber.
			columnNumber := columnNumber + (fieldSpace at: fieldName).
		].
		printArray at: lineNumber put: newArray.	
		lineNumber := 	lineNumber + 1.	
	].! !

!SinglelineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 10:48'!
generatePrintArray_original
	"The receiver creates anArray with values to export to a CSV file"
	| fieldSpace previousColumunShift firsElement |

	fieldSpace := self getFieldsNameSpace. "aDictionary fieldName-->columunShift"
	lineNumber := 3.
	previousColumunShift := 1.
	printArray at: 1 put: (Array new: 500).
	printArray at: 2 put: (Array new: 500).
	printArray at: 3 put: Array new.
	firsElement := formsCollection isEmpty ifTrue: [nil] ifFalse: [formsCollection first].
	fieldSpace keysAndValuesDo: [:fieldName :columunShift | | label |
		(printArray at: 1) at: previousColumunShift put: fieldName.
		label := self getLabelOf: fieldName.
		(printArray at: 2) at: previousColumunShift put: label.
		previousColumunShift := previousColumunShift + columunShift].

	formsCollection do: [:eachForm | | newArray columnNumber |
		newArray := Array new: 500.
		columnNumber := 0.
		eachForm getAllFields keysAndValuesDo: [:fieldName :valueCollection | 
			newArray addAll: (valueCollection collect: [:e | e copyWithoutAll: String lf]) afterIndex: columnNumber.
			columnNumber := columnNumber + (fieldSpace at: fieldName).
		].
		printArray at: lineNumber put: newArray.	
		lineNumber := 	lineNumber + 1.	
	].! !

!SinglelineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 05/11/2015 05:22'!
getFieldsNameSpace
	| fieldSpace |

	fieldSpace := Dictionary new.
	formsCollection do: [:eachForm |
		eachForm getAllFields keysAndValuesDo: [:fieldName :valueCollection |
			(fieldSpace includesKey: fieldName)
			ifTrue: [fieldSpace at: fieldName put: ((fieldSpace at: fieldName) max: valueCollection size).]
			ifFalse: [fieldSpace at: fieldName put: valueCollection size].		
		].
	].

	^fieldSpace! !

!SinglelineRecursiveFormExport methodsFor: 'other' stamp: 'BrunoBB 07/29/2016 10:15'!
getFieldsNameSpace2
	"Answer a dictionary where the key is the field's name and the value is an integer that is the greatest size of the repeatable values.
	This is to reserve room for all fields values (including repeatable) in the Matrix"
	| fieldSpace |

	fieldSpace := OrderPreservingDictionary new.
	formsCollection do: [:eachForm |
		eachForm fieldDefinitionsByTabIndex do:[:fieldDef | | valueHolder |
			valueHolder := eachForm holderValueAt: fieldDef name.
			(fieldSpace includesKey: fieldDef name)
			ifTrue: [fieldSpace at: fieldDef name put: ((fieldSpace at: fieldDef name) max: valueHolder currentSize).]
			ifFalse: [fieldSpace at: fieldDef name put: valueHolder currentSize].		
		].
	].

	^fieldSpace! !

FormExportProcess subclass: #SinglelineSimpleFormExport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!SinglelineSimpleFormExport methodsFor: 'other' stamp: 'BrunoBB 04/25/2016 11:35'!
generatePrintArray

	printArray at: 1 put: (Array with: 'ID').
	printArray at: 2 put: (Array with: 'ID').

	definition getFieldsNames keysAndValuesDo: [:index :fieldName |  
		(printArray at: 1) at: index + 1 put: fieldName. 
		(printArray at: 2) at: index + 1 put: (self getLabelOf: fieldName)].

	lineNumber := 3. "the actual data is put from row 3 ..."

	formsCollection do: [:each |
		printArray at: lineNumber put: (Array with: each id).
		self printInstance: each.
		lineNumber := lineNumber + 1].! !

!SinglelineSimpleFormExport methodsFor: 'other' stamp: 'BrunoBB 04/25/2016 11:37'!
printInstance: anOrbeonFormInstance

	definition getFieldsNames keysAndValuesDo: [:index :fieldName |  | fieldValue |
		fieldValue := ((GRCodec forEncoding: 'utf-8') decode: (anOrbeonFormInstance valueAt: fieldName)) asString copyWithoutAll: String lf.
		(printArray at: lineNumber) at: index + 1 put: fieldValue].! !

Object subclass: #FormImportReport
	instanceVariableNames: 'app form numberOfForms warnings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormImportReport commentStamp: '' prior: 0!
This class used after importing Forms from Orbeon (Jave) application into in GemStone.

Instances of this class has detailed information of the import process.

Is a NON persistent class.!

!FormImportReport class methodsFor: 'other' stamp: 'BrunoBB 03/04/2015 08:11'!
app: appName form: formName

	^super new initialize
		app: appName;
		form: formName;
		yourself! !

!FormImportReport methodsFor: 'other' stamp: 'BrunoBB 03/04/2015 13:37'!
addWarning: aFormImportWarning

	aFormImportWarning report: self.

	^warnings add: aFormImportWarning! !

!FormImportReport methodsFor: 'Accessing' stamp: 'BrunoBB 03/04/2015 08:13'!
app

   "Return the value of the instance variable 'app'."
   ^app! !

!FormImportReport methodsFor: 'Updating' stamp: 'BrunoBB 03/04/2015 08:13'!
app: newValue

   "Modify the value of the instance variable 'app'."
   app := newValue! !

!FormImportReport methodsFor: 'Accessing' stamp: 'BrunoBB 03/04/2015 08:13'!
form

   "Return the value of the instance variable 'form'."
   ^form! !

!FormImportReport methodsFor: 'Updating' stamp: 'BrunoBB 03/04/2015 08:13'!
form: newValue

   "Modify the value of the instance variable 'form'."
   form := newValue! !

!FormImportReport methodsFor: 'other' stamp: 'BrunoBB 03/04/2015 09:16'!
hasWarnings

	^warnings notEmpty! !

!FormImportReport methodsFor: 'other' stamp: 'BrunoBB 03/04/2015 08:00'!
initialize

	super initialize.

	app := ''.
	form := ''.
	numberOfForms := 0.
	warnings := OrderedCollection new.! !

!FormImportReport methodsFor: 'Accessing' stamp: 'BrunoBB 03/04/2015 08:13'!
numberOfForms

   "Return the value of the instance variable 'numberOfForms'."
   ^numberOfForms! !

!FormImportReport methodsFor: 'Updating' stamp: 'BrunoBB 03/04/2015 08:13'!
numberOfForms: newValue

   "Modify the value of the instance variable 'numberOfForms'."
   numberOfForms := newValue! !

!FormImportReport methodsFor: 'Accessing' stamp: 'BrunoBB 03/04/2015 08:13'!
warnings

   "Return the value of the instance variable 'warnings'."
   ^warnings! !

!FormImportReport methodsFor: 'Updating' stamp: 'BrunoBB 03/04/2015 08:13'!
warnings: newValue

   "Modify the value of the instance variable 'warnings'."
   warnings := newValue! !

Object subclass: #FormImportWarning
	instanceVariableNames: 'report field warning'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormImportWarning class methodsFor: 'other' stamp: 'BrunoBB 03/04/2015 13:38'!
report: aFormImportReport fieldName: fieldName

	^super new initialize
		report: aFormImportReport;
		field: fieldName;
		yourself! !

!FormImportWarning methodsFor: 'Accessing' stamp: 'BrunoBB 03/04/2015 13:36'!
field

   "Return the value of the instance variable 'field'."
   ^field! !

!FormImportWarning methodsFor: 'Updating' stamp: 'BrunoBB 03/04/2015 13:36'!
field: newValue

   "Modify the value of the instance variable 'field'."
   field := newValue! !

!FormImportWarning methodsFor: 'Accessing' stamp: 'BrunoBB 03/04/2015 13:36'!
report

   "Return the value of the instance variable 'report'."
   ^report! !

!FormImportWarning methodsFor: 'Updating' stamp: 'BrunoBB 03/04/2015 13:36'!
report: newValue

   "Modify the value of the instance variable 'report'."
   report := newValue! !

!FormImportWarning methodsFor: 'Accessing' stamp: 'BrunoBB 03/04/2015 13:36'!
warning

   "Return the value of the instance variable 'warning'."
   ^warning! !

!FormImportWarning methodsFor: 'Updating' stamp: 'BrunoBB 03/04/2015 14:14'!
warningDetails: newValue

   "Modify the value of the instance variable 'warning'."
   warning := newValue! !

!Object class methodsFor: '*orbeon' stamp: 'BrunoBB 02/08/2016 11:11'!
getOrbeonApplicationCache
	"The receiver answer the cache of OrbeonApplication "

	^self getOrbeonApplicationClass cache! !

!Object class methodsFor: '*orbeon' stamp: 'BrunoBB 02/08/2016 11:09'!
getOrbeonApplicationClass
	"The receiver answer the class used as Orbeon applications holder"

	^OrbeonApplication! !

!Object class methodsFor: '*orbeon' stamp: 'BrunoBB 02/26/2016 13:46'!
getOrbeonCache

	^AbstractApplication getOrbeonCache! !

!Object methodsFor: '*orbeon' stamp: 'BrunoBB 02/08/2016 11:12'!
getOrbeonApplicationCache
	"The receiver answer the cache of OrbeonApplication "

	^self class getOrbeonApplicationCache! !

!Object methodsFor: '*orbeon' stamp: 'BrunoBB 02/08/2016 11:08'!
getOrbeonApplicationClass
	"The receiver answer the class used as Orbeon applications holder"

	^self class getOrbeonApplicationClass! !

!Object methodsFor: '*orbeon' stamp: 'BrunoBB 02/26/2016 13:47'!
getOrbeonCache

	^self class getOrbeonCache! !

Object subclass: #OrbeonDatabaseMigrator
	instanceVariableNames: 'application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonDatabaseMigrator class methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:07'!
forApplicationNamed: appName

	^super new initialize
		setApplicationNamed: appName;
		yourself! !

!OrbeonDatabaseMigrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 17:24'!
apiConfiguration

	^OrbeonApiConfiguration default! !

!OrbeonDatabaseMigrator methodsFor: 'Accessing' stamp: 'BrunoBB 10/14/2015 14:07'!
application
	^application! !

!OrbeonDatabaseMigrator methodsFor: 'Updating' stamp: 'BrunoBB 10/14/2015 14:07'!
application: newValue
	application := newValue! !

!OrbeonDatabaseMigrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:00'!
defaultDatabaseReaderClass

	^self subclassResponsibility! !

!OrbeonDatabaseMigrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:12'!
getOrbeonHandler

	^OrbeonHandler default! !

!OrbeonDatabaseMigrator methodsFor: 'other' stamp: 'BrunoBB 02/08/2016 12:49'!
setApplicationNamed: appName

	application := self getOrbeonApplicationClass 
							applicationNamed: appName 
							ifNone: [self getOrbeonApplicationClass new applicationName: appName; addToCache; yourself].! !

OrbeonDatabaseMigrator subclass: #OrbeonDB2Migrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonDB2Migrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:02'!
defaultDatabaseReaderClass

	^OrbeonDB2Reader! !

OrbeonDatabaseMigrator subclass: #OrbeonMySQLMigrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonMySQLMigrator commentStamp: 'BrunoBB 10/18/2015 11:44' prior: 0!
This class perform a migration from MySQL to GemStone/S.

- Migrate Form Published Definitions (saved but not published are not included)
- Migrate Form Instances (drafts are not included)

To Perform the Migration:
1- Publish all saved forms
2- Save as final all available drafts
3- Perform the migration!

!OrbeonMySQLMigrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:02'!
defaultDatabaseReaderClass

	^Orbeon46MySQL! !

!OrbeonMySQLMigrator methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 10:48'!
migrateDefinitionAttachmentsOf: anOrbeonFormDefinition
	"The receiver export all attachments of <anOrbeonFormDefinition> to the file system.
	This is done directly from MySql via: select file_content INTO DUMPFILE 'fileName'.
	Note: the files are stored in MySql host. If MySql and GemStone/S are in different host a manual file copy must be performed from MySql host to GemStone/S host.
	The MySql host must mimic the file structure where the Orbeon attachments are stored in GemStone/S:
	../temp
	../drafts
	../instances
	../definitions/saved
	../definitions/published"

	anOrbeonFormDefinition attachments do: [:defIdfileId | | path fileId definitionId |
			path := self apiConfiguration getSavedDefinitionsPathFor: anOrbeonFormDefinition formName in: anOrbeonFormDefinition getApplicationName.
			definitionId := (defIdfileId subStrings: '-') first.
			fileId := (defIdfileId subStrings: '-') last.
			self getOrbeonHandler fileoutDefinitionAttachmentOn: path defintionId: definitionId fileId: fileId version: anOrbeonFormDefinition version.
		].! !

!OrbeonMySQLMigrator methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 10:48'!
migrateDefinitionCustomAttachmentOf: anOrbeonFormDefinition
	"The receiver export all attachments of <anOrbeonFormDefinition> to the file system.
	This is done directly from MySql via: select file_content INTO DUMPFILE 'fileName'.
	Note: the files are stored in MySql host. If MySql and GemStone/S are in different host a manual file copy must be performed from MySql host to GemStone/S host.
	The MySql host must mimic the file structure where the Orbeon attachments are stored in GemStone/S:
	../temp
	../drafts
	../instances
	../definitions/saved
	../definitions/published"

	anOrbeonFormDefinition customAttachment ifNotNil: [ | path definitionId fileId |
			path := self apiConfiguration getSavedDefinitionsPathFor: anOrbeonFormDefinition formName in: anOrbeonFormDefinition getApplicationName.
			definitionId := (anOrbeonFormDefinition printCompleteCustomAttachmentId subStrings: '-') first.
			fileId := (anOrbeonFormDefinition printCompleteCustomAttachmentId subStrings: '-') last.
			self getOrbeonHandler fileoutDefinitionAttachmentOn: path defintionId: definitionId fileId: fileId version: anOrbeonFormDefinition version.
		].! !

!OrbeonMySQLMigrator methodsFor: 'migration api' stamp: 'BrunoBB 02/08/2016 12:45'!
migrateFormDefinitionNamed: aString
	"The receiver migrate ALL VERSIONS (from MySQL to GemStone) of the form definition named <aString>."

	^(self getOrbeonHandler getAllVersionsNumberOf: aString in: application applicationName)
		do: [:eachVersion | self migrateFormDefinitionNamed: aString version: eachVersion]! !

!OrbeonMySQLMigrator methodsFor: 'migration api' stamp: 'BrunoBB 02/08/2016 12:45'!
migrateFormDefinitionNamed: aString version: version
	"The receiver migrate (from MySQL to GemStone) the form definition named <aString> with version <version>.
	An OrbeonFormDefinition will be create inside OrbeonApplication <application>"
	| definition instanceDefinition dbString |

	definition := self getOrbeonHandler importFormDefinitionNamed: aString version: version in: application applicationName addToApp: true.
	definition setMetadata.
	dbString := self getOrbeonHandler getDBStringForInstanceDefinitionWithId: definition id version: version.
	"it could be a form definition with version = 2, but NO form instance definition with version 2 (it was never save after being published) --> we get the last"
	dbString isEmpty ifTrue: [dbString := self getOrbeonHandler getDBStringForInstanceDefinitionWithId: definition id].

	instanceDefinition := self getOrbeonHandler generateFormInstanceDefinition: aString app: application applicationName from: dbString.
	instanceDefinition setMetadata.
	definition instanceDefinition: instanceDefinition.
	instanceDefinition definition: definition.
	application addSavedDefinition: instanceDefinition.

	self getOrbeonHandler importDataFromScratchOf: definition. "import data <OrbeonFormInstance> filtered by <definition> version"

	self migrateDefinitionCustomAttachmentOf: definition.
	self migrateDefinitionAttachmentsOf: definition.

	self migrateInstanceAttachmentsOf: definition.! !

!OrbeonMySQLMigrator methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 10:49'!
migrateInstanceAttachmentsOf: anOrbeonFormDefinition
	"The receiver export all attachments of <anOrbeonFormDefinition> instances to the file system.
	This is done directly from MySql via: select file_content INTO DUMPFILE 'fileName'.
	Note: the files are stored in MySql host. If MySql and GemStone/S are in different host a manual file copy must be performed from MySql host to GemStone/S host.
	The MySql host must mimic the file structure where the Orbeon attachments are stored in GemStone/S:
	../temp
	../drafts
	../instances
	../definitions/saved
	../definitions/published"

	anOrbeonFormDefinition instances do:[:eachInstance |
		eachInstance getAttachmentIds do: [:instIdFileId | | path instanceId fileId |
			path := self apiConfiguration getInstancesPathFor: anOrbeonFormDefinition formName in: anOrbeonFormDefinition getApplicationName.
			instanceId := (instIdFileId subStrings: '-') first.
			fileId := (instIdFileId subStrings: '-') last.
			self getOrbeonHandler fileoutInstanceAttachmentOn: path defintionId: instanceId fileId: fileId version: anOrbeonFormDefinition version.
		]
	].! !

OrbeonDatabaseMigrator subclass: #OrbeonOracleMigrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonOracleMigrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:02'!
defaultDatabaseReaderClass

	^OrbeonOracleReader! !

OrbeonDatabaseMigrator subclass: #OrbeonPostgreSQLMigrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonPostgreSQLMigrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:02'!
defaultDatabaseReaderClass

	^OrbeonPostgreReader! !

OrbeonDatabaseMigrator subclass: #OrbeonSQLServerMigrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonSQLServerMigrator methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 14:03'!
defaultDatabaseReaderClass

	^OrbeonSQLServerReader! !

Object subclass: #OrbeonDatabaseReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonDatabaseReader subclass: #OrbeonDB2Reader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonDatabaseReader subclass: #OrbeonMySQLReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonMySQLReader subclass: #Orbeon46MySQL
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 10/14/2015 14:48'!
selectAllVersionOf: formNamed in: appName
	"Answer a SQL sentence with all versions of the form definition named <formNamed> in application <appName>"

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select form_version from orbeon.orbeon_form_definition where app = ''', appName, ''' and form = ''', formNamed, ''' group by form_version " '! !

!Orbeon46MySQL methodsFor: 'attachments' stamp: 'BrunoBB 10/16/2015 13:59'!
sqlDumpFileDefinitionAttachmentOn: path definitionId: definitionId fileId: fileId version: version

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select file_content INTO DUMPFILE ''', path, definitionId, '-', fileId, ''' from orbeon.orbeon_form_definition_attach where file_name = ''', fileId , ''' and form_version = ', version printString, ' " '! !

!Orbeon46MySQL methodsFor: 'attachments' stamp: 'BrunoBB 10/16/2015 20:21'!
sqlDumpFileInstanceAttachmentOn: path definitionId: definitionId fileId: fileId version: version

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select file_content INTO DUMPFILE ''', path, definitionId, '-', fileId, ''' from orbeon.orbeon_form_data_attach where file_name = ''', fileId , ''' and form_version = ', version printString, ' " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 05/26/2015 05:50'!
sqlFormDataWithId: formId

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where xml is not null and document_id = ''', formId, '''  order by last_modified_time desc limit 1" '! !

!Orbeon46MySQL methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 17:13'!
sqlFormDefinitionLastUpdateFor: app form: formName

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select last_modified_time from orbeon.orbeon_form_definition where app = ''', app , ''' and form = ''', formName , ''' and xml is not null order by last_modified_time desc" '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 11/19/2014 03:34'!
sqlGetAllDocumentIdsFrom: form of: app
	"Answer all document_id of the form in <orbeon_form_data> for the form <form> in the application <app>"

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select  document_id  from orbeon.orbeon_form_data where xml is not null and app = ''', app, ''' and form =''', form, '''  " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 10/14/2015 14:25'!
sqlGetAllDocumentIdsFrom: form of: app version: version
	"Answer all document_id of the form in <orbeon_form_data> for the form <form> in the application <app> with version <version>"

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select  document_id  from orbeon.orbeon_form_data where xml is not null and app = ''', app, ''' and form =''', form, ''' and form_version = ', version printString, ' " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 11/18/2014 09:12'!
sqlGetAllFormsData: form of: app

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where xml is not null and app = ''', app, ''' and form =''', form, '  " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 17:12'!
sqlGetAllFormsData: form of: app after: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where xml is not null and app = ''', app, ''' and form =''', form, ''' and last_modified_time > ''',  timeStampString, ''' " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 11/18/2014 05:06'!
sqlGetAllFormsData: form of: app after: timeStampString limit: stringLimit

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where xml is not null and app = ''', app, ''' and form =''', form, ''' and last_modified_time > ''',  timeStampString, ''' ', stringLimit, ' " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 03/29/2015 13:33'!
sqlGetAllFormsData: form version: version of: app after: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where app = ''', app, ''' and form =''', form, ''' and form_version = ', version ,' and  xml is not null and last_modified_time > ''',  timeStampString, ''' " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 17:12'!
sqlGetAllFormsDataAfter: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select * from orbeon.orbeon_form_data where xml is not null and last_modified_time > ',  timeStampString, ' " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 11/19/2014 03:25'!
sqlGetAllFormsWithId: documentId
	"Answer all stored versions of the form with id <documentId>"

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where xml is not null and document_id = ''', documentId, ''' " '! !

!Orbeon46MySQL methodsFor: 'instance definition' stamp: 'BrunoBB 10/15/2015 11:19'!
sqlGetAllInstanceDefinitions

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, document_id, lastMod, user, nameofgroup ,xml from (SELECT created, last_modified_time, document_id, concat(''_'', last_modified_by) lastMod, concat(''_'', username) user, concat(''_'', groupname) nameofgroup, xml FROM orbeon.orbeon_form_data where xml is not null and app=''orbeon'' and form=''builder'' order by last_modified_time desc) lastDef group by document_id; " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 17:13'!
sqlGetAllInstancesOf: formName in: appName 

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, last_modified_by, document_id, deleted, username, groupname from orbeon.orbeon_form_data where xml is not null and app=''', appName, ''' and form=''', formName,'''  order by last_modified_time desc" '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 17:14'!
sqlGetAllLastFormInstances

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select * from (select * from orbeon.orbeon_form_data where xml is not null order by last_modified_time desc) lastDef group by document_id" '! !

!Orbeon46MySQL methodsFor: 'attachments' stamp: 'BrunoBB 10/14/2015 12:17'!
sqlGetDataAttachmentContentFor: formId fileId: fileId

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select file_content from orbeon.orbeon_form_data_attach where document_id = ''', formId , ''' and file_name = ''', fileId, ''' " '! !

!Orbeon46MySQL methodsFor: 'attachments' stamp: 'BrunoBB 10/14/2015 12:17'!
sqlGetDataAttachmentsFor: formId

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select document_id, file_name from orbeon.orbeon_form_data_attach where document_id = ''', formId , ''' " '! !

!Orbeon46MySQL methodsFor: 'attachments' stamp: 'BrunoBB 10/16/2015 12:33'!
sqlGetDefinitionAttachmentContentFor: fileId version: version

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select file_content from orbeon.orbeon_form_definition_attach where file_name = ''', fileId , ''' and form_version = ', version printString, ' " '! !

!Orbeon46MySQL methodsFor: 'definitions' stamp: 'BrunoBB 03/29/2015 13:47'!
sqlGetFormDefinition: form version: version of: app after: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,
	' -N -B -e "select * from (select created, last_modified_time, concat(''_'', last_modified_by), app, form, deleted, form_version ,xml from orbeon.orbeon_form_definition where app = ''', 
		app, ''' and form= ''', form, '''', ' and form_version = ', version printString,' and last_modified_time > ''',  timeStampString, ''' and deleted=''N'' order by last_modified_time desc) lastDef group by form " '! !

!Orbeon46MySQL methodsFor: 'instance' stamp: 'BrunoBB 11/19/2014 03:31'!
sqlGetFormWithId: documentId
	"Answer the last stored versions of the form with id <documentId>"

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where xml is not null and document_id = ''', documentId, ''' order by last_modified_time desc limit 1 " '! !

!Orbeon46MySQL methodsFor: 'instance definition' stamp: 'BrunoBB 10/15/2015 14:06'!
sqlGetInstanceDefinitionTimeStamp: definitionTimeStamp
	"Answer a register from MySQL Orbeon.Data that represent a FormDefinition."
	| lowerTimeStamp |

	lowerTimeStamp := definitionTimeStamp minusSeconds: 60.
	"lowerTimeStamp := definitionTimeStamp subtractSeconds: 60."

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select last_modified_time, document_id, xml from (SELECT last_modified_time, document_id, xml FROM orbeon.orbeon_form_data where xml is not null and app=''orbeon'' and form=''builder'' and last_modified_time <= ''', definitionTimeStamp printMySqlString, ''' and last_modified_time >= ''', lowerTimeStamp printMySqlString, ''' order by last_modified_time desc) lastDef group by document_id; " '! !

!Orbeon46MySQL methodsFor: 'instance definition' stamp: 'BrunoBB 10/15/2015 12:34'!
sqlGetInstanceDefinitionWithId: id

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, document_id, lastMod, user, nameofgroup, xml from (SELECT created, last_modified_time, document_id, concat(''_'', last_modified_by) lastMod, concat(''_'', username) user, concat(''_'', groupname) nameofgroup, xml FROM orbeon.orbeon_form_data where xml is not null and app=''orbeon'' and form=''builder'' and document_id = ''', id, ''' order by last_modified_time desc) lastDef limit 1; " '! !

!Orbeon46MySQL methodsFor: 'instance definition' stamp: 'BrunoBB 10/15/2015 12:35'!
sqlGetInstanceDefinitionWithId: id version: version

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, document_id, lastMod, user, nameofgroup, xml from (SELECT created, last_modified_time, document_id, concat(''_'', last_modified_by) lastMod, concat(''_'', username) user, concat(''_'', groupname) nameofgroup, xml FROM orbeon.orbeon_form_data where xml is not null and app=''orbeon'' and form=''builder'' and document_id = ''', id, ''' and form_version = ', version printString, ' order by last_modified_time desc) lastDef limit 1; " '! !

!Orbeon46MySQL methodsFor: 'instance definition' stamp: 'BrunoBB 10/16/2015 07:59'!
sqlGetXMLInstanceDefinitionWithId: id

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select xml from (SELECT xml FROM orbeon.orbeon_form_data where xml is not null and app=''orbeon'' and form=''builder'' and document_id = ''', id, ''' order by last_modified_time desc) lastDef limit 1; " '! !

!OrbeonMySQLReader methodsFor: 'other' stamp: 'BrunoBB 02/03/2016 08:58'!
databaseIP

	^self getOrbeonDatabaseConfiguration databaseIP! !

!OrbeonMySQLReader methodsFor: 'other' stamp: 'BrunoBB 02/03/2016 08:58'!
databasePassword

	^self getOrbeonDatabaseConfiguration databasePassword! !

!OrbeonMySQLReader methodsFor: 'other' stamp: 'BrunoBB 02/03/2016 08:58'!
databaseUser

	^self getOrbeonDatabaseConfiguration databaseUser! !

!OrbeonMySQLReader methodsFor: 'other' stamp: 'BrunoBB 10/31/2014 16:58'!
initialize

	super initialize.! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlFormDefinitionLastUpdateFor: app form: formName

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select last_modified_time from orbeon.orbeon_form_definition where app = ''', app , ''' and form = ''', formName , ''' order by last_modified_time desc" '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 04/01/2015 07:42'!
sqlFormDefinitionLastUpdateFor: app form: formName version: version

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select last_modified_time from orbeon.orbeon_form_definition where app = ''', app , ''' and form = ''', formName , ''' and form_version = ', version, ' order by last_modified_time desc" '! !

!OrbeonMySQLReader methodsFor: 'application' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllApplications

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select distinct app from orbeon.orbeon_form_definition " '! !

!OrbeonMySQLReader methodsFor: 'application' stamp: 'BrunoBB 03/29/2015 12:48'!
sqlGetAllApplicationWithDefinitions

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select app, form from orbeon.orbeon_form_definition group by app, form " '! !

!OrbeonMySQLReader methodsFor: 'application' stamp: 'BrunoBB 03/29/2015 12:48'!
sqlGetAllApplicationWithDefinitionVersions

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select app, form, form_version from orbeon.orbeon_form_definition
group by app, form, form_version " '! !

!OrbeonMySQLReader methodsFor: 'application' stamp: 'BrunoBB 04/01/2015 08:00'!
sqlGetAllDefinitionVersionsOf: appName

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select app, form, form_version from orbeon.orbeon_form_definition where app = ''', appName , ''' group by app, form, form_version " '! !

!OrbeonMySQLReader methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllFormsData: form of: app after: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, concat(''_'', last_modified_by), app, form, document_id, deleted, xml, concat(''_'', username), concat(''_'', groupname), form_version from orbeon.orbeon_form_data where app = ''', app, ''' and form =''', form, ''' and last_modified_time > ''',  timeStampString, ''' " '! !

!OrbeonMySQLReader methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllFormsDataAfter: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select * from orbeon.orbeon_form_data where last_modified_time > ',  timeStampString, ' " '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllFormsDefinition

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select * from orbeon.orbeon_form_definition " '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllFormsDefinition: fileName

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select * from orbeon.orbeon_form_definition " > ', fileName! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 03/29/2015 12:48'!
sqlGetAllFormsDefinition: form of: app after: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,
	' -N -B -e "select * from (select created, last_modified_time, concat(''_'', last_modified_by), app, form, deleted, form_version ,xml from orbeon.orbeon_form_definition where app = ''', 
		app, ''' and form= ''', form, ''' and last_modified_time > ''',  timeStampString, ''' and deleted=''N'' 
		order by last_modified_time desc) lastDef group by form " '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 04/02/2015 14:48'!
sqlGetAllFormsDefinition: form of: app version: version after: timeStampString

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,
	' -N -B -e "select * from (select created, last_modified_time, concat(''_'', last_modified_by), app, form, deleted, form_version ,xml from orbeon.orbeon_form_definition where app = ''', 
		app, ''' and form= ''', form, ''' and form_version = ', version ,' and last_modified_time > ''',  timeStampString, ''' and deleted=''N'' 
		order by last_modified_time desc) lastDef group by form " '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllFormsDefinitionOf: appName

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select * from (select form from orbeon.orbeon_form_definition where app=''', appName, ''' and deleted=''N'' order by last_modified_time) lastDef group by form "'! !

!OrbeonMySQLReader methodsFor: 'instance definition' stamp: 'BrunoBB 10/15/2015 11:20'!
sqlGetAllInstanceDefinitions

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, document_id, lastMod, user, nameofgroup ,xml from (SELECT created, last_modified_time, document_id, concat(''_'', last_modified_by) lastMod, concat(''_'', username) user, concat(''_'', groupname) nameofgroup, xml FROM orbeon.orbeon_form_data where xml is not null and app=''orbeon'' and form=''builder'' order by last_modified_time desc) lastDef group by document_id; " '! !

!OrbeonMySQLReader methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllInstancesOf: formName in: appName 

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select created, last_modified_time, last_modified_by, document_id, deleted, username, groupname from orbeon.orbeon_form_data where app=''', appName, ''' and form=''', formName,'''  order by last_modified_time desc" '! !

!OrbeonMySQLReader methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllInstancesOf: formName in: appName writeTo: fileName

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select xml from orbeon.orbeon_form_data where app=''', appName, ''' and form=''', formName,'''  order by last_modified_time desc" > ', fileName! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllLastFormDefinitions

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select * from (select * from orbeon.orbeon_form_definition order by last_modified_time desc) lastDef group by app, form" '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllLastFormDefinitions: fileName

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select * from (select * from orbeon.orbeon_form_definition order by last_modified_time desc) lastDef group by app, form" > ', fileName! !

!OrbeonMySQLReader methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllLastFormInstances

	^'mysql -h ', self databaseIP,' -u ', self databaseUser,' -p', self databasePassword,' -N -B -e "select * from (select * from orbeon.orbeon_form_data order by last_modified_time desc) lastDef group by document_id" '! !

!OrbeonMySQLReader methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllLastFormInstances: fileName

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select * from (select * from orbeon.orbeon_form_data order by last_modified_time desc) lastDef group by document_id" > ', fileName! !

!OrbeonMySQLReader methodsFor: 'instance' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetAllLastInstances: fileName

	^'mysql -h ', self databaseIP,' -u ', self databaseUser,' -p', self databasePassword,' -N -B -e "select * from orbeon.orbeon_form_data " > ', fileName! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 03/29/2015 12:48'!
sqlGetFormDefinitionsVersionsOf: form in: app

	^'mysql -h ', self databaseIP,' -u ',self databaseUser,' -p', self databasePassword,' -N -B -e "select app, form, form_version from (select app, form, form_version from orbeon.orbeon_form_definition where app = ''', app,''' and form = ''', form ,''' order by last_modified_time desc) lastDef group by app, form, form_version" '! !

!OrbeonMySQLReader methodsFor: 'instance definition' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetInstanceDefinitionTimeStamp: definitionTimeStamp
	"Answer a register from MySQL Orbeon.Data that represent a FormDefinition."
	| lowerTimeStamp |

	lowerTimeStamp := definitionTimeStamp minusSeconds: 60.

	^'mysql -h ',self databaseIP,' -u ', self databaseUser,' -p',self databasePassword,' -N -B -e "select last_modified_time, document_id, xml from (SELECT last_modified_time, document_id, xml FROM orbeon.orbeon_form_data where app=''orbeon'' and form=''builder'' and last_modified_time <= ''', definitionTimeStamp printMySqlString, ''' and last_modified_time >= ''', lowerTimeStamp printMySqlString, ''' order by last_modified_time desc) lastDef group by document_id; " '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetLastDefinitionOf: formName in: appName

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select xml from orbeon.orbeon_form_definition where app=''', appName, ''' and form=''', formName,'''  order by last_modified_time desc limit 1" '! !

!OrbeonMySQLReader methodsFor: 'definitions' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlGetLastDefinitionOf: formName in: appName writeTo: fileName

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "select xml from orbeon.orbeon_form_definition where app=''', appName, ''' and form=''', formName,'''  order by last_modified_time desc limit 1" > ', fileName! !

!OrbeonMySQLReader methodsFor: 'insert' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlInsertFieldDefinition: aFormFieldDefinition id: fieldId

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "INSERT INTO orbeon_metadata.field_definition (field_id, form_definition_id, name, isSummary, isSearch, label, field_type, repeat_name) VALUES (''', fieldId printString, ''',', '''',aFormFieldDefinition definition id, ''',', '''', aFormFieldDefinition name, ''',', '''', aFormFieldDefinition isSummary asSqlString, ''',', '''', aFormFieldDefinition isSearch asSqlString, ''',', '''', aFormFieldDefinition label, ''',', '''', aFormFieldDefinition type, ''',', '''no repeat name''', ')"'! !

!OrbeonMySQLReader methodsFor: 'insert' stamp: 'BrunoBB 10/31/2014 16:58'!
sqlInsertFieldValue: aFieldValue id: fieldId valueField: valueField

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "INSERT INTO orbeon_metadata.field_value (field_id, form_instance_id, name, type, ',valueField, ', original_value, error_string) VALUES (''', fieldId printString, ''',', '''', aFieldValue instanceId, ''',', '''', aFieldValue name, ''',', '''', aFieldValue typeName, ''',', '''', aFieldValue value printMySqlString, ''',', '''', aFieldValue originalValue, ''',', '''', aFieldValue errorString printString, '''', ')"'! !

!OrbeonMySQLReader methodsFor: 'insert' stamp: 'BrunoBB 04/28/2016 11:04'!
sqlInsertOrbeonDefinition: anOrbeonFormDefinition

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "INSERT INTO orbeon_metadata.orbeon_form_definition (orbeon_id, form_name, app, version, created_time, modified_time, deleted, isPublished) VALUES (''', anOrbeonFormDefinition id, ''',', '''',anOrbeonFormDefinition formName, ''',', '''', anOrbeonFormDefinition app, ''',','''01'',', '''',anOrbeonFormDefinition createdTime printMySqlString, ''',', '''', anOrbeonFormDefinition modifiedTime printMySqlString, ''',', '''N'',', '''S''', ')"'! !

!OrbeonMySQLReader methodsFor: 'insert' stamp: 'BrunoBB 04/28/2016 11:05'!
sqlInsertOrbeonInstance: anOrbeonFormInstance

	^'mysql -h ',self databaseIP,' -u ',self databaseUser,' -p',self databasePassword,' -N -B -e "INSERT INTO orbeon_metadata.orbeon_form_instance (orbeon_instance_id, form_definition_id, form, app, created_time, modified_time, modified_user, username, groupname, deleted) VALUES (''', anOrbeonFormInstance id, ''',', '''',anOrbeonFormInstance definitionId, ''',' ,'''',anOrbeonFormInstance formName, ''',', '''', anOrbeonFormInstance getApplicationName, ''',', '''',anOrbeonFormInstance createdTime printMySqlString, ''',', '''',anOrbeonFormInstance modifiedTime printMySqlString, ''',', '''',anOrbeonFormInstance modifiedUser, ''',', '''',anOrbeonFormInstance username, ''',', '''',anOrbeonFormInstance groupname, ''',' ,'''N''', ')"'! !

OrbeonDatabaseReader subclass: #OrbeonOracleReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonDatabaseReader subclass: #OrbeonPostgreReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonDatabaseReader subclass: #OrbeonSQLServerReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

Object subclass: #OrbeonFormReader
	instanceVariableNames: 'xmlDefinition xmlDocument xmlInstanceDocument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
definitionExample

	^'<xh:html xmlns:xh="http://www.w3.org/1999/xhtml" xmlns:xf="http://www.w3.org/2002/xforms"
         xmlns:xs="http://www.w3.org/2001/XMLSchema"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:ev="http://www.w3.org/2001/xml-events"
         xmlns:xi="http://www.w3.org/2001/XInclude"
         xmlns:xxi="http://orbeon.org/oxf/xml/xinclude"
         xmlns:xxf="http://orbeon.org/oxf/xml/xforms"
         xmlns:exf="http://www.exforms.org/exf/1-0"
         xmlns:fr="http://orbeon.org/oxf/xml/form-runner"
         xmlns:saxon="http://saxon.sf.net/"
         xmlns:sql="http://orbeon.org/oxf/xml/sql"
         xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
         xmlns:fb="http://orbeon.org/oxf/xml/form-builder">
    <xh:head>
        <xh:title>FORMULARIO PARA REALIZAR DENUNCIAS ANTE LA UNIDAD REGULADORA Y DE CONTROL DE DATOS PERSONALES</xh:title>
        <xf:model id="fr-form-model" xxf:expose-xpath-types="true">

            <!!-- Main instance -->
            <xf:instance id="fr-form-instance">
                <form>
                    <section-1>
                        <montevideo/>
                        <fechaActual/>
                        
                    </section-1>
                    <section-4>
                        <nombreDenunciante/>
                        <cedulaDenunciante/>
                        <direccionDenunciante/>
                        <ciudadDenunciante/>
                        <deptoDenunciante/>
                        <telDenunciante/>
                        <correoDenunciante/>
                    </section-4>
                    <section-5>
                        <nombreDenunciado/>
                        <cedulaDenunciado/>
                        <direccionDenunciado/>
                        <ciudadDenunciado/>
                        <deptoDenunciado/>
                        <telefonoDenunciado/>
                        <correoDenunciado/>
                    </section-5>
                    <section-21>
                        <detalleDenuncia/>
                    </section-21>
                    <section-7>
                        <firma/>
                        <confirmacionDenuncia/>
                    </section-7>
                </form>
            </xf:instance>

            <!!-- Bindings -->
            <xf:bind xmlns:dataModel="java:org.orbeon.oxf.fb.DataModel" id="fr-form-binds"
                     ref="instance(''fr-form-instance'')">
                <xf:bind id="section-1-bind" name="section-1" ref="section-1">
                    <xf:bind name="montevideo" id="montevideo-bind" ref="montevideo" readonly="true()"/>
                    <xf:bind name="fechaActual" id="fechaActual-bind" ref="fechaActual" type="xf:date"
                             readonly="true()"
                             xxf:default="current-date()"/>
                    
                </xf:bind>
                <xf:bind name="section-4" id="section-4-bind" ref="section-4">
                    <xf:bind name="cedulaDenunciante" id="cedulaDenunciante-bind" ref="cedulaDenunciante"/>
                    <xf:bind name="direccionDenunciante" id="direccionDenunciante-bind"
                             ref="direccionDenunciante"/>
                    <xf:bind name="ciudadDenunciante" id="ciudadDenunciante-bind" ref="ciudadDenunciante"/>
                    <xf:bind name="deptoDenunciante" id="deptoDenunciante-bind" ref="deptoDenunciante"/>
                    <xf:bind name="telDenunciante" id="telDenunciante-bind" ref="telDenunciante"/>
                    <xf:bind name="correoDenunciante" id="correoDenunciante-bind" ref="correoDenunciante"/>
                    <xf:bind name="nombreDenunciante" id="nombreDenunciante-bind" ref="nombreDenunciante"/>
                </xf:bind>
                <xf:bind name="section-5" id="section-5-bind" ref="section-5">
                    <xf:bind name="nombreDenunciado" id="nombreDenunciado-bind" ref="nombreDenunciado"/>
                    <xf:bind name="cedulaDenunciado" id="cedulaDenunciado-bind" ref="cedulaDenunciado"/>
                    <xf:bind name="direccionDenunciado" id="direccionDenunciado-bind"
                             ref="direccionDenunciado"/>
                    <xf:bind name="ciudadDenunciado" id="ciudadDenunciado-bind" ref="ciudadDenunciado"/>
                    <xf:bind name="deptoDenunciado" id="deptoDenunciado-bind" ref="deptoDenunciado"/>
                    <xf:bind name="telefonoDenunciado" id="telefonoDenunciado-bind" ref="telefonoDenunciado"/>
                    <xf:bind name="correoDenunciado" id="correoDenunciado-bind" ref="correoDenunciado"/>
                </xf:bind>
                <xf:bind name="section-7" id="section-7-bind" ref="section-7">
                    <xf:bind name="firma" id="firma-bind" ref="firma" readonly="true()"/>
                    <xf:bind name="confirmacionDenuncia" id="confirmacionDenuncia-bind"
                             ref="confirmacionDenuncia"
                             type="xs:boolean"
                             required="true()"/>
                </xf:bind>
                <xf:bind name="section-21" id="section-21-bind" ref="section-21">
                    <xf:bind name="detalleDenuncia" id="detalleDenuncia-bind" ref="detalleDenuncia"/>
                </xf:bind>
            </xf:bind>

            <!!-- Metadata -->
            <xf:instance xxf:readonly="true" id="fr-form-metadata">
                <metadata>
                    <application-name>URCDP</application-name>
                    <form-name>Denuncias</form-name>
                    <title xml:lang="en">FORMULARIO PARA REALIZAR DENUNCIAS ANTE LA UNIDAD REGULADORA Y DE CONTROL DE DATOS PERSONALES</title>
                    <description xml:lang="en">Formulario para el ingreso de denuncias de violacin de datos personales</description>
                </metadata>
            </xf:instance>

            <!!-- Attachments -->
            <xf:instance id="fr-form-attachments">
                <attachments>
                    <css mediatype="text/css" filename="" size=""/>
                    <pdf mediatype="application/pdf" filename="" size=""/>
                </attachments>
            </xf:instance>

            <!!-- All form resources -->
            <!!-- Dont make readonly by default in case a service modifies the resources -->
            <xf:instance id="fr-form-resources" xxf:readonly="false">
                <resources>
                    <resource xml:lang="en">
                        <firma>
                            <label>Por lo expuesto, SOLICITO se inicien las actuaciones administrativas correspondientes con el objeto de determinar la existencia de una eventual violacin a las disposiciones de la Ley N 18.331 y tomar las medidas que correspondan a derecho.</label>
                            <hint/>
                            <help/>
                            <alert/>
                        </firma>
                        <confirmacionDenuncia>
                            <label>Seleccione esta opcin para confirmar la denuncia</label>
                            <hint>Seleccione para confirmar</hint>
                            <help/>
                            <alert/>
                        </confirmacionDenuncia>
                        <detalleDenuncia>
                            <label>De acuerdo con lo previsto en el artculo 34 literal A) de la Ley N 18.331, de 11 de agosto de 2008 y normas concordantes, pongo en conocimiento de la Unidad Reguladora y de Control de Datos Personales, los siguientes HECHOS:</label>
                            <hint>Ingrese detalle de la denuncia</hint>
                            <help/>
                            <alert/>
                        </detalleDenuncia>
                        <nombreDenunciante>
                            <label>Sr/Sra</label>
                            <hint>Ingrese nombre del denunciante</hint>
                            <help/>
                            <alert/>
                        </nombreDenunciante>
                        <nombreDenunciado>
                            <label>Sr/Sra</label>
                            <hint>Ingrese nombre del denunciado</hint>
                            <help/>
                            <alert/>
                        </nombreDenunciado>
                        <cedulaDenunciado>
                            <label>Cedula de Identidad</label>
                            <hint>Ingrese cedula del denunciado</hint>
                            <help/>
                            <alert/>
                        </cedulaDenunciado>
                        <direccionDenunciado>
                            <label>Direccin</label>
                            <hint>Ingrese direccin del denunciado</hint>
                            <help/>
                            <alert/>
                        </direccionDenunciado>
                        <ciudadDenunciado>
                            <label>Ciudad</label>
                            <hint>Ingrese ciudad del denunciado</hint>
                            <help/>
                            <alert/>
                        </ciudadDenunciado>
                        <deptoDenunciado>
                            <label>Departamento</label>
                            <hint>Seleccione Departamento del denunciado</hint>
                            <help/>
                            <alert/>
                            <item>
                                <label>Montevideo</label>
                                <value>montevideo</value>
                            </item>
                            <item>
                                <label>Artigas</label>
                                <value>artigas</value>
                            </item>
                            <item>
                                <label>Canelones</label>
                                <value>canelones</value>
                            </item>
                            <item>                  
                                <label>Cerro Largo</label>
                                <value>cerro-largo</value>
                            </item>
                            <item>                  
                                <label>Colonia</label>
                                <value>colonia</value>
                            </item>
                            <item>                  
                                <label>Durazno</label>
                                <value>durazno</value>
                            </item>
                            <item>                  
                                <label>Flores</label>
                                <value>flores</value>
                            </item>
                            <item>                  
                                <label>Florida</label>
                                <value>florida</value>
                            </item>
                            <item>                  
                                <label>Lavalleja</label>
                                <value>lavalleja</value>
                            </item>
                            <item>                  
                                <label>Maldonado</label>
                                <value>maldonado</value>
                            </item>
                            <item>                  
                                <label>Paysand</label>
                                <value>paysandu</value>
                            </item>
                            <item>                  
                                <label>Ro Negro</label>
                                <value>rio-negro</value>
                            </item>
                            <item>                  
                                <label>Rivera</label>
                                <value>rivera</value>
                            </item>
                            <item>                  
                                <label>Rocha</label>
                                <value>rocha</value>
                            </item>
                            <item>                  
                                <label>Salto</label>
                                <value>salto</value>
                            </item>
                            <item>                  
                                <label>San Jos</label>
                                <value>san-jose</value>
                            </item>
                            <item>                  
                                <label>Soriano</label>
                                <value>soriano</value>
                            </item>
                            <item>                  
                                <label>Tacuaremb</label>
                                <value>tacuarembo</value>
                            </item>
                            <item>                  
                                <label>Treinta y Tres</label>
                                <value>treinta-y-tres</value>
                            </item>
                        </deptoDenunciado>
                        <telefonoDenunciado>
                            <label>Telfono</label>
                            <hint>Ingrese telfono del denunciado</hint>
                            <help/>
                            <alert/>
                        </telefonoDenunciado>
                        <correoDenunciado>
                            <label>Correo Electrnico</label>
                            <hint>Ingrese correo electrnico del denunciado</hint>
                            <help/>
                            <alert/>
                        </correoDenunciado>
                        <cedulaDenunciante>
                            <label>Cedula de Identidad</label>
                            <hint>Ingrese cedula del denunciante</hint>
                            <help/>
                            <alert/>
                        </cedulaDenunciante>
                        <direccionDenunciante>
                            <label>Direccin</label>
                            <hint>Ingrese direccin del denunciante</hint>
                            <help/>
                            <alert/>
                        </direccionDenunciante>
                        <ciudadDenunciante>
                            <label>Ciudad</label>
                            <hint>Ingrese ciudad del denunciante</hint>
                            <help/>
                            <alert/>
                        </ciudadDenunciante>
                        <deptoDenunciante>
                            <label>Departamento</label>
                            <hint>Seleccione Departamento del denunciante</hint>
                            <help/>
                            <alert/>
                            <item>
                                <label>Montevideo</label>
                                <value>montevideo</value>
                            </item>
                            <item>
                                <label>Artigas</label>
                                <value>artigas</value>
                            </item>
                            <item>
                                <label>Canelones</label>
                                <value>canelones</value>
                            </item>
                            <item>                  
                                <label>Cerro Largo</label>
                                <value>cerro-largo</value>
                            </item>
                            <item>                  
                                <label>Colonia</label>
                                <value>colonia</value>
                            </item>
                            <item>                  
                                <label>Durazno</label>
                                <value>durazno</value>
                            </item>
                            <item>                  
                                <label>Flores</label>
                                <value>flores</value>
                            </item>
                            <item>                  
                                <label>Florida</label>
                                <value>florida</value>
                            </item>
                            <item>                  
                                <label>Lavalleja</label>
                                <value>lavalleja</value>
                            </item>
                            <item>                  
                                <label>Maldonado</label>
                                <value>maldonado</value>
                            </item>
                            <item>                  
                                <label>Paysand</label>
                                <value>paysandu</value>
                            </item>
                            <item>                  
                                <label>Ro Negro</label>
                                <value>rio-negro</value>
                            </item>
                            <item>                  
                                <label>Rivera</label>
                                <value>rivera</value>
                            </item>
                            <item>                  
                                <label>Rocha</label>
                                <value>rocha</value>
                            </item>
                            <item>                  
                                <label>Salto</label>
                                <value>salto</value>
                            </item>
                            <item>                  
                                <label>San Jos</label>
                                <value>san-jose</value>
                            </item>
                            <item>                  
                                <label>Soriano</label>
                                <value>soriano</value>
                            </item>
                            <item>                  
                                <label>Tacuaremb</label>
                                <value>tacuarembo</value>
                            </item>
                            <item>                  
                                <label>Treinta y Tres</label>
                                <value>treinta-y-tres</value>
                            </item>
                        </deptoDenunciante>
                        <telDenunciante>
                            <label>Telfono</label>
                            <hint>Ingrese telfono del denunciante</hint>
                            <help/>
                            <alert/>
                        </telDenunciante>
                        <correoDenunciante>
                            <label>Correo Electrnico</label>
                            <hint>Ingrese correo electrnico del denunciante</hint>
                            <help/>
                            <alert/>
                        </correoDenunciante>
                        <montevideo>
                            <label>Montevideo</label>
                            <hint>Ciudad de Montevideo</hint>
                            <help/>
                            <alert/>
                        </montevideo>
                        <fechaActual>
                            <label>Fecha Actual</label>
                            <hint>Fecha Actual</hint>
                            <help/>
                            <alert/>
                        </fechaActual>
                        <section-1>
                            <label>Fecha de Ingreso</label>
                            <help/>
                        </section-1>
                        <section-4>
                            <label>Datos del Denunciante</label>
                            <help/>
                        </section-4>
                        <section-5>
                            <label>Datos del Denunciado</label>
                            <help/>
                        </section-5>
                        <section-21>
                            <label>Detalle de la Denuncia</label>
                            <help/>
                        </section-21>
                        <section-7>
                            <label>Firma</label>
                            <help/>
                        </section-7>
                        
                    </resource>
                </resources>
            </xf:instance>

            <!!-- Utility instances for services -->
            <xf:instance id="fr-service-request-instance" xxf:exclude-result-prefixes="#all">
                <request/>
            </xf:instance>

            <xf:instance id="fr-service-response-instance" xxf:exclude-result-prefixes="#all">
                <response/>
            </xf:instance>

        </xf:model>
    </xh:head>
    <xh:body>
        <fr:view>
            <fr:body xmlns:xbl="http://www.w3.org/ns/xbl"
                     xmlns:dataModel="java:org.orbeon.oxf.fb.DataModel"
                     xmlns:oxf="http://www.orbeon.com/oxf/processors"
                     xmlns:p="http://www.orbeon.com/oxf/pipeline">
                <fr:section id="section-1-control" bind="section-1-bind">
                    <xf:label ref="$form-resources/section-1/label"/>
                    <xf:help ref="$form-resources/section-1/help"/>
                    <fr:grid>
                        <xh:tr>
                            
                            <xh:td>
                                <xf:output id="montevideo-control" bind="montevideo-bind">
                                    <xf:label ref="$form-resources/montevideo/label"/>
                                    <xf:hint ref="$form-resources/montevideo/hint"/>
                                    <xf:help ref="$form-resources/montevideo/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:output>
                            </xh:td>
                            <xh:td>
                                <xf:input id="fechaActual-control" bind="fechaActual-bind">
                                    <xf:label ref="$form-resources/fechaActual/label"/>
                                    <xf:hint ref="$form-resources/fechaActual/hint"/>
                                    <xf:help ref="$form-resources/fechaActual/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                        </xh:tr>
                    </fr:grid>
                </fr:section>
                <fr:section id="section-4-control" bind="section-4-bind">
                    <xf:label ref="$form-resources/section-4/label"/>
                    <xf:help ref="$form-resources/section-4/help"/>
                    <fr:grid>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="nombreDenunciante-control" bind="nombreDenunciante-bind">
                                    <xf:label ref="$form-resources/nombreDenunciante/label"/>
                                    <xf:hint ref="$form-resources/nombreDenunciante/hint"/>
                                    <xf:help ref="$form-resources/nombreDenunciante/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td>
                                <xf:input id="cedulaDenunciante-control" bind="cedulaDenunciante-bind">
                                    <xf:label ref="$form-resources/cedulaDenunciante/label"/>
                                    <xf:hint ref="$form-resources/cedulaDenunciante/hint"/>
                                    <xf:help ref="$form-resources/cedulaDenunciante/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                        </xh:tr>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="direccionDenunciante-control" bind="direccionDenunciante-bind">
                                    <xf:label ref="$form-resources/direccionDenunciante/label"/>
                                    <xf:hint ref="$form-resources/direccionDenunciante/hint"/>
                                    <xf:help ref="$form-resources/direccionDenunciante/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td/>
                        </xh:tr>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="ciudadDenunciante-control" bind="ciudadDenunciante-bind">
                                    <xf:label ref="$form-resources/ciudadDenunciante/label"/>
                                    <xf:hint ref="$form-resources/ciudadDenunciante/hint"/>
                                    <xf:help ref="$form-resources/ciudadDenunciante/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td>
                                <xf:select1 id="deptoDenunciante-control" appearance="minimal" bind="deptoDenunciante-bind">
                                    <xf:label ref="$form-resources/deptoDenunciante/label"/>
                                    <xf:hint ref="$form-resources/deptoDenunciante/hint"/>
                                    <xf:help ref="$form-resources/deptoDenunciante/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                    <xf:item>
                                        <xf:label>[Select...]</xf:label>
                                        <xf:value/>
                                    </xf:item>
                                    <xf:itemset ref="$form-resources/deptoDenunciante/item">
                                        <xf:label ref="label"/>
                                        <xf:value ref="value"/>
                                    </xf:itemset>
                                </xf:select1>
                            </xh:td>
                        </xh:tr>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="telDenunciante-control" bind="telDenunciante-bind">
                                    <xf:label ref="$form-resources/telDenunciante/label"/>
                                    <xf:hint ref="$form-resources/telDenunciante/hint"/>
                                    <xf:help ref="$form-resources/telDenunciante/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td>
                                <xf:input id="correoDenunciante-control" bind="correoDenunciante-bind">
                                    <xf:label ref="$form-resources/correoDenunciante/label"/>
                                    <xf:hint ref="$form-resources/correoDenunciante/hint"/>
                                    <xf:help ref="$form-resources/correoDenunciante/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                        </xh:tr>
                    </fr:grid>
                </fr:section>
                <fr:section id="section-5-control" bind="section-5-bind">
                    <xf:label ref="$form-resources/section-5/label"/>
                    <xf:help ref="$form-resources/section-5/help"/>
                    <fr:grid>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="nombreDenunciado-control" bind="nombreDenunciado-bind">
                                    <xf:label ref="$form-resources/nombreDenunciado/label"/>
                                    <xf:hint ref="$form-resources/nombreDenunciado/hint"/>
                                    <xf:help ref="$form-resources/nombreDenunciado/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td>
                                <xf:input id="cedulaDenunciado-control" bind="cedulaDenunciado-bind">
                                    <xf:label ref="$form-resources/cedulaDenunciado/label"/>
                                    <xf:hint ref="$form-resources/cedulaDenunciado/hint"/>
                                    <xf:help ref="$form-resources/cedulaDenunciado/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                        </xh:tr>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="direccionDenunciado-control" bind="direccionDenunciado-bind">
                                    <xf:label ref="$form-resources/direccionDenunciado/label"/>
                                    <xf:hint ref="$form-resources/direccionDenunciado/hint"/>
                                    <xf:help ref="$form-resources/direccionDenunciado/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td/>
                        </xh:tr>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="ciudadDenunciado-control" bind="ciudadDenunciado-bind">
                                    <xf:label ref="$form-resources/ciudadDenunciado/label"/>
                                    <xf:hint ref="$form-resources/ciudadDenunciado/hint"/>
                                    <xf:help ref="$form-resources/ciudadDenunciado/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td>
                                <xf:select1 id="deptoDenunciado-control" appearance="minimal" bind="deptoDenunciado-bind">
                                    <xf:label ref="$form-resources/deptoDenunciado/label"/>
                                    <xf:hint ref="$form-resources/deptoDenunciado/hint"/>
                                    <xf:help ref="$form-resources/deptoDenunciado/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                    <xf:item>
                                        <xf:label>[Select...]</xf:label>
                                        <xf:value/>
                                    </xf:item>
                                    <xf:itemset ref="$form-resources/deptoDenunciado/item">
                                        <xf:label ref="label"/>
                                        <xf:value ref="value"/>
                                    </xf:itemset>
                                </xf:select1>
                            </xh:td>
                        </xh:tr>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="telefonoDenunciado-control" bind="telefonoDenunciado-bind">
                                    <xf:label ref="$form-resources/telefonoDenunciado/label"/>
                                    <xf:hint ref="$form-resources/telefonoDenunciado/hint"/>
                                    <xf:help ref="$form-resources/telefonoDenunciado/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td>
                                <xf:input id="correoDenunciado-control" bind="correoDenunciado-bind">
                                    <xf:label ref="$form-resources/correoDenunciado/label"/>
                                    <xf:hint ref="$form-resources/correoDenunciado/hint"/>
                                    <xf:help ref="$form-resources/correoDenunciado/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                        </xh:tr>
                    </fr:grid>
                </fr:section>
                <fr:section id="section-21-control" bind="section-21-bind">
                    <xf:label ref="$form-resources/section-21/label"/>
                    <xf:help ref="$form-resources/section-21/help"/>
                    <fr:grid>
                        <xh:tr>
                            <xh:td>
                                <xf:textarea id="detalleDenuncia-control" bind="detalleDenuncia-bind">
                                    <xf:label ref="$form-resources/detalleDenuncia/label"/>
                                    <xf:hint ref="$form-resources/detalleDenuncia/hint"/>
                                    <xf:help ref="$form-resources/detalleDenuncia/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:textarea>
                            </xh:td>
                        </xh:tr>
                    </fr:grid>
                </fr:section>
                <fr:section id="section-7-control" bind="section-7-bind">
                    <xf:label ref="$form-resources/section-7/label"/>
                    <xf:help ref="$form-resources/section-7/help"/>
                    <fr:grid>
                        <xh:tr>
                            <xh:td>
                                <xf:output id="firma-control" bind="firma-bind">
                                    <xf:label ref="$form-resources/firma/label"/>
                                    <xf:hint ref="$form-resources/firma/hint"/>
                                    <xf:help ref="$form-resources/firma/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:output>
                            </xh:td>
                        </xh:tr>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="confirmacionDenuncia-control" bind="confirmacionDenuncia-bind">
                                    <xf:label ref="$form-resources/confirmacionDenuncia/label"/>
                                    <xf:hint ref="$form-resources/confirmacionDenuncia/hint"/>
                                    <xf:help ref="$form-resources/confirmacionDenuncia/help"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                        </xh:tr>
                    </fr:grid>
                </fr:section>
            </fr:body>
        </fr:view>
    </xh:body>
</xh:html>'! !

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
example1

	^self for: (FileStream fileNamed: '/opt/encuesta-ute2.xml') contents.! !

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
example2
 "
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute2.xml') contents with: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents.
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents
" 

	^self for: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents.! !

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
example3
 "
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute2.xml') contents with: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents.
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents
" 

	^self for: (FileStream fileNamed: '/opt/definition-orbeon46.xml') contents.! !

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
example4
 "
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute2.xml') contents with: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents.
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents
" 

	^self for: (FileStream fileNamed: '/opt/orbeon-def46.xml') contents.! !

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
example5
 "
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute2.xml') contents with: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents.
OrbeonFormReader for: (FileStream fileNamed: '/opt/encuesta-ute-form.xml') contents
" 

	^self for: (FileStream fileNamed: '/opt/orbeon-def46.xml') contents 
			with: (FileStream fileNamed: '/opt/instance-orbeon46.xml') contents! !

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
for: xmlOrbeonFormDefinition

	^super new initialize
		xmlDefinition: xmlOrbeonFormDefinition;
		parseXMLDefinition;
		yourself! !

!OrbeonFormReader class methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
for: xmlOrbeonFormDefinition with: xmlOrbeonInstance

	^super new initialize
		xmlDefinition: xmlOrbeonFormDefinition;
		parseXMLDefinition;
		setXmlInstanceDocument: xmlOrbeonInstance;
		yourself! !

!OrbeonFormReader methodsFor: 'permissions' stamp: 'BrunoBB 08/16/2016 05:33'!
addAnyOnePermissionTo: permissions fromString: stringOperations
	"The receiver add permissions to OrbeonPermissions <permissions> took from string <stringOperations>.
	<stringOperations> format: 'read update delete' "

	(stringOperations subStrings: ' ') do: [:oper | permissions add: oper to: 'anyone']! !

!OrbeonFormReader methodsFor: 'permissions' stamp: 'BrunoBB 08/16/2016 05:33'!
addPermissionTo: permissions fromString: stringOperations using: permissionNode
	"The receiver add permission to OrbeonPermissions <permissions> to roles in node <permissionNode> with grant <stringOperations>"
	| subNode |

	subNode := permissionNode nodes first.
	
	(stringOperations subStrings: ' ') do: [:oper | 
		(subNode name = 'user-role')
		ifFalse: [permissions add: oper to: subNode name] "for group-members and owner"
		"for specific roles. -any-of- can have more than one role"
		ifTrue: [((subNode attributeAt: 'any-of') subStrings: ' ') do: [:rol | permissions add: oper to: rol]] 
	].! !

!OrbeonFormReader methodsFor: 'recursive section' stamp: 'BrunoBB 08/16/2016 05:33'!
createFieldsInstancesFor: aFormSectionInstance xmlSegment: xmlInstanceSegment
	"The receiver iterate over <aFormSectionInstance> fields names to get the values of each one contained in <xmlInstanceSegment>.
	Store each value in <aFormSectionInstance>"

	aFormSectionInstance fieldNames do: [:fieldName | | node value |

		node := xmlInstanceSegment elementAt: fieldName. "get xml field node"

		value := (node isNil or: [node hasChildren not]) ifTrue: [''] ifFalse: [self getValueFrom: node]. "get field value"

		aFormSectionInstance fieldAt: fieldName put: value.  "store the field value in the form instance <aFormSectionInstance>"

		"values at: fieldName put: (FormFieldValure for: value)." ].! !

!OrbeonFormReader methodsFor: 'recursive section' stamp: 'BrunoBB 08/16/2016 15:56'!
createGridsFor: aFormSectionInstance xmlSegment: xmlSubSegment
	"Creates and store FormGridInstance on the form instance <aFormSectionInstance> getting values from xml node <xmlSubSegment>"
	| sectionDefinition |

	sectionDefinition := aFormSectionInstance sectionDefinition.
	"iteration over grid definitions of the <FormSectionDefinition> sectionDefinition "
	sectionDefinition gridNames do: [:gridName | | gridDefinition grid xmlGrid |

		gridDefinition := sectionDefinition getGridDefinitionNamed: gridName. "get the actual grid definition"
		grid := FormGridInstance name: gridName definition: gridDefinition. "create the grid instance"
		"iterate over each grid iterations. Each Grid Iteration is a Dictionary where the key is the field name and the value is the field value "
		xmlGrid := xmlSubSegment elementsAt: gridName do: [:xmlGridIteration | | iteration |
			iteration := Dictionary new.
			"iterate over fields of the current grid iteration to get field names and values"
			gridDefinition fieldDefinitions do: [:fieldDefinition | | fieldValue |
				fieldValue := xmlGridIteration elementAt: fieldDefinition name.
				fieldValue := (fieldValue isNil or: [fieldValue hasChildren not]) ifTrue: [''] ifFalse: [self getValueFrom: fieldValue].
				iteration at: fieldDefinition name put: fieldValue].

			grid addIteration: iteration]. "store the iteration in the grid instance"

		aFormSectionInstance addGrid: grid.]. "store the grid instance in the section instance"! !

!OrbeonFormReader methodsFor: 'recursive section' stamp: 'BrunoBB 08/17/2016 09:12'!
createInnerStructureOf: aFormSectionInstance basedOn: aFormSectionDefinition xmlSegment: xmlSegment
	"The receiver create the inner structure of section <aFormSectionInstance> with definition <aFormSectionDefinition> taking data from xml node <xmlSegment>"

	"create Fields Instances and Grid Instances"
	self createSimpleComponentsOf: aFormSectionInstance basedOn: aFormSectionDefinition xmlSegment: xmlSegment.

	"create the recursive subsections of the receiver"
	aFormSectionDefinition subSections do: [:subSectionDef | | sections |
		sections := (xmlSegment xPath: subSectionDef getXpathForNodeContent)
								ifNotEmpty: [:collection | collection] 
								ifEmpty: [OrbeonInvalidXMLException signal: 'Invalid Subsections Structure'].

		sections do: [:xmlSection | | subSection |
			subSection := NormalSectionInstance	name: subSectionDef name definition: subSectionDef.
			aFormSectionInstance addSubSection: subSection.
			self createInnerStructureOf: subSection basedOn: subSectionDef xmlSegment: xmlSection].
	].! !

!OrbeonFormReader methodsFor: 'recursive section' stamp: 'BrunoBB 08/17/2016 09:06'!
createMainSectionInstancesWith: anOrbeonFormDefinition xmlSegment: xmlInstanceSegment
	"The receiver answer anOrderedCollection of FormSectionInstance based on form definition <anOrbeonFormDefinition> using the xml node <xmlInstanceSegment> (/form for main sections).
	Iterate over Main Sections definitions of <anOrbeonFormDefinition> and recursively creates the sub sections.
	1. Create field instances. 2- Create grid instances. 3- Recursively create sub sections"
	| sectionsInstances |

	sectionsInstances := OrderedCollection new.
	anOrbeonFormDefinition sectionDefinitions do:[:section | | xmlSubSegments |
		xmlSubSegments := section isRepeatable
			ifTrue: [OrderedCollection withAll: (xmlInstanceSegment xPath: ('/form/', section getXpathForNodeContent))]
			ifFalse: [ | subSegment | "elementAt: should not answer nil, ifNil --> there is a mismatch between the instance and it's definition"
						subSegment := xmlInstanceSegment 
													elementAt: section name 
													ifAbsent: [OrbeonInvalidXMLException signal: 'The Form definition structure does not match with the Form Instance structure']. 
						OrderedCollection with: subSegment].

		(xmlSubSegments select: [:each | each notNil]) do: [:xmlSegment | | sectionInst |
			sectionInst := NormalSectionInstance name: section name definition: section.
			sectionsInstances add: sectionInst. "store the section instance in the dictionary collector"
			"if <xmlSubSegment> is NIL then the definition <anOrbeonFormDefinition> does NOT MATCH with the xml <xmlInstanceSegment>.
			This could be the case when in Orbeon a field is added/removed/renamed and the definition is saved with the same version"
			"if <xmlSubSegment> is NOT NIL then creates the inner (recursive) elements of <sectionInst> as defined in <section>"
			self createInnerStructureOf: sectionInst basedOn: section xmlSegment: xmlSegment.
		].
	].
	^sectionsInstances "a dictionary of FormSectionInstance"! !

!OrbeonFormReader methodsFor: 'recursive section' stamp: 'BrunoBB 08/16/2016 05:33'!
createSectionsInstancesWith: anOrbeonFormDefinition
	"The receiver answer anOrderedCollection of FormSectionInstance. 
	The instances are created based in the definition parameter <anOrbeonFormDefinition> starting in the xml node 'form' "

	^self createMainSectionInstancesWith: anOrbeonFormDefinition xmlSegment: (xmlInstanceDocument elementAt: 'form')! !

!OrbeonFormReader methodsFor: 'recursive section' stamp: 'BrunoBB 08/16/2016 05:33'!
createSimpleComponentsOf: aFormSectionInstance basedOn: aFormSectionDefinition xmlSegment: xmlSegment
	"The receiver create the simple components of section <aFormSectionInstance> with definition <aFormSectionDefinition> taking data from xml node <xmlSegment>"

	"iterate over <section> fields names to get the values from <xmlSubSegment>"
	self createFieldsInstancesFor: aFormSectionInstance xmlSegment: xmlSegment.

	"create grid instances for the current section definition"
	aFormSectionDefinition gridDefinitions do: [:gridDefnition | self createGridsFor: aFormSectionInstance xmlSegment: xmlSegment].! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
formFieldsIn: xmlSection
	"The receiver get the fields of Orbeon Form Definition <xmlDefinition>. Components inside -repeat element- are NOT listed.
	The forms field are defined in the XForms after:    
		<xh:body> <fr:view>  tags"
	| fields collectableFields |

	collectableFields := self orbeonCollectableFieldsTags.
	fields := OrderedCollection new.
	xmlSection elementsAt: 'fr:grid' do: [:eachGrid | (eachGrid includesAttribute: 'repeat') 
		ifFalse: [collectableFields do:[:nodeName | eachGrid allElementsNamed: nodeName do: [:eachField | fields add: eachField]]]].
	^fields! !

!OrbeonFormReader methodsFor: 'orbeon fields' stamp: 'BrunoBB 08/16/2016 05:33'!
formFieldsLabels
	"The receiver answer a Dictionary with all fields label of all components of the form <xmlDocument>"
	| fieldsLabels labelNode |

	fieldsLabels := Dictionary new.
	labelNode := xmlDocument allElementsNamed: 'xf:instance' do: [:each | ((each includesAttribute: 'id') and: [(each attributeAt: 'id') = 'fr-form-resources']) ifTrue: [labelNode := each]]. "the node where labels are in '/xf:instance/resources/resource' "

	(labelNode findElementNamed: 'resource') elementsDo: [:each |  | originalLabel textNode |
		textNode := each elementAt: 'label' ifAbsent: [nil].
		originalLabel := textNode 
			ifNil: ['No Label'] 
			ifNotNil: [textNode rawContentString 
							ifEmpty: ['No Label'] 
							ifNotEmpty: [:value | value]
						].
		fieldsLabels at: each name put: originalLabel].

	^fieldsLabels! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
formGridComponentsIn: xmlSection
	"The receiver get the repeat component of Orbeon Form Definition <xmlDefinition>"
	| grids |

	grids := OrderedCollection new.
	xmlSection elementsAt: 'fr:grid' do: [:eachGrid | (eachGrid includesAttribute: 'repeat') 
		ifTrue: [grids add: eachGrid]].
	^grids! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
formGridFieldsIn: xmlGrid
	"The receiver get the fields of Orbeon Form Definition <xmlDefinition>, components inside -repeat element- are NOT listed.
	The forms field are defined in the XForms after:    
		<xh:body> <fr:view>  tags"
	| fields collectableFields|

	collectableFields := self orbeonCollectableFieldsTags.
	fields := OrderedCollection new.
	xmlGrid elementsDo: [:eachGrid | (eachGrid includesAttribute: 'repeat') 
		ifFalse: [collectableFields do:[:nodeName | eachGrid allElementsNamed: nodeName do: [:eachField | fields add: eachField]]]].
	^fields! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
formGridFieldsNameIn: xmlGrid
	"The receiver get the name od the fields of Orbeon Form Definition <xmlDefinition>, components inside -repeat element- are NOT listed "

	^(self formGridFieldsIn: xmlGrid) collect: [:each | self getNameFrom: each]! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
formMainSections
	| element |

	element := xmlDocument elementsAt: 'xh:html'.
	element := element isEmpty ifFalse: [element last] ifTrue: [^OrbeonInvalidXMLException signal: 'Bad format - No <xh:html> element'].

	element := element elementsAt: 'xh:body'.
	element := element isEmpty ifFalse: [element last] ifTrue: [^OrbeonInvalidXMLException signal: 'Bad format - No <xh:body> element'].

	element := element elementsAt: 'fr:view'.
	element := element isEmpty ifFalse: [element last] ifTrue: [^OrbeonInvalidXMLException signal: 'Bad format - No <fr:view> element'].

	element := element elementsAt: 'fr:body'.
	element := element isEmpty ifFalse: [element last] ifTrue: [^OrbeonInvalidXMLException signal: 'Bad format - No <fr:body> element'].

	^(element elementsAt: 'fr:section') asOrderedCollection! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
formMainSectionsRepeatable: aBoolean

	aBoolean ifFalse: [^self formMainSections reject: [:eachSection | (eachSection includesAttribute: 'repeat')]].

	^self formMainSections select: [:eachSection | (eachSection includesAttribute: 'repeat')]! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
formSubSectionsOf: xmlElement
	| subSections |

	subSections := OrderedCollection new.
	xmlElement elementsAt: 'fr:section' do: [:eachSection | subSections add: eachSection].
	^subSections! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
getAllFormFieldsIn: xmlNode
	"The receiver get the fields of Orbeon Form Definition <xmlDefinition>.
	The forms field are defined in the XForms after:    
		<xh:body> <fr:view>  tags"
	| fields collectableFields |

	collectableFields := self orbeonCollectableFieldsTags.
	fields := OrderedCollection new.
	collectableFields do:[:nodeName | xmlNode allElementsNamed: nodeName do: [:eachField | fields add: eachField]].

	^fields! !

!OrbeonFormReader methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
getApplicationName

	^(xmlDocument findElementNamed: 'application-name')
		ifNotNil: [:value | value strings isEmpty ifTrue: [''] ifFalse: [value strings last]]
		ifNil: [''].! !

!OrbeonFormReader methodsFor: 'attachments' stamp: 'BrunoBB 08/16/2016 05:33'!
getAttachmentIdsFrom: anOrbeonFormDefinition
	"The receiver collect all attachment's id of the form definition <anOrbeonFormDefinition>"
	| ids modelNode formNode |

	modelNode := (xmlDocument findElementNamed: 'xf:model') ifNotNil: [:value | value] ifNil: [^#()].
	formNode := (modelNode findElementNamed: 'form') ifNotNil: [:value | value] ifNil: [^#()].
	ids := OrderedCollection new.
	"iterate over attachment fields #getAttachmentFields"
	anOrbeonFormDefinition getAttachmentFields do: [:each |
		(formNode findElementNamed: each name) 
		ifNotNil: [:attachNode | (self getValueFrom: attachNode) ifNotNil: [:value | ids add: value]].
	].
	"the file name format is: <definitionId>-<fileId>"
	^ids collect: [:each | | macIndex attachmentId |
			macIndex := each findString: 'mac='.
			macIndex ifNil: [^OrbeonInvalidXMLException signal: 'No "mac=" sentence in attachment value'].
			attachmentId := each copyFrom: macIndex + 4 to: each size.
			anOrbeonFormDefinition getId, '-', attachmentId]! !

!OrbeonFormReader methodsFor: 'attachments' stamp: 'BrunoBB 08/16/2016 05:33'!
getCustomAttachmentId
	"The receiver answer the id of the Custom PDF attachment. This custom PDF is used to create the PDF of the Form"
	| attachNode pdfNode |

	attachNode := xmlDocument findElementNamed: 'attachments'.
	attachNode ifNil: [^nil].
	pdfNode := attachNode findElementNamed: 'pdf'.
	pdfNode ifNil: [^nil].

	^pdfNode rawContentString 
		ifEmpty: [nil] 
		ifNotEmpty: [:value | (value subStrings: '=') last]! !

!OrbeonFormReader methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
getDescription

	^(xmlDocument findElementNamed: 'description')
		ifNotNil: [:value | value strings isEmpty ifTrue: [''] ifFalse: [value strings last]]
		ifNil: [''].! !

!OrbeonFormReader methodsFor: 'query xml' stamp: 'BrunoBB 08/16/2016 05:33'!
getFieldTypes
	"Answer aDictionary with all fields types. The <key> is the name of the field and the <value> is name of the type"
	| fieldTypes |

	fieldTypes := Dictionary new.
	xmlDocument allElementsNamed: 'xf:bind' do: [:eachBindNode | 
			fieldTypes at: (eachBindNode attributeAt: 'name') 
							put: (self getTypeFrom: eachBindNode)].
	^fieldTypes! !

!OrbeonFormReader methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
getFormName

	^(xmlDocument findElementNamed: 'form-name')
		ifNotNil: [:value | value strings isEmpty ifTrue: [''] ifFalse: [value strings last]]
		ifNil: [''].! !

!OrbeonFormReader methodsFor: 'orbeon fields' stamp: 'BrunoBB 08/16/2016 05:33'!
getNameFrom: xmlNode
	| fieldName |

	fieldName := xmlNode attributeAt: 'id'.
	(fieldName size <= 8) ifTrue: [^OrbeonControlSizeException signal: 'No "-control" word found in field''s name'].
	fieldName := fieldName allButLast: 8. "remove <-control> word from original id"
	^fieldName! !

!OrbeonFormReader methodsFor: 'permissions' stamp: 'BrunoBB 08/16/2016 05:33'!
getPermissions
	"The receiver answer aOrbeonPermissions instances with all permissions in <xmlDocument>"
	| permissions permissionNodes |

	permissions := OrbeonPermissions new.
	permissionNodes := xmlDocument allElementsNamed: 'permission'.

	permissionNodes do: [:node |
		node hasContentNodes " -anyone- role does not have content (is an empty node)"
		ifTrue: [self addPermissionTo: permissions fromString: (node attributeAt: 'operations') using: node]
		ifFalse: [self addAnyOnePermissionTo: permissions fromString: (node attributeAt: 'operations')] 
	].

	^permissions! !

!OrbeonFormReader methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
getTitle

	^(xmlDocument findElementNamed: 'title')
		ifNotNil: [:value | value strings isEmpty ifTrue: [''] ifFalse: [value strings last]]
		ifNil: [''].! !

!OrbeonFormReader methodsFor: 'query xml' stamp: 'BrunoBB 08/16/2016 05:33'!
getTypeFrom: eachBindNode
	"This method HAS TO BE IMPROVED. Get the type from the XML node <eachBindNode> "

	(eachBindNode includesAttribute: 'type') ifTrue: [^eachBindNode attributeAt: 'type']. "has the attribute -type-"

	^'xf:string'! !

!OrbeonFormReader methodsFor: 'output' stamp: 'BrunoBB 08/16/2016 05:33'!
getValueFrom: xmlNode
	"The receiver answer the content of the XMLElement <xmlNode>."

	^self getValueFrom: xmlNode ifNone: [nil]! !

!OrbeonFormReader methodsFor: 'output' stamp: 'BrunoBB 08/16/2016 05:33'!
getValueFrom: xmlNode ifNone: aBlock
	"The receiver answer the content of the XMLElement <xmlNode> if the content is empty then evaluate the zero argument block <aBlock>.
	This new implementation address the following issues: 
		#71 OrbeonPersistenceLayer.
		#230 BpmFlow"
	| content |

	content := xmlNode rawContentString.

	^content trimBlanks 
		ifEmpty: aBlock
		ifNotEmpty: [content]! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
hasSubSections: xmlElement
	"Answer whether or not <xmlElement> has sub sections"

	^(xmlElement elementsAt: 'fr:section') notEmpty! !

!OrbeonFormReader methodsFor: 'orbeon fields' stamp: 'BrunoBB 08/16/2016 05:33'!
isAttachmentField: fieldNode

	^((fieldNode localName = 'attachment') "regular attachments - the attachment can be removed from form instance"
			or: [fieldNode localName = 'image-attachment']) "image attachments - the attachment can be removed from form instance"
				or: [(fieldNode attributeAt: 'class' ifAbsent: [^false]) includesSubstring: 'fb-attachment'] "fixed image attachments - attachment can NOT be removed from form instance"! !

!OrbeonFormReader methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
isAvailable
	"Answer whether or not the form definition is published or not"

	^(xmlDocument findElementNamed: 'available')
		ifNotNil: [:value | value strings isEmpty ifTrue: [false] ifFalse: [(value strings last) = 'true']]
		ifNil: [false].! !

!OrbeonFormReader methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 05:33'!
isSingleton
	"Answer whether or not the form definition is a Singleton form or not"

	^(xmlDocument findElementNamed: 'singleton')
		ifNotNil: [:value | value strings isEmpty ifTrue: [false] ifFalse: [(value strings last) = 'true']]
		ifNil: [false].! !

!OrbeonFormReader methodsFor: 'orbeon fields' stamp: 'BrunoBB 08/16/2016 05:33'!
orbeonCollectableFieldsTags
	"Answer a collection of Orbeon tags designed to represent components in a XForms definition.
	At render time Orbeon transform each of these tags to an HMTL tag."

	^#('xf:input' 'xf:select1' 'xf:textarea' 'fr:number' 'xf:select' 'fr:dropdown-date' 'xf:secret' 'fr:tinymce' 'us:phone' 'fr:currency' 'fr:fields-date' 'fr:date-picker' 'fr:box-select' 'fr:databound-select1' 'fr:autocomplete' 'xf:output' 'fr:image-attachment' 'fr:attachment' 'fr:wpaint' 'fr:dropdown-select1' 'fr:us-phone')! !

!OrbeonFormReader methodsFor: 'parsing' stamp: 'BrunoBB 08/16/2016 05:33'!
parseXMLDefinition
	"Creates a new instance of the receiver for a file named <aString>"

	xmlDocument := XMLDOMParser parse: xmlDefinition.
	^xmlDocument! !

!OrbeonFormReader methodsFor: 'parsing' stamp: 'BrunoBB 08/16/2016 05:33'!
parseXMLInstance: orbeonXMLInstance
	"Parse the xml orbeon instnace of the receiver "

	xmlInstanceDocument := XMLDOMParser parse: orbeonXMLInstance.
	^xmlInstanceDocument! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
searchFormFieldsIn: xmlSection
	"The receiver answer a collection with the field definitions in Form Section <xmlSection> that are field search"

	^(self getAllFormFieldsIn: xmlSection) select: [:each | ((each attributeAt: 'class') findString: 'fr-search' startingAt: 1) ~= 0]! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
searchFormFieldsNameIn: xmlSection
	"The receiver get the name od the fields of Orbeon Form Definition <xmlDefinition>, components inside -repeat element- are NOT listed "

	^(self searchFormFieldsIn: xmlSection) collect: [:each | self getNameFrom: each]! !

!OrbeonFormReader methodsFor: 'parsing' stamp: 'BrunoBB 08/16/2016 05:33'!
setXmlInstanceDocument: xmlOrbeonInstance
	"Parse the XML Form instance from xml string <xmlOrbeonInstance>"

	"self parseXMLInstance: (self workAroundBugForXML: xmlOrbeonInstance)"

	self parseXMLInstance: xmlOrbeonInstance! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
summaryFormFieldsIn: xmlSection
	"The receiver get the fields of Orbeon Form Definition <xmlDefinition>, components inside -repeat element- are NOT listed "
	
	^(self getAllFormFieldsIn: xmlSection) select: [:each | ((each attributeAt: 'class') findString:  'fr-summary' startingAt: 1) ~= 0]! !

!OrbeonFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 08/16/2016 05:33'!
summaryFormFieldsNameIn: xmlSection
	"The receiver get the name od the fields of Orbeon Form Definition <xmlDefinition>, components inside -repeat element- are NOT listed "

	^(self summaryFormFieldsIn: xmlSection) collect: [:each | self getNameFrom: each]! !

!OrbeonFormReader methodsFor: 'Accessing' stamp: 'BrunoBB 08/16/2016 05:33'!
xmlDefinition

   "Return the value of the instance variable 'xmlDefinition'."
   ^xmlDefinition! !

!OrbeonFormReader methodsFor: 'bug' stamp: 'BrunoBB 08/16/2016 05:33'!
xmlDefinition: newValue
   "Modify the value of the instance variable 'xmlDefinition'."

	"xmlDefinition := self workAroundBugForXML: newValue."

	xmlDefinition := newValue! !

!OrbeonFormReader methodsFor: 'Accessing' stamp: 'BrunoBB 08/16/2016 05:33'!
xmlDocument

   "Return the value of the instance variable 'xmlDocument'."
   ^xmlDocument! !

!OrbeonFormReader methodsFor: 'Updating' stamp: 'BrunoBB 08/16/2016 05:33'!
xmlDocument: newValue

   "Modify the value of the instance variable 'xmlDocument'."
   xmlDocument := newValue! !

!OrbeonFormReader methodsFor: 'Accessing' stamp: 'BrunoBB 08/16/2016 05:33'!
xmlInstanceDocument

   "Return the value of the instance variable 'xmlInstanceDocument'."
   ^xmlInstanceDocument! !

!OrbeonFormReader methodsFor: 'Updating' stamp: 'BrunoBB 08/16/2016 05:33'!
xmlInstanceDocument: newValue

   "Modify the value of the instance variable 'xmlInstanceDocument'."
   xmlInstanceDocument := newValue! !

OrbeonFormReader subclass: #OrbeonXPathFormReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonXPathFormReader methodsFor: 'permissions' stamp: 'BrunoBB 07/15/2016 06:10'!
addPermissionTo: permissions fromString: stringOperations using: permissionNode
	"The receiver add permission to OrbeonPermissions <permissions> to roles in node <permissionNode> with grant <stringOperations>"
	| subNode |

	"subNode := permissionNode nodes first."
	subNode := (permissionNode xPath: 'user-role') 
						ifNotEmpty: [:col | col last]
						ifEmpty: [(permissionNode xPath: 'group-member') 
										ifNotEmpty: [:col | col last] 
										ifEmpty: [(permissionNode xPath: 'owner') ifNotEmpty: [:col | col last] ifEmpty: [^self]]]. 
	
	(stringOperations subStrings: ' ') do: [:oper | 
		(subNode name = 'user-role')
		ifFalse: [permissions add: oper to: subNode name] "for group-members and owner"
		"for specific roles. -any-of- can have more than one role"
		ifTrue: [((subNode attributeAt: 'any-of') subStrings: ' ') do: [:rol | permissions add: oper to: rol]] 
	].! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/17/2016 10:33'!
formFieldsIn: xmlSection
	"The receiver answer a collection with all fields in Section <xmlSection> excluding Repeatable Sections.
	Fields inside Repeatable GRIDS [fr:grid] are also ingnored"
	| fields |

	fields := OrderedCollection new.
	(xmlSection xPath: 'fr:grid[not(@repeat)]') "does NOT contain attribute <repeat>"
		do: [:eachGrid | 
			(eachGrid xPath: 'xh:tr/xh:td/*[starts-with(name(), ''fr:'')]') addAllTo: fields.
			(eachGrid xPath: 'xh:tr/xh:td/*[starts-with(name(), ''us:'')]') addAllTo: fields.
			(eachGrid xPath: 'xh:tr/xh:td/*[starts-with(name(), ''xf:'')]') addAllTo: fields.].

	^fields! !

!OrbeonXPathFormReader methodsFor: 'orbeon fields' stamp: 'BrunoBB 08/01/2016 10:21'!
formFieldsLabels
	"The receiver answer a Dictionary with all fields label of all components of the form <xmlDocument>"
	| fieldsLabels |

	fieldsLabels := Dictionary new.

	(xmlDocument xPath: '//xf:instance[@id=''fr-form-resources'']/resources//resource/*') do: [:each |  | originalLabel textNode |
		textNode := each elementAt: 'label' ifAbsent: [nil].

		originalLabel := textNode 
			ifNil: ['No Label'] 
			ifNotNil: [textNode rawContentString 
							ifEmpty: ['No Label'] 
							ifNotEmpty: [:value | value].
						].
		fieldsLabels at: each name put: originalLabel].

	^fieldsLabels! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/17/2016 12:13'!
formGridComponentsIn: xmlSection
	"The receiver get the repeat component of Orbeon Form Definition <xmlDefinition>"
	| grids |

	grids := OrderedCollection new.

	"includes attribute -repeat-"
	(xmlSection xPath: 'fr:grid[@repeat]') addAllTo: grids. 
	
	^grids! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/16/2016 11:03'!
formGridFieldsIn: xmlGrid
	"The receiver get the fields of Orbeon Form Definition <xmlDefinition>, components inside -repeat element- are NOT listed.
	The forms field are defined in the XForms after:    
		<xh:body> <fr:view>  tags"
	| fields |

	fields := OrderedCollection new.

	(xmlGrid xPath: 'xh:tr/xh:td/*[starts-with(name(), ''fr:'')]') addAllTo: fields.
	(xmlGrid xPath: 'xh:tr/xh:td/*[starts-with(name(), ''us:'')]') addAllTo: fields.
	(xmlGrid xPath: 'xh:tr/xh:td/*[starts-with(name(), ''xf:'')]') addAllTo: fields.

	^fields! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/16/2016 11:12'!
formMainSections

	"check for the existence of the structure"
	(xmlDocument xPath: 'boolean(xh:html/xh:body/fr:view/fr:body)') 
	ifFalse: [^OrbeonInvalidXMLException signal: 'Bad format - No <xh:html> element'].

	^(xmlDocument xPath: 'xh:html/xh:body/fr:view/fr:body/fr:section') asOrderedCollection! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/16/2016 12:28'!
formSubSectionsOf: xmlElement
	"Answer a collection with the subsection of <xmlElement>"

	^(xmlElement xPath: 'fr:section') asOrderedCollection! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/17/2016 10:47'!
getAllFormFieldsIn: xmlNode
	"The receiver get the fields of Orbeon Form Definition <xmlDefinition>.
	The forms field are defined in the XForms after:    
		<xh:body> <fr:view>  tags"
	| fields |

	fields := OrderedCollection new.
	(xmlNode xPath: './/fr:grid/xh:tr/xh:td/*[starts-with(name(), ''fr:'')]') addAllTo: fields.
	(xmlNode xPath: './/fr:grid/xh:tr/xh:td/*[starts-with(name(), ''us:'')]') addAllTo: fields.
	(xmlNode xPath: './/fr:grid/xh:tr/xh:td/*[starts-with(name(), ''xf:'')]') addAllTo: fields.

	^fields! !

!OrbeonXPathFormReader methodsFor: 'other' stamp: 'BrunoBB 03/16/2016 12:44'!
getApplicationName

	^(xmlDocument xPath: '//metadata/application-name')
		ifEmpty: [OrbeonInvalidXMLException signal: 'Application node missing'] 
		ifNotEmpty: [:collection | 
			collection first strings isEmpty 
				ifTrue: [''] 
				ifFalse: [collection first strings last]]! !

!OrbeonXPathFormReader methodsFor: 'attachments' stamp: 'BrunoBB 07/15/2016 08:42'!
getAttachmentIdsFrom: anOrbeonFormDefinition
	"The receiver collect all attachment's id of the form definition <anOrbeonFormDefinition>.
	Each attachment has an associated file. 
	Empty Attachment field definition (without file in the Form Definition) are NOT listed in this method"
	| ids nativeOrbeonIds nodes formNode |

	nodes := xmlDocument xPath: '//xf:instance/form'.
	(nodes size = 1) ifFalse: [^OrbeonInvalidXMLException signal: 'Only one [form] node is allowed'].
	formNode  := nodes last.
	nativeOrbeonIds := Dictionary new.
	ids := Dictionary new.
	"iterate over attachment fields #getAttachmentFields"
	anOrbeonFormDefinition getAttachmentFields do: [:each |
		(formNode xPath: './/',each name) 
		ifNotEmpty: [:collection | | attachNode |
			attachNode := collection last.
			(self getValueFrom: attachNode) ifNotNil: [:value | nativeOrbeonIds at: each name put: value]]
	].
	"the file name format is: <definitionId>-<fileId>"
	nativeOrbeonIds keysAndValuesDo: [:key :value |  | macIndex attachmentId |
		macIndex := value findString: 'mac='.
		macIndex ifNil: [^OrbeonInvalidXMLException signal: 'No "mac=" sentence in attachment value'].
		attachmentId := value copyFrom: macIndex + 4 to: value size.
		ids at: key put: (anOrbeonFormDefinition getId, '-', attachmentId)].

	^ids! !

!OrbeonXPathFormReader methodsFor: 'attachments' stamp: 'BrunoBB 03/19/2016 13:03'!
getAttachmentsFields

	^(xmlDocument xPath: '//fr:view//fr:attachment') , 
		(xmlDocument xPath: '//fr:view//fr:image-attachment'),
		(xmlDocument xPath: '//fr:view//contains[@class, ''fb-attachment'']')! !

!OrbeonXPathFormReader methodsFor: 'attachments' stamp: 'BrunoBB 08/01/2016 10:16'!
getCustomAttachmentId
	"The receiver answer the id of the Custom PDF attachment. This custom PDF is used to create the PDF of the Form"
	| pdfQueryNode pdfNode |

	pdfQueryNode := xmlDocument xPath: '//xf:instance/attachments/pdf'.
	pdfQueryNode ifEmpty: [^nil].

	pdfNode := pdfQueryNode last.

	^pdfNode rawContentString 
		ifEmpty: [nil]
		ifNotEmpty: [:value | (value subStrings: '=') last]! !

!OrbeonXPathFormReader methodsFor: 'custom iteration name' stamp: 'BrunoBB 08/16/2016 05:39'!
getCustomIterationNameFor: sectionName bindName: sectionNameBindName
	"Answer the -custom iteration name- for the section bind element named <sectionNameBindName>"
	| bindNode iterationName |

	bindNode := (xmlDocument xPath: '//xf:bind[@id=''', sectionNameBindName, ''']/xf:bind') 
		ifEmpty: [^nil] 
		ifNotEmpty: [:collection | (collection size = 1) ifTrue: [collection last] ifFalse:[^nil]].

	iterationName := bindNode attributeAt: 'name'.
	
	^((sectionName, '-iteration') = iterationName) 
			ifFalse: [iterationName]
			ifTrue: [nil]! !

!OrbeonXPathFormReader methodsFor: 'other' stamp: 'BrunoBB 03/16/2016 12:44'!
getDescription

	^(xmlDocument xPath: '//metadata/description')
		ifEmpty: [OrbeonInvalidXMLException signal: 'Description node missing'] 
		ifNotEmpty: [:collection | 
			collection first strings isEmpty 
				ifTrue: [''] 
				ifFalse: [collection first strings last]]! !

!OrbeonXPathFormReader methodsFor: 'query xml' stamp: 'BrunoBB 03/16/2016 15:10'!
getFieldTypes
	"Answer aDictionary with all fields types. The <key> is the name of the field and the <value> is name of the type"
	| fieldTypes |

	fieldTypes := Dictionary new.

	(xmlDocument xPath: '//xf:bind') do: [:eachBindNode |  
		fieldTypes at: (eachBindNode attributeAt: 'name') put: (self getTypeFrom: eachBindNode)].

	^fieldTypes.! !

!OrbeonXPathFormReader methodsFor: 'other' stamp: 'BrunoBB 03/16/2016 12:45'!
getFormName

	^(xmlDocument xPath: '//metadata/form-name')
		ifEmpty: [OrbeonInvalidXMLException signal: 'Form-Name node missing'] 
		ifNotEmpty: [:collection | 
			collection first strings isEmpty 
				ifTrue: [''] 
				ifFalse: [collection first strings last]]! !

!OrbeonXPathFormReader methodsFor: 'query xml' stamp: 'BrunoBB 07/15/2016 05:55'!
getPermissions
	"The receiver answer aOrbeonPermissions instances with all permissions in <xmlDocument>"
	| permissions permissionNodes |

	permissions := OrbeonPermissions new.
	permissionNodes := xmlDocument xPath: '//metadata/permissions/permission'.

	permissionNodes do: [:node |
		node hasContentNodes " -anyone- role does not have content"
		ifTrue: [self addPermissionTo: permissions fromString: (node attributeAt: 'operations') using: node]
		ifFalse: [self addAnyOnePermissionTo: permissions fromString: (node attributeAt: 'operations')] 
	].

	^permissions! !

!OrbeonXPathFormReader methodsFor: 'other' stamp: 'BrunoBB 03/16/2016 12:45'!
getTitle

	^(xmlDocument xPath: '//metadata/title')
		ifEmpty: [OrbeonInvalidXMLException signal: 'Title node missing'] 
		ifNotEmpty: [:collection | 
			collection first strings isEmpty 
				ifTrue: [''] 
				ifFalse: [collection first strings last]]! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/16/2016 12:28'!
hasSubSections: xmlElement
	"Answer whether or not <xmlElement> has sub sections"

	^(self formSubSectionsOf: xmlElement) notEmpty! !

!OrbeonXPathFormReader methodsFor: 'attachments' stamp: 'BrunoBB 03/19/2016 14:24'!
isAttachmentField: fieldNode
	"'fr:attachment' - (regular attachments) the attachment can be removed from form instance.
	'fr:image-attachment' - (image attachments) - the attachment can be removed from form instance.
	'fr:wpaint' - (image annotation) - the attachment is an image to make annotations.
	class= 'fb-attachment' - (fixed image attachments) - attachment can NOT be removed from form instance"

	^((fieldNode xPath: 'name()') = 'fr:attachment')
		or:[((fieldNode xPath: 'name()') = 'fr:image-attachment')
			or:[((fieldNode xPath: 'name()') = 'fr:wpaint')
				or: [(fieldNode xPath: 'element[contains(@class, ''fb-attachment'')]') notEmpty]]]! !

!OrbeonXPathFormReader methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:33'!
isAvailable
	"Answer whether or not the form definition is published or not"

	^(xmlDocument xPath: '//metadata/available')
		ifEmpty: [false] 
		ifNotEmpty: [:collection | 
			collection first strings isEmpty 
				ifTrue: [false] 
				ifFalse: [(collection first strings last = 'true')]]! !

!OrbeonXPathFormReader methodsFor: 'other' stamp: 'BrunoBB 05/01/2016 09:13'!
isSingleton
	"Answer whether or not the form definition is a Singleton form or not"

	^(xmlDocument xPath: '//metadata/singleton')
		ifEmpty: [false] 
		ifNotEmpty: [:collection | 
			collection first strings isEmpty 
				ifTrue: [false] 
				ifFalse: [(collection first strings last = 'true')]]! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/17/2016 14:16'!
searchFormFieldsIn: xmlSection
	"The receiver answer a collection with the field definitions in Form Section <xmlSection> that are field search"

	^(xmlSection xPath: './/fr:grid/xh:tr/xh:td/*[contains(@class,''fr-search'')]') asOrderedCollection.! !

!OrbeonXPathFormReader methodsFor: 'orbeon sections' stamp: 'BrunoBB 03/17/2016 14:17'!
summaryFormFieldsIn: xmlSection
	"The receiver get the fields of Orbeon Form Definition <xmlDefinition>, components inside -repeat element- are NOT listed "
	
	^(xmlSection xPath: './/fr:grid/xh:tr/xh:td/*[contains(@class,''fr-summary'')]') asOrderedCollection.! !

Object subclass: #OrbeonHandler
	instanceVariableNames: 'mySqlReader formReader instanceDefinitions synchronizations orbeonIP'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonHandler class methodsFor: 'other' stamp: 'BrunoBB 10/31/2014 16:27'!
default

	Default ifNil: [Default := super new initialize].

	^Default! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 02/03/2016 08:58'!
configuration

	^self getOrbeonDatabaseConfiguration default! !

!OrbeonHandler methodsFor: 'mysql' stamp: 'BrunoBB 10/31/2014 16:27'!
defaultMySqlClass

	^Orbeon46MySQL

	"^OrbeonMySQLReader"! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 02/04/2016 12:42'!
execute: aString
	"Execute the command <aString> in the Linux Server whereas GemStone/S is installed"
	| result strippedResult |

	result := System performOnServer: aString.

	(result includesSubString: 'Warning: Using a password on the command line interface can be insecure.')
	ifFalse: [^result].

	strippedResult := result allButFirst: 72. "we remove this trash string of the new mysql client (5.5)"

	^strippedResult! !

!OrbeonHandler methodsFor: 'attachments' stamp: 'BrunoBB 10/16/2015 19:37'!
fileoutDefinitionAttachmentOn: path defintionId: definitionId fileId: fileId version: version
	| result |

	result := self execute: (mySqlReader sqlDumpFileDefinitionAttachmentOn: path definitionId: definitionId fileId: fileId version: version).

	^result! !

!OrbeonHandler methodsFor: 'attachments' stamp: 'BrunoBB 10/16/2015 20:21'!
fileoutInstanceAttachmentOn: path defintionId: definitionId fileId: fileId version: version
	| result |

	result := self execute: (mySqlReader sqlDumpFileInstanceAttachmentOn: path definitionId: definitionId fileId: fileId version: version).

	^result! !

!OrbeonHandler methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2013 08:39'!
formReader

   "Return the value of the instance variable 'formReader'."
   ^formReader! !

!OrbeonHandler methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2013 08:39'!
formReader: newValue

   "Modify the value of the instance variable 'formReader'."
   formReader := newValue! !

!OrbeonHandler methodsFor: 'mapping' stamp: 'BrunoBB 04/28/2016 10:05'!
generateFormDefinition: formName app: appName from: dbArray
	"Create a new definition of the Form based on <dbArray>"
	| version formDefinition |

	version := Integer fromString: (dbArray at: 7).

	formDefinition := OrbeonPublishedFormDefinition app: appName form: formName.

	^formDefinition updateFrom: dbArray;
		version: version;
		xmlString: ((dbArray at: 8) copyReplaceAll: '\n' with: Character lf); "-\n- is inserted by Bash "
		yourself! !

!OrbeonHandler methodsFor: 'mapping' stamp: 'BrunoBB 04/28/2016 10:53'!
generateFormInstanceDefinition: formName app: appName from: aDBString
	"Create a new instance of the Form based on <dbArray>"
	| instance resultArray |

	resultArray := aDBString subStrings: String tab.
	instance := OrbeonSavedFormDefinition new.
	^instance setApplication: appName;
		formName: formName;
		updateFrom: resultArray;
		xmlString: ((resultArray at: 7) copyReplaceAll: '\n' with: Character lf); "-\n- is inserted by Bash "
		"setApplicationAndFormName;"
		setMetadata;
		yourself.! !

!OrbeonHandler methodsFor: 'mapping' stamp: 'BrunoBB 04/29/2016 14:49'!
generateFormInstanceOf: anOrbeonFormDefinition from: dbArray
	"Create a new instance of the Form based on <dbArray>"
	| formId formInstance |

	formId := dbArray at: 6 ifAbsent: [nil]. "the position <6> has the value of ID in the database, check <Orbeon46MySQL>"
	formInstance := anOrbeonFormDefinition "if the instance already exist we use it if NOT --> create a newone"
		getInstanceWithId: formId 
		ifNone: [OrbeonFormInstance newFor: anOrbeonFormDefinition].

	^formInstance
			updateFrom: dbArray;
			xmlString: ((dbArray at: 8) copyReplaceAll: '\n' with: Character lf); "-\n- is inserted by Bash "
			yourself! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 03/29/2015 13:40'!
getAllApplicationDefinitionVersionFromDB

	^(self execute: mySqlReader sqlGetAllApplicationWithDefinitionVersions) subStrings: String lf! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 12/20/2013 08:39'!
getAllApplicationsName
	"Answer a collection of string with all application name's of Orbeon Application"

	^self execute: mySqlReader sqlGetAllApplications! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:19'!
getAllDefinitionsInEdition
	"Answer a collection of FormDefinitions that are in edition state"

	^OrbeonPublishedFormDefinition cache select: [:e | e isInEdition]! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 04/28/2016 10:46'!
getAllDocumentIdsForDefinition: anOrbeonFormDefinition

	"^self execute: (mySqlReader sqlGetAllDocumentIdsFrom: anOrbeonFormDefinition form of: anOrbeonFormDefinition app)"

	^self execute: (mySqlReader sqlGetAllDocumentIdsFrom: anOrbeonFormDefinition formName of: anOrbeonFormDefinition getApplicationName version: anOrbeonFormDefinition version)! !

!OrbeonHandler methodsFor: 'mysql to gs' stamp: 'BrunoBB 03/10/2014 07:06'!
getAllEditionVersion
	"Get all OrbeonDefinitionForm's that are stored in orbeon_data table but are OrbeonDefinitions.
	When a XForms is being edited inside Orbeon Builder and then saved. A row is added to orbeon_data.
	Only when the form is published an entry to orbeon_definition is added.
	Answer OrbeonDefinitions that are being edited but not published yet (from orbeon_data)"
	| instancesString dbString instDef |

	instDef := OrderedCollection new.
	instancesString := self execute: mySqlReader sqlGetAllInstanceDefinitions.
	instancesString isEmpty ifTrue: [^instanceDefinitions].
	dbString := instancesString subStrings: String lf.
	dbString do: [:eachDbLine |
			instDef add: (self generateFormInstanceDefinition: 'builder' app: 'orbeon' from: eachDbLine).].
	
	"instanceDefinitions do: [:each | each addToCache]."

	^instDef! !

!OrbeonHandler methodsFor: 'mysql to gs' stamp: 'BrunoBB 01/21/2014 05:45'!
getAllFormsDefinitionNameOf: appName
	"Answer a collection of <String> with all forms definition of Orbeon Application <appName> stored in MySQL"

	^self execute: (mySqlReader sqlGetAllFormsDefinitionOf: appName)! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 02/08/2016 12:29'!
getAllOrbeonApplications
	"Answer a collection of string with all application name's of Orbeon Application"
	| dbApplicationsNames |

	dbApplicationsNames := (self getAllApplicationsName subStrings: String lf) "a collection of strings with App names"
												reject: [:each | self getOrbeonApplicationClass hasApplicationNamed: each]. "we filter existing app"

	^(dbApplicationsNames collect: [:eachName | self getOrbeonApplicationClass newNamed: eachName]) "we create a new app for each new row"
					asSortedCollection: [:a :b | a applicationName <= b applicationName]! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 10/14/2015 15:01'!
getAllVersionsNumberOf: formName in: app
	"Answer a collection with all version number of form definition named <formName> in application named <app>"

	^((self execute: (mySqlReader selectAllVersionOf: formName in: app)) subStrings: String lf) collect: [:each | Number fromString: each]! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 04/01/2015 08:01'!
getApplicationDefinitionVersionOf: appName

	^(self execute: (mySqlReader sqlGetAllDefinitionVersionsOf: appName)) subStrings: String lf! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 10/15/2015 12:34'!
getDBStringForInstanceDefinitionWithId: id

	^self execute: (mySqlReader sqlGetInstanceDefinitionWithId: id)! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 10/15/2015 11:24'!
getDBStringForInstanceDefinitionWithId: id version: version

	^self execute: (mySqlReader sqlGetInstanceDefinitionWithId: id version: version)! !

!OrbeonHandler methodsFor: 'attachments' stamp: 'BrunoBB 10/14/2015 17:26'!
getDefinitionAttachmentContentFor: fileId version: version

	^self execute: (mySqlReader sqlGetDefinitionAttachmentContentFor: fileId version: version)! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 11/19/2014 03:27'!
getFormWithId: documentId
	"Answer the last stored version of the form with id <documentId>"

	^self execute: (mySqlReader sqlGetFormWithId: documentId)! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 04/28/2016 11:01'!
getIdForDefinition: anOrbeonFormDefinition
	"The receiver get the [id] of definition <anOrbeonFormDefinition> from the database if it is the first version otherwise get the [id] from the already stored form definition"
	| sqlResult query |

	(anOrbeonFormDefinition version = 1) "no previous version --> we get the [id] from the database (must be at least one saved definition)"
	ifTrue: [query := mySqlReader sqlGetInstanceDefinitionTimeStamp: anOrbeonFormDefinition modifiedTime.
				sqlResult := self execute: query.
				sqlResult isEmpty ifTrue: [^NoSqlDataRetrievedException signal: 'No data found in [orbeon_form_data] table for: ', anOrbeonFormDefinition applicationName, '.', anOrbeonFormDefinition formName]. 
				^(sqlResult subStrings: String tab) second].

	"<anOrbeonFormDefinition> has previous version --> we get the [id] from stored definitions"

	^(self getOrbeonApplicationClass formDefinitionNamed: anOrbeonFormDefinition formName in: anOrbeonFormDefinition applicationName) id! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 15:16'!
getLastUpdateOf: formName in: appName version: version
	"Answer the last update time for form named <formNamed> in Orbeon applicaiton <appName> in MySql database"
	| sqlResult |

	sqlResult := self execute: (mySqlReader sqlFormDefinitionLastUpdateFor: appName form: formName version: version).
	
	sqlResult isEmpty ifTrue:[^self getOldTimeStamp].

	^(sqlResult subStrings: String lf) first! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 15:14'!
getOldTimeStamp

	^'1900-01-01 15:13:05'! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 10/31/2014 16:27'!
host

	^mySqlReader host! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 10/14/2015 15:06'!
importDataFromScratchOf: anOrbeonFormDefinition
	"The receiver import into Orbeon Application <anOrbeonApplication> a form named <formDefinitionName>.
	This method has NO SENDERS is intented to use manually (programmatically) to import a Form Definition and Instances from the Scratch.
	Optimized for definitions with a lof of form instances."
	| ids gsObjects |

	ids := self getAllDocumentIdsForDefinition: anOrbeonFormDefinition. "answer aString (filter by anOrbeonFormDefinition version)"

	gsObjects := (ids subStrings: String lf) collect: [:eachId | | sqlResult dbArray | 
			sqlResult := self getFormWithId: eachId.
			dbArray := sqlResult subStrings: String tab.
			self generateFormInstanceOf: anOrbeonFormDefinition from: dbArray.].

	gsObjects do: [:each | 
		each definition: anOrbeonFormDefinition;
			addToDefinition;
			processXML "process the XML and map it to a GsObject"].

	^gsObjects! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 05/03/2016 15:16'!
importDataOf: anOrbeonFormDefinition version: version
	"The receiver import into Orbeon Application <anOrbeonApplication> a form named <formDefinitionName>"
	| dbInstance gsObjects |

	dbInstance := self execute: (mySqlReader sqlGetAllFormsData: anOrbeonFormDefinition formName version: version of: anOrbeonFormDefinition getApplicationName after: self getOldTimeStamp).

	gsObjects := (dbInstance subStrings: String lf) collect: [:eachDBLine | | dbArray |
			dbArray := eachDBLine subStrings: String tab.
			self generateFormInstanceOf: anOrbeonFormDefinition from: dbArray.].

	gsObjects do: [:each | 
		each 
			definition: anOrbeonFormDefinition;
			addToDefinition;
			processXML "process the XML and map it to a GsObject"].

	^gsObjects! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 02/08/2016 11:16'!
importDataOfFormId: formId form: formName app: app
	"The receiver import into Orbeon Application <anOrbeonApplication> a form named <formDefinitionName> with id <formId>"
	| dbInstanceArray definition |

	dbInstanceArray := (self execute: (mySqlReader sqlFormDataWithId: formId)) subStrings: String tab.
	definition := self getOrbeonApplicationClass formDefinitionNamed: formName in: app.

	definition ifNil:[^OrbeonDefinitionNotFoundException signal: 'Definition named [', formName, '] for app [', app, '] not found'].

	^(self generateFormInstanceOf: definition from: dbInstanceArray)
		definition: definition;
		addToDefinition;
		updateFrom: dbInstanceArray; "update common database values - values common to all type of forms"
		processXML; "update specific form values - change for each type of form "
		yourself! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 05/18/2016 10:50'!
importFormDefinitionNamed: formDefinitionName version: version in: appName addToApp: aBoolean
	"The receiver import into Orbeon Application <anOrbeonApplication> a form named <formDefinitionName>"
	| dbDefnition gsDefinition dbArray application mySqlString |

	mySqlString := mySqlReader sqlGetFormDefinition: formDefinitionName version: version of: appName after: self getOldTimeStamp.

	dbDefnition := self execute: mySqlString.
	dbArray := dbDefnition trimLeadingSeparators subStrings: String tab.
	gsDefinition := self generateFormDefinition: formDefinitionName app: appName from: dbArray. "generate object from <dbArray>"
	"get the db id from the form definition"
	gsDefinition id: (self getIdForDefinition: gsDefinition). 
	"gsDefinition id: 'ec6132c917fa583f3d527ae4fd1d526ad7354a8c'."

	gsDefinition processXMLDefinition. "set the OrbeonFormDefinition with it's values -xml-"

	aBoolean ifTrue: [application := self getOrbeonApplicationClass applicationNamed: appName.
							application addPublishedDefinition: gsDefinition].

	^gsDefinition! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 04/02/2015 14:39'!
importNewApplications
	"The receiver perform the first and initial synchronization with the Orbeon Mysql database.
	Get all applications and synchronize all forms of each application"
	| formsToMigrate newInstances |

	"get all -triplet- (app, form, version) from Orbeon DataBase"
	formsToMigrate := self getAllApplicationDefinitionVersionFromDB.
	newInstances := OrderedCollection new.
	formsToMigrate do: [:eachAppFormVersion | | resultArray app form version |
		resultArray := eachAppFormVersion subStrings: String tab.
		app := resultArray at: 1. "app"
		form := resultArray at: 2. "form"
		version := resultArray at: 3. "version"
		"import and create the OrbeonFormDefinition into OrbeonApplication <application>"
		newInstances addAll: (self importNewDefinitionAndDataOf: form version: version application: app addDefToApp: true excludeExistingApp: true).
	].

	^newInstances! !

!OrbeonHandler methodsFor: 'db import' stamp: 'BrunoBB 07/22/2015 13:25'!
importNewApplicationsButNoData
	"The receiver perform the first and initial synchronization with the Orbeon Mysql database.
	Get all applications and synchronize all forms of each application"
	| formsToMigrate newInstances |

	"get all -triplet- (app, form, version) from Orbeon DataBase"
	formsToMigrate := self getAllApplicationDefinitionVersionFromDB.
	newInstances := OrderedCollection new.
	formsToMigrate do: [:eachAppFormVersion | | resultArray app form version |
		resultArray := eachAppFormVersion subStrings: String tab.
		app := resultArray at: 1. "app"
		form := resultArray at: 2. "form"
		version := resultArray at: 3. "version"
		"import and create the OrbeonFormDefinition into OrbeonApplication <application>"
		newInstances addAll: (self importNewDefinition: form version: version application: app addDefToApp: true excludeExistingApp: true).
	].

	^newInstances! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 02/08/2016 11:17'!
importNewDefinition: form version: version application: appName addDefToApp: addToAppBool excludeExistingApp: excludeBoolean
	"The receiver import new definition and data of OrbeonApplication named <appName> for form definition named <form> with version <version>.
	If <aBoolean> is <true> then exclude existing OrbeonApplication from the process"
	| application |

	(excludeBoolean and: [self getOrbeonApplicationClass hasApplicationNamed: appName]) ifTrue: [^#()].

	application := self getOrbeonApplicationClass 
							applicationNamed: appName 
							ifNone: [(self getOrbeonApplicationClass newNamed: appName) addToCache].
	"create the new OrbeonFormDefinition and add it to the new OrbeonApplication"
	^self importFormDefinitionNamed: form version: version in: appName addToApp: addToAppBool.! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 02/08/2016 11:17'!
importNewDefinitionAndDataOf: form version: version application: appName addDefToApp: addToAppBool excludeExistingApp: excludeBoolean
	"The receiver import new definition and data of OrbeonApplication named <appName> for form definition named <form> with version <version>.
	If <aBoolean> is <true> then exclude existing OrbeonApplication from the process"
	| newDefinition application |

	(excludeBoolean and: [self getOrbeonApplicationClass hasApplicationNamed: appName]) ifTrue: [^#()].

	application := self getOrbeonApplicationClass 
							applicationNamed: appName 
							ifNone: [(self getOrbeonApplicationClass newNamed: appName) addToCache].
	"create the new OrbeonFormDefinition and add it to the new OrbeonApplication"
	newDefinition := self importFormDefinitionNamed: form version: version in: appName addToApp: addToAppBool.

	"importe the new OrbeonFormInstances"
	^self importDataOf: newDefinition version: version.! !

!OrbeonHandler methodsFor: 'other' stamp: 'BrunoBB 03/19/2016 12:08'!
initialize

	super initialize.

	instanceDefinitions := OrderedCollection new.
	synchronizations := Dictionary new.

	mySqlReader := self defaultMySqlClass new.
	formReader := OrbeonXPathFormReader new.! !

!OrbeonHandler methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2013 08:39'!
instanceDefinitions

   "Return the value of the instance variable 'instanceDefinitions'."
   ^instanceDefinitions! !

!OrbeonHandler methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2013 08:39'!
instanceDefinitions: newValue

   "Modify the value of the instance variable 'instanceDefinitions'."
   instanceDefinitions := newValue! !

!OrbeonHandler methodsFor: 'migration script' stamp: 'BrunoBB 04/01/2015 08:30'!
migrateAll
	"Migrate all Definitions and Instances from Orbeon to GemStone/S. This method MUST called only one Time !!!!!!"
	| formsToMigrate |

	"get all -triplet- (app, form, version) from Orbeon DataBase"
	formsToMigrate := self getAllApplicationDefinitionVersionFromDB.

	formsToMigrate do: [:eachAppFormVersion | | resultArray app form version |
		resultArray := eachAppFormVersion subStrings: String tab.
		app := resultArray at: 1. "app"
		form := resultArray at: 2. "form"
		version := resultArray at: 3. "version"
		"import and create the OrbeonFormDefinition into OrbeonApplication <application>"
		self importNewDefinitionAndDataOf: form version: version application: app addDefToApp: true excludeExistingApp: false.
	]! !

!OrbeonHandler methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2013 08:39'!
mySqlReader

   "Return the value of the instance variable 'mySqlReader'."
   ^mySqlReader! !

!OrbeonHandler methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2013 08:39'!
mySqlReader: newValue

   "Modify the value of the instance variable 'mySqlReader'."
   mySqlReader := newValue! !

!OrbeonHandler methodsFor: 'Accessing' stamp: 'BrunoBB 10/31/2014 16:28'!
orbeonIP

   "Return the value of the instance variable 'orbeonIP'."
   ^orbeonIP! !

!OrbeonHandler methodsFor: 'Updating' stamp: 'BrunoBB 10/31/2014 16:28'!
orbeonIP: newValue

   "Modify the value of the instance variable 'orbeonIP'."
   orbeonIP := newValue! !

!OrbeonHandler methodsFor: 'mysql' stamp: 'BrunoBB 10/31/2014 17:08'!
setMySQLhost: host user: user password: password

	mySqlReader := self defaultMySqlClass new 
			host: host;
			user: user;
			password: password; 
			yourself.

	^self! !

!OrbeonHandler methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2013 08:39'!
synchronizations

   "Return the value of the instance variable 'synchronizations'."
   ^synchronizations! !

!OrbeonHandler methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2013 08:39'!
synchronizations: newValue

   "Modify the value of the instance variable 'synchronizations'."
   synchronizations := newValue! !

!OrbeonHandler methodsFor: 'update gs objects' stamp: 'BrunoBB 02/08/2016 12:40'!
updateApplication: anOrbeonApplication
	| formsToMigrate instances |

	instances := OrderedCollection new.
	"get all -triplet- (app, form, version) from Orbeon DataBase"
	formsToMigrate := self getApplicationDefinitionVersionOf: anOrbeonApplication applicationName.

	formsToMigrate do: [:eachAppFormVersion | 
		| resultArray app form version currentDefinition newDefinition comparator shouldCreateAnotherVersion dbArray |
		resultArray := eachAppFormVersion subStrings: String tab.
		app := resultArray at: 1. "app"
		form := resultArray at: 2. "form"
		version := resultArray at: 3. "version"
		instances addAll: (self updateFormDefinitionNamed: form app: app version: version).
	].

	^instances! !

!OrbeonHandler methodsFor: 'update gs objects' stamp: 'BrunoBB 02/08/2016 12:41'!
updateApplicationButNotDataOf: anOrbeonApplication
	| formsToMigrate instances |

	instances := OrderedCollection new.
	"get all -triplet- (app, form, version) from Orbeon DataBase"
	formsToMigrate := self getApplicationDefinitionVersionOf: anOrbeonApplication applicationName.

	formsToMigrate do: [:eachAppFormVersion | 
		| resultArray app form version currentDefinition newDefinition comparator shouldCreateAnotherVersion dbArray |
		resultArray := eachAppFormVersion subStrings: String tab.
		app := resultArray at: 1. "app"
		form := resultArray at: 2. "form"
		version := resultArray at: 3. "version"
		instances addAll: (self updateFormDefinitionNamedButNoData: form app: app version: version).
	].

	^instances! !

!OrbeonHandler methodsFor: 'update gs objects' stamp: 'BrunoBB 05/04/2016 13:54'!
updateFormDefinitionNamed: form app: app version: version
	"The receiver update or create the OrbeonFormDefinition named <form> with version <version> in OrbeonApplication named <app>.
	1. Get the current definition named <form>
	2. If NOT exist then it creates a new defnition and store it in OrbeonApplication named <app>
	3. If already exist then compare the current version with the new one and decide whether or not is necesary another definition.
	4. If is necesary then add the new definition to OrbeonApplication named <app>
	5. If NOT necesary then update the existing one"
	| currentDefinition newDefinition comparator shouldCreateAnotherVersion |

	currentDefinition := self getOrbeonApplicationClass getLastPublishedDefinitionNamed: form fromApplicationNamed: app withVersion: (Integer fromString: version) ifNone: [nil].

	^currentDefinition 
		ifNil: [newDefinition := self importFormDefinitionNamed: form version: version in: app addToApp: true.
			self importDataOf: newDefinition version: version]
		ifNotNil: [newDefinition := self importFormDefinitionNamed: form version: version in: app addToApp: false.
			comparator := FormDefinitionComparator formOne: currentDefinition formTwo: newDefinition.
			shouldCreateAnotherVersion := comparator shouldCreateAnotherVersion.
			shouldCreateAnotherVersion "is a different form"
				ifTrue: [(self getOrbeonApplicationClass applicationNamed: app) addPublishedDefinition: newDefinition.
							self importDataOf: newDefinition version: version] 
				ifFalse: [ | dbDefnition dbArray |
					dbDefnition := self execute: (mySqlReader sqlGetAllFormsDefinition: form of: app version: version after: self getOldTimeStamp).
					dbArray := dbDefnition subStrings: String tab.
					currentDefinition updateFrom: dbArray. "the form is almost the same --> we update the existing one"
					currentDefinition processXMLDefinition.
					self importDataOf: currentDefinition version: version].].! !

!OrbeonHandler methodsFor: 'update gs objects' stamp: 'BrunoBB 05/04/2016 13:55'!
updateFormDefinitionNamedButNoData: form app: app version: version
	"The receiver update or create the OrbeonFormDefinition named <form> with version <version> in OrbeonApplication named <app>.
	1. Get the current definition named <form>
	2. If NOT exist then it creates a new defnition and store it in OrbeonApplication named <app>
	3. If already exist then compare the current version with the new one and decide whether or not is necesary another definition.
	4. If is necesary then add the new definition to OrbeonApplication named <app>
	5. If NOT necesary then update the existing one"
	| currentDefinition newDefinition comparator shouldCreateAnotherVersion |

	currentDefinition := self getOrbeonApplicationClass getLastPublishedDefinitionNamed: form fromApplicationNamed: app withVersion: (Integer fromString: version) ifNone: [nil].

	^currentDefinition 
		ifNil: [newDefinition := self importFormDefinitionNamed: form version: version in: app addToApp: true.
			self importDataOf: newDefinition version: version]
		ifNotNil: [newDefinition := self importFormDefinitionNamed: form version: version in: app addToApp: false.
			comparator := FormDefinitionComparator formOne: currentDefinition formTwo: newDefinition.
			shouldCreateAnotherVersion := comparator shouldCreateAnotherVersion.
			shouldCreateAnotherVersion "is a different form"
				ifTrue: [(self getOrbeonApplicationClass applicationNamed: app) addPublishedDefinition: newDefinition] 
				ifFalse: [ | dbDefnition dbArray |
					dbDefnition := self execute: (mySqlReader sqlGetAllFormsDefinition: form of: app version: version after: self getOldTimeStamp).
					dbArray := dbDefnition subStrings: String tab.
					currentDefinition updateFrom: dbArray. "the form is almost the same --> we update the existing one"
					currentDefinition processXMLDefinition.].].! !

Object subclass: #OrbeonManagerRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonManagerRole subclass: #OrbeonManagerAdminRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonManagerAdminRole class methodsFor: 'other' stamp: 'BrunoBB 03/18/2015 16:14'!
roleName

	^'System Administrator'! !

!OrbeonManagerAdminRole methodsFor: 'other' stamp: 'BrunoBB 12/23/2013 06:25'!
isAdmin

	^true! !

!OrbeonManagerRole class methodsFor: 'other' stamp: 'BrunoBB 02/16/2014 13:18'!
allManagerRolNames

	^self subclasses collect: [:each | each roleName]! !

!OrbeonManagerRole class methodsFor: 'other' stamp: 'BrunoBB 02/16/2014 13:29'!
getClassForName: aString

	^self subclasses detect: [:each | each roleName = aString] ifNone: [OrbeonManagerUserRole]! !

!OrbeonManagerRole class methodsFor: 'other' stamp: 'BrunoBB 02/16/2014 13:16'!
roleName

	^self subclassResponsibility! !

!OrbeonManagerRole methodsFor: 'other' stamp: 'BrunoBB 12/23/2013 06:25'!
isAdmin

	^false! !

!OrbeonManagerRole methodsFor: 'other' stamp: 'BrunoBB 02/16/2014 13:21'!
roleName

	^self class roleName! !

OrbeonManagerRole subclass: #OrbeonManagerUserRole
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonManagerUserRole class methodsFor: 'other' stamp: 'BrunoBB 03/18/2015 16:14'!
roleName

	^'Application Administrator'! !

Object subclass: #OrbeonObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonObject subclass: #FormDefinitionRegistry
	instanceVariableNames: 'orbeonApplication currentVersions oldVersions savedDefinitions'
	classVariableNames: 'Cache'
	poolDictionaries: ''
	category: 'Orbeon'!

!FormDefinitionRegistry class methodsFor: 'other' stamp: 'BrunoBB 07/28/2016 10:35'!
newFor: anOrbeonApplication

	^super new initialize
		orbeonApplication: anOrbeonApplication;
		yourself! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
addCurrentVersion: newFormDefinition
	"The receiver set <newFormDefinition> as the current version. 
	First check if a definition with the same version exist. If not <newFormDefinition> is added to <currentVersions>.
	If exist then move the current version to <oldVersion> collection and set <newFormDefinition> as the current version in <currentVersions> collection"
	| oldVersion |

	oldVersion := self getDefinitionWithSameVersionAs: newFormDefinition.
	"there is no registered definition with the same version as <newFormDefinition>"
	oldVersion ifNil: [^currentVersions add: newFormDefinition]. 

	"the current version is removed from <currentVersions> and repleced with <newFormDefinition>"
	currentVersions remove: oldVersion. 
	oldVersions add: oldVersion. 
	^currentVersions add: newFormDefinition! !

!FormDefinitionRegistry methodsFor: 'saved definitions' stamp: 'BrunoBB 07/28/2016 10:35'!
addSavedDefinition: anOrbeonSavedFormDefinition
	"The receiver add the saved definition (not published yet) <anOrbeonSavedFormDefinition>.
	If there is a previous -instance definition- then remove it and add the new one <anOrbeonSavedFormDefinition>"
	| savedCollection oldSavedVersion |

	savedCollection := savedDefinitions at: anOrbeonSavedFormDefinition id ifAbsentPut: [OrderedCollection new].

	oldSavedVersion := savedCollection detect: [:each | each version = anOrbeonSavedFormDefinition version] 
														ifNone: [^savedCollection add: anOrbeonSavedFormDefinition].

	savedCollection remove: oldSavedVersion ifAbsent: [].

	^savedCollection add: anOrbeonSavedFormDefinition! !

!FormDefinitionRegistry methodsFor: 'other' stamp: 'BrunoBB 10/19/2016 07:05'!
applicationName

	^orbeonApplication applicationName! !

!FormDefinitionRegistry methodsFor: 'Accessing' stamp: 'BrunoBB 07/28/2016 10:35'!
currentVersions

   "Return the value of the instance variable 'currentVersions'."
   ^currentVersions! !

!FormDefinitionRegistry methodsFor: 'Updating' stamp: 'BrunoBB 07/28/2016 10:35'!
currentVersions: newValue

   "Modify the value of the instance variable 'currentVersions'."
   currentVersions := newValue! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getAllInstances
	""
	| all |

	all := OrderedCollection new.

	self getAllVersion do: [:each | all addAll: each instances asArray].

	^all! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getAllInstancesKeys
	"The receiver answer all keys of all form instances"
	| allKeys |

	allKeys := OrderedCollection new.
	(currentVersions, oldVersions) do: [:each | allKeys addAll: each instances keys].

	^allKeys! !

!FormDefinitionRegistry methodsFor: 'published def instances' stamp: 'BrunoBB 07/28/2016 10:35'!
getAllInstancesOf: version
	| currentInstances oldInstances |

	currentInstances := (self getDefinitionWithVersion: version) ifNil: [#()] ifNotNil: [:def | def instances].
	oldInstances := (self getOldDefinitionWithVersion: version) ifNil: [#()] ifNotNil: [:def | def instances].

	^currentInstances asArray, oldInstances asArray! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 09/21/2016 08:10'!
getAllInstancesSize
	"Answer the number of instances in all versions"
	| count |

	count := 0.

	self getAllVersion do: [:each | count := count + each instances size].

	^count! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getAllSearchFields
	"The receiver answer a collection with all search fields of all versions"
	| searchFields |

	searchFields := Set new.

	self getAllVersion do: [:each | searchFields addAll: each searchFields].

	^searchFields! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getAllVersion
	"Answer anArray with all OrbeonFormDefinition that the receiver holds (old + current) "

	^(oldVersions, currentVersions) asArray! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getAnyDraftWithId: stringId ifNone: aBlock
	"The receiver answer anOrbeonFormInstance with id <stringId> if none evaluates the block <aBlock>"

	(currentVersions, oldVersions) do: [:each | (each getDraftWithId: stringId) ifNotNil: [:value | ^value]].

	^aBlock value! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getAnyInstanceWithId: stringId ifNone: aBlock
	"The receiver answer anOrbeonFormInstance with id <stringId> if none evaluates the block <aBlock>"

	(currentVersions, oldVersions) do: [:each | (each getInstanceWithId: stringId) ifNotNil: [:value | ^value]].

	^aBlock value! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getCurrentVersionNumbers
	"Answer a collection with current version number (of each OrbeonFormDefinition)"

	^currentVersions collect: [:each | each version]! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getDefinitionVersion: versionNumber
	"Answer the last published version of the receiver <anOrbeonFormDefinition>"

	^self getDefinitionVersion: versionNumber ifNone: [nil]! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getDefinitionVersion: versionNumber ifNone: aBlock
	"Answer the last published version of the receiver <anOrbeonFormDefinition>"

	currentVersions isEmpty ifTrue: [^aBlock value].

	^currentVersions detect: [:each | each version = versionNumber] ifNone: aBlock! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 12/09/2016 07:02'!
getDefinitionWithId: stringId ifNone: aBlock
	"The receiver answer <anOrbeonPublishedFormDefinition> with id <stringId> if none evaluate the block <aBlock>"

	^currentVersions detect: [:eachDef | eachDef id = stringId] ifNone: aBlock! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getDefinitionWithSameVersionAs: aFormDefinition
	"The receiver answer anOrbeonFormDefinition with the same version as <aFormDefinition>, checking the <currentVersions> collection"

	^self getDefinitionWithVersion: aFormDefinition version! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getDefinitionWithVersion: version
	"The receiver answer anOrbeonFormDefinition with the same version as <aFormDefinition>, checking the <currentVersions> collection"

	^currentVersions detect: [:each | each version = version] ifNone: [nil]! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 10/19/2016 06:01'!
getFormName

	self getAllVersion do: [:each | each formName ifNotNil: [:value | ^value]].

	^nil! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:08'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate

	^self getInstancesDayDistributionQuantityUsing: #createdTime from: startDate to: endDate satisfying: [:each | true]! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:08'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock

	^self getInstancesDayDistributionQuantityUsing: #createdTime from: startDate to: endDate satisfying: formConditionBlock! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 10:25'!
getInstancesCreationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	self getAllVersion do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDistributionByMonthFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 18:02'!
getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := Dictionary new.
	self getAllVersion do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:06'!
getInstancesDayDistributionQuantityUsing: sortSelector from: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := Dictionary new.
	self getAllVersion do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesDayDistributionQuantityUsing: sortSelector from: startDate to: endDate satisfying: formConditionBlock.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 10:43'!
getInstancesDistributionQuantityFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.

	self getAllVersion do: [:each | | currentQuantity currentVersionName |
		currentVersionName := each version printString.
		(distribution includesKey: currentVersionName) ifFalse: [distribution at: currentVersionName put: 0].
		currentQuantity := distribution at: currentVersionName.
		distribution at: currentVersionName put: ((each getInstancesQuantityFrom: startDate to: endDate) + currentQuantity).
	].

	^distribution! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/17/2016 14:22'!
getInstancesDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := Dictionary new.

	self getAllVersion do: [:each | | currentQuantity currentVersionName |
		currentVersionName := each version printString.
		(distribution includesKey: currentVersionName) ifFalse: [distribution at: currentVersionName put: 0].
		currentQuantity := distribution at: currentVersionName.
		distribution at: currentVersionName put: ((each getInstancesQuantityFrom: startDate to: endDate satisfying: formConditionBlock) + currentQuantity).
	].

	^distribution! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:09'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate

	^self getInstancesDayDistributionQuantityUsing: #createdTime from: startDate to: endDate satisfying: [:each | true]! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:09'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock

	^self getInstancesDayDistributionQuantityUsing: #createdTime from: startDate to: endDate satisfying: formConditionBlock! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 10:26'!
getInstancesModificationDistributionByMonthFrom: startDate to: endDate
	| distribution |

	distribution := Dictionary new.
	self getAllVersion do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDistributionByMonthFrom: startDate to: endDate.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 18:01'!
getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := Dictionary new.
	self getAllVersion do: [:each |  | definitionDistribution |
		definitionDistribution := each getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock.
		definitionDistribution keysAndValuesDo: [:key : value |
			(distribution includesKey: key) 
			ifTrue: [distribution at: key put: (distribution at: key) + value] 
			ifFalse: [distribution at: key put: value] 
		] 
	].
		
	^distribution! !

!FormDefinitionRegistry methodsFor: 'published def instances' stamp: 'BrunoBB 07/28/2016 10:35'!
getInstancesOfAllVersions
	| instances |

	instances := OrderedCollection new.

	currentVersions do: [:each | instances addAll: each instances].
	oldVersions do: [:each | instances addAll: each instances].

	^instances! !

!FormDefinitionRegistry methodsFor: 'published def instances' stamp: 'BrunoBB 10/07/2016 07:01'!
getInstancesQuantityFrom: startDate to: endDate

	^(oldVersions ,currentVersions) inject: 0 into:  [:sum :each | sum + (each getInstancesQuantityFrom: startDate to: endDate)]! !

!FormDefinitionRegistry methodsFor: 'saved definitions' stamp: 'BrunoBB 07/28/2016 10:35'!
getLastSavedDefinition
	"The receiver answer the last saved definition <anOrbeonFormInstanceDefinition>"

	^savedDefinitions isEmpty
		ifFalse: [savedDefinitions values last isEmpty "answer anOrderedCollection with all saved definitions"
					ifFalse: [savedDefinitions values last last] "the last element of the last OrderedCollection"
					ifTrue: [NoOrbeonFormException signal: 'There is no last saved definition']]	
		ifTrue: [NoOrbeonFormException signal: 'There is no last saved definition']! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getLastVersion
	"Answer the last published version of the receiver <anOrbeonFormDefinition>"

	currentVersions isEmpty ifTrue: [^nil].

	^currentVersions last! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getLastVersionNumber
	"Answer the version of the last published definition of the receiver"

	^self getLastVersion 
		ifNotNil: [:lastDef | lastDef version]
		ifNil: [0]! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 12/09/2016 09:09'!
getOldDefinitionWithVersion: version
	"The receiver answer anOrbeonFormDefinition with the same version as <aFormDefinition>, checking the <oldVersions> collection"

	^self getOldDefinitionWithVersion: version ifNone: [nil]! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 12/09/2016 09:09'!
getOldDefinitionWithVersion: version ifNone: aBlock
	"The receiver answer anOrbeonFormDefinition with the same version as <aFormDefinition>, checking the <oldVersions> collection"

	^oldVersions detect: [:each | each version = version] ifNone: aBlock! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
getOldVersionNumbers

	^oldVersions collect: [:each | each version]! !

!FormDefinitionRegistry methodsFor: 'Accessing' stamp: 'BrunoBB 07/28/2016 10:35'!
getSavedDefinitions

	^(savedDefinitions collect: [:each | each last]) asOrderedCollection! !

!FormDefinitionRegistry methodsFor: 'saved definitions' stamp: 'BrunoBB 07/28/2016 10:35'!
getSavedDefinitionWithId: stringId ifNone: aBlock
	"The receiver answer <anOrbeonFormInstanceDefinition> with id <stringId> if none evaluate the block <aBlock>"
	| savedCollection |

	savedCollection := savedDefinitions at: stringId ifAbsent: [^aBlock value].

	^savedCollection isEmpty
		ifFalse: [savedCollection last] "the last element of the saved collection"
		ifTrue: aBlock! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
hasVersions

	^currentVersions notEmpty! !

!FormDefinitionRegistry methodsFor: 'saved definitions' stamp: 'BrunoBB 07/28/2016 10:35'!
includesSavedDefinitionWithId: stringId
	"Answer wheter or not the receiver includes a save definition with id <stringId>"
	| savedCollection |

	savedCollection := savedDefinitions at: stringId ifAbsent: [^false].

	^savedCollection notEmpty! !

!FormDefinitionRegistry methodsFor: 'other' stamp: 'BrunoBB 07/28/2016 11:02'!
initialize

	super initialize.

	currentVersions := OrderedCollection new.
	oldVersions := OrderedCollection new.
	savedDefinitions := Dictionary new.! !

!FormDefinitionRegistry methodsFor: 'saved definitions' stamp: 'BrunoBB 07/28/2016 10:35'!
migrateSaveDefinitions
	"Migrate the old structure to new one (30/09/2015)"
	| oldDefnitions |

	oldDefnitions := savedDefinitions asOrderedCollection.

	savedDefinitions := OrderPreservingDictionary new.

	oldDefnitions do: [:each | self addSavedDefinition: each]! !

!FormDefinitionRegistry methodsFor: 'other' stamp: 'BrunoBB 07/28/2016 10:35'!
moveTo: applicationName currentName: currentFormName withNewName: formName
	"The receiver is moved to new <OrbeonApplication> named <applicationName> from current form definition named <currentFormName> to the new name <formName>"

	orbeonApplication unregisterDefinitionNamed: currentFormName. "detach from the old <OrbeonApplication> registry"

	orbeonApplication := self getOrbeonApplicationClass getExistingApplicationOrCreateNewNamed: applicationName. "get the new application"

	orbeonApplication setRegistryFormNamed: formName with: self. "atach the receiver to the new application"

	"update the values for the new registry"
	currentVersions do: [:each | each application: orbeonApplication; formName: formName].
	oldVersions do: [:each | each application: orbeonApplication; formName: formName].
	"<savedDefinitions> is aDictionary of collections"
	savedDefinitions do: [:eachCollection | 
		eachCollection do:[:each | each application: orbeonApplication; formName: formName]].! !

!FormDefinitionRegistry methodsFor: 'other' stamp: 'BrunoBB 10/17/2016 14:10'!
numberOfInstances
	| total |

	total := 0.

	currentVersions do: [:each | total := total + each instances size].

	oldVersions do: [:each | total := total + each instances size].

	^total! !

!FormDefinitionRegistry methodsFor: 'Accessing' stamp: 'BrunoBB 07/28/2016 10:35'!
oldVersions

   "Return the value of the instance variable 'oldVersions'."
   ^oldVersions! !

!FormDefinitionRegistry methodsFor: 'Updating' stamp: 'BrunoBB 07/28/2016 10:35'!
oldVersions: newValue

   "Modify the value of the instance variable 'oldVersions'."
   oldVersions := newValue! !

!FormDefinitionRegistry methodsFor: 'Accessing' stamp: 'BrunoBB 07/28/2016 10:35'!
orbeonApplication

   "Return the value of the instance variable 'orbeonApplication'."
   ^orbeonApplication! !

!FormDefinitionRegistry methodsFor: 'Updating' stamp: 'BrunoBB 07/28/2016 10:35'!
orbeonApplication: newValue

   "Modify the value of the instance variable 'orbeonApplication'."
   orbeonApplication := newValue! !

!FormDefinitionRegistry methodsFor: 'other' stamp: 'BrunoBB 07/28/2016 10:35'!
printAllVersionNumbers
	| printStream |

	printStream := PrintStream on: String new.

	printStream nextPutAll: 'Current: ', String lf.

	self getCurrentVersionNumbers do: [:each | printStream nextPutAll: each printString , ' , '].

	printStream nextPutAll: String lf, 'Old: ', String lf.

	self getOldVersionNumbers do: [:each | printStream nextPutAll: each printString, ' , '].

	^printStream contents! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 07/28/2016 10:35'!
removeDefinition: anOrbeonFormDefinition

	currentVersions remove: anOrbeonFormDefinition ifAbsent: [].

	oldVersions remove: anOrbeonFormDefinition ifAbsent: [].! !

!FormDefinitionRegistry methodsFor: 'saved definitions' stamp: 'BrunoBB 07/28/2016 10:35'!
removeInstanceDefinition: anOrbeonFormInstanceDefinition
	| savedCollection |

	savedCollection := savedDefinitions at: anOrbeonFormInstanceDefinition id ifAbsent: [^anOrbeonFormInstanceDefinition].

	^savedCollection remove: anOrbeonFormInstanceDefinition ifAbsent: []! !

!FormDefinitionRegistry methodsFor: 'published definition' stamp: 'BrunoBB 12/11/2016 14:07'!
replaceCurrentDefinition: currentDefinition with: newDefinition
	"The receiver replace the current published definition <currentDefinition> with the new definition <newDefinition>.
	This is a special case --> <currentDefinition> must be removed from <currentVersions> collection and <newDefinition> must be added at the same index in the <currentVersions> collection"
	| stream |

	currentVersions add: newDefinition after: currentDefinition.

	currentVersions remove: currentDefinition ifAbsent: [].

	oldVersions isEmpty ifTrue: [^oldVersions add: currentDefinition].
 
	stream := ReadStream on: oldVersions. "<oldVersions> must keep the sequence of the version numbers"
	
	[stream atEnd] whileFalse: [ | definition |
		definition := stream next.
		(definition version = currentDefinition version)  ifTrue: [oldVersions add: currentDefinition after: definition].
		(definition version > currentDefinition version)  ifTrue: [oldVersions add: currentDefinition before: definition].
	].! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/17/2016 05:23'!
reportCreationDistributionByMonthFrom: startDate to: endDate

	^self reportCreationDistributionByMonthFrom: startDate to: endDate satisfying: [:each | true]! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/17/2016 05:11'!
reportCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := self getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock.

	^(OrbeonFormDefRegistryDistributionReport applicationName: orbeonApplication applicationName formName: self getFormName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			conditionStringBlock: formConditionBlock blockSource;
			yourself! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 06:00'!
reportInstancesDayDistributionUsing: sortSelector from: startDate to: endDate satisfying: formConditionBlock
	| distribution secondsToRun |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesDayDistributionQuantityUsing: sortSelector from: startDate to: endDate satisfying: formConditionBlock].

	^(OrbeonFormDefRegistryDistributionReport applicationName: self applicationName formName: self getFormName distribution: distribution)
		startDate: startDate;
		endDate: endDate;
		conditionStringBlock: formConditionBlock blockSource;
		distributionMethod: sortSelector;
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/17/2016 05:14'!
reportModificationDistributionByMonthFrom: startDate to: endDate

	^self reportModificationDistributionByMonthFrom: startDate to: endDate satisfying: [:each | true]! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/17/2016 05:12'!
reportModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := self getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock.

	^(OrbeonFormDefRegistryDistributionReport applicationName: orbeonApplication applicationName formName: self getFormName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			conditionStringBlock: formConditionBlock blockSource;
			yourself! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/17/2016 14:20'!
reportQuantityDistributionFrom: startDate to: endDate
	| secondsToRun distribution |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesDistributionQuantityFrom: startDate to: endDate].

	^(OrbeonFormDefRegistryDistributionReport applicationName: orbeonApplication applicationName formName: self getFormName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			sumSecondsToConstruct: secondsToRun;
			yourself! !

!FormDefinitionRegistry methodsFor: 'distribution' stamp: 'BrunoBB 10/17/2016 14:22'!
reportQuantityDistributionFrom: startDate to: endDate satisfying: formConditionBlock
	| secondsToRun distribution |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock].

	^(OrbeonFormDefRegistryDistributionReport applicationName: orbeonApplication applicationName formName: self getFormName distribution: distribution)
			startDate: startDate;
			endDate: endDate;
			sumSecondsToConstruct: secondsToRun;
			yourself! !

!FormDefinitionRegistry methodsFor: 'Accessing' stamp: 'BrunoBB 07/28/2016 10:35'!
savedDefinitions
	^savedDefinitions! !

!FormDefinitionRegistry methodsFor: 'Updating' stamp: 'BrunoBB 07/28/2016 10:35'!
savedDefinitions: newValue
	savedDefinitions := newValue! !

OrbeonObject subclass: #FormFieldDefinition
	instanceVariableNames: 'name isSummary isSearch label type repeatName isAttachmentField tabIndex parentComponent bind'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormFieldDefinition class methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
name: aString

	^super new initialize
		name: aString;
		yourself! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
bind
	^bind! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
bind: newValue
	bind := newValue! !

!FormFieldDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getParentTabIndex
	"Answer the <FormTabIndex> from the parent component"

	^parentComponent tabIndex! !

!FormFieldDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getTopIndex

	^parentComponent getTopIndex! !

!FormFieldDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getTreeLevel
	"Answer the tree level of the receiver in the form tree structure"

	^parentComponent getTreeLevel +  1! !

!FormFieldDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
initialize

	super initialize.

	name := ''.
	isSummary := false.
	isSearch := false.
	label := ''.
	type := ''.
	isAttachmentField := false.
	tabIndex := FormTabIndex for: self withIndex: 0.! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
isAttachmentField
	^isAttachmentField! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
isAttachmentField: newValue
	isAttachmentField := newValue! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
isSearch

   "Return the value of the instance variable 'isSearch'."
   ^isSearch! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
isSearch: newValue

   "Modify the value of the instance variable 'isSearch'."
   isSearch := newValue! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
isSummary

   "Return the value of the instance variable 'isSummary'."
   ^isSummary! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
isSummary: newValue

   "Modify the value of the instance variable 'isSummary'."
   isSummary := newValue! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
label

   "Return the value of the instance variable 'label'."
   ^label! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
label: newValue

   "Modify the value of the instance variable 'label'."
   label := newValue! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
name

   "Return the value of the instance variable 'name'."
   ^name! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
parentComponent
	^parentComponent! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
parentComponent: newValue
	parentComponent := newValue! !

!FormFieldDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: '[', name, ' t:', self printTabIndex, ']'! !

!FormFieldDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
printTabIndex

	^parentComponent printTabIndex, '.', tabIndex printIndex! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
repeatName

   "Return the value of the instance variable 'repeatName'."
   ^repeatName! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
repeatName: newValue

   "Modify the value of the instance variable 'repeatName'."
   repeatName := newValue! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
tabIndex
	^tabIndex! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
tabIndex: newValue

	tabIndex setTabIndex: newValue.! !

!FormFieldDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
type

   "Return the value of the instance variable 'type'."
   ^type! !

!FormFieldDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
type: newValue

   "Modify the value of the instance variable 'type'."
   type := newValue! !

OrbeonObject subclass: #FormGridDefinition
	instanceVariableNames: 'name sectionDefinition fieldDefinitions tabIndex iterationName bind'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormGridDefinition commentStamp: '' prior: 0!
This class represent the GRIDs definition in Orbeon Forms. GRID's fields are repeatable.

name 					- name of the grid definition
sectionDefinition 	- section where the receiver belongs.
fieldDefinitions		- field definitions inside the grid!

!FormGridDefinition class methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
name: aString

	^super new initialize
		name: aString;
		yourself! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
addFieldDefinition: aFormFieldDefinition

	aFormFieldDefinition tabIndex: (fieldDefinitions size + 1).
	aFormFieldDefinition parentComponent: self.

	^fieldDefinitions at: aFormFieldDefinition name put: aFormFieldDefinition! !

!FormGridDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
bind
	^bind! !

!FormGridDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
bind: newValue
	bind := newValue! !

!FormGridDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
fieldDefinitions

   "Return the value of the instance variable 'fieldDefinitions'."
   ^fieldDefinitions! !

!FormGridDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
fieldDefinitions: newValue

   "Modify the value of the instance variable 'fieldDefinitions'."
   fieldDefinitions := newValue! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getFieldNamed: aString
	"The receiver answer a field definition <FormFieldDefinition> named <aString>."

	^self getFieldNamed: aString ifAbsent: []! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getFieldNamed: aString ifAbsent: aBlock
	"The receiver answer a field definition <FormFieldDefinition> named <aString>."

	^fieldDefinitions at: aString ifAbsent: aBlock! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getIterationName
	"Answer the XML node name for iterations when the receiver is repeatable"
	| customIterationName |

	"does have a Custom Iteration Name or not ?"
	customIterationName := iterationName 
									ifNotNil: [iterationName] "custome iteration name"
									ifNil: [name]. "does NOT have a Custom Iteration Name"


	^customIterationName! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getParentTabIndex
	"Answer the <FormTabIndex> from the parent component"

	^sectionDefinition tabIndex! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getTopIndex

	^sectionDefinition getTopIndex! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
getTreeLevel
	"Answer the tree level of the receiver in the form tree structure"

	^sectionDefinition getTreeLevel + 1! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
hasCustomIterationName
	"The receiver answer whether or not has a -Custom Iteration Name-"

	^iterationName notNil! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
includesFieldDefinition: aFormFieldDefinition

	^fieldDefinitions includes: aFormFieldDefinition! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
initialize

	super initialize.

	name := ''.
	fieldDefinitions := Dictionary new.
	tabIndex := FormTabIndex for: self withIndex: 0.! !

!FormGridDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
iterationName
	^iterationName! !

!FormGridDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
iterationName: newValue
	iterationName := newValue! !

!FormGridDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
name

   "Return the value of the instance variable 'name'."
   ^name! !

!FormGridDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
printStructureOn: aPrintStream count: counter
	"Print the structure of the receiver on stream <aPrintStream>. It use a counter <counter> to ident the resulting stream.
	Print Field Definitions of the receiver."

	1 to: (counter + 1) do: [:each | aPrintStream nextPutAll: String tab].
	aPrintStream nextPutAll: '(', name, ')'.
	aPrintStream nextPutAll: String lf.

	fieldDefinitions do: [:each | 
		1 to: (counter + 2) do: [:each | aPrintStream nextPutAll: String tab].
		aPrintStream nextPutAll: each name.
		aPrintStream nextPutAll: ', ', each type.
		aPrintStream nextPutAll: String lf.].! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
printTabIndex

	^sectionDefinition printTabIndex, '.g', tabIndex printIndex! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
removeFieldDefinition: aFormFieldDefinition

	^fieldDefinitions removeKey: aFormFieldDefinition name ifAbsent: []! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
searchFields
	"Answer a collection with the search fields of the receiver"

	^fieldDefinitions select: [:each | each isSearch]! !

!FormGridDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
sectionDefinition

   "Return the value of the instance variable 'sectionDefinition'."
   ^sectionDefinition! !

!FormGridDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
sectionDefinition: newValue

   "Modify the value of the instance variable 'sectionDefinition'."
   sectionDefinition := newValue! !

!FormGridDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:20'!
summaryFields

	^fieldDefinitions select: [:each | each isSummary]! !

!FormGridDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:20'!
tabIndex
	^tabIndex! !

!FormGridDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:20'!
tabIndex: newValue

	tabIndex setTabIndex: newValue.! !

OrbeonObject subclass: #FormGridInstance
	instanceVariableNames: 'name section gridDefinition gridIterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormGridInstance commentStamp: '' prior: 0!
This class represent the GRIDs instances in Orbeon Forms. GRID's fields are repeatable.

name 				- name of the grid
section 			- section where the receiver belongs
gridDefinition	- definition of the receiver
gridIterations	- iterations of the grid.

Grid Iterations is a collection of Dictionary, where each Dictionary represent an iteration.

Dictionary's keys are the fieldsName and the each value is an instance of FormFieldValue.!

!FormGridInstance class methodsFor: 'other' stamp: 'BrunoBB 08/06/2014 17:57'!
name: aString

	^super new initialize
		name: aString;
		yourself! !

!FormGridInstance class methodsFor: 'other' stamp: 'BrunoBB 08/06/2014 11:38'!
name: aString definition: aFormGridDefinition 

	^super new initialize
		name: aString;
		gridDefinition: aFormGridDefinition;
		yourself! !

!FormGridInstance methodsFor: 'other' stamp: 'BrunoBB 08/06/2014 06:46'!
addIteration: aDictionary

	^gridIterations add: aDictionary! !

!FormGridInstance methodsFor: 'other' stamp: 'BrunoBB 04/04/2016 15:15'!
getAllValuesFor: fieldName
	"The receiver answer a collection with all values in all iterations for field named <fieldName>"
	| values |

	values := OrderedCollection new.

	gridIterations do: [:eachDictionary | | value |
		value := eachDictionary at: fieldName ifAbsent: [GridValueNotFoundException signal: 'Values not found for [', fieldName, ']'].
		values add: value].

	^values! !

!FormGridInstance methodsFor: 'Accessing' stamp: 'BrunoBB 08/05/2014 11:15'!
gridDefinition

   "Return the value of the instance variable 'gridDefinition'."
   ^gridDefinition! !

!FormGridInstance methodsFor: 'Updating' stamp: 'BrunoBB 08/05/2014 11:15'!
gridDefinition: newValue

   "Modify the value of the instance variable 'gridDefinition'."
   gridDefinition := newValue! !

!FormGridInstance methodsFor: 'Accessing' stamp: 'BrunoBB 08/05/2014 11:13'!
gridIterations

   "Return the value of the instance variable 'gridIterations'."
   ^gridIterations! !

!FormGridInstance methodsFor: 'Updating' stamp: 'BrunoBB 08/05/2014 11:13'!
gridIterations: newValue

   "Modify the value of the instance variable 'gridIterations'."
   gridIterations := newValue! !

!FormGridInstance methodsFor: 'other' stamp: 'BrunoBB 02/22/2015 11:57'!
initialize

	super initialize.

	name := ''.
	gridIterations := OrderedCollection new. "a collection of Dictionaries. Each dictionary is am iteration"! !

!FormGridInstance methodsFor: 'Accessing' stamp: 'BrunoBB 08/05/2014 11:13'!
name

   "Return the value of the instance variable 'name'."
   ^name! !

!FormGridInstance methodsFor: 'Updating' stamp: 'BrunoBB 08/05/2014 11:13'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue! !

!FormGridInstance methodsFor: 'other' stamp: 'BrunoBB 02/24/2015 03:50'!
printStructureOn: aPrintStream count: counter
	"Print the structure of the receiver on stream <aPrintStream>. It use the counter <counter> to iden the resulting string.
	1- Iterate over each grid iterations (aDictionary) and print each of them."

	| iterationIndex |

	1 to: (counter + 1) do: [:each | aPrintStream nextPutAll: String tab].
	aPrintStream nextPutAll: '(', name, ')'.
	aPrintStream nextPutAll: String lf.

	iterationIndex := 0.
	gridIterations do: [:eachIterationDict | "iterate over grid iteration"
		iterationIndex := iterationIndex + 1.
		1 to: (counter + 2) do: [:each | aPrintStream nextPutAll: String tab].
		aPrintStream nextPutAll: '(iteration-', iterationIndex printString, ')'. "put the number of the current iteration"
		aPrintStream nextPutAll: String lf.
		eachIterationDict keysAndValuesDo: [:key :value | "print the fields values of each iteration"
			1 to: (counter + 3) do: [:each | aPrintStream nextPutAll: String tab].
			aPrintStream nextPutAll: key, ': ', value.
			aPrintStream nextPutAll: String lf.].
		].! !

!FormGridInstance methodsFor: 'Accessing' stamp: 'BrunoBB 08/05/2014 11:15'!
section

   "Return the value of the instance variable 'section'."
   ^section! !

!FormGridInstance methodsFor: 'Updating' stamp: 'BrunoBB 08/05/2014 11:15'!
section: newValue

   "Modify the value of the instance variable 'section'."
   section := newValue! !

!FormGridInstance methodsFor: 'other' stamp: 'BrunoBB 02/28/2015 09:44'!
sectionName

	section ifNil: [^''].

	^section name! !

OrbeonObject subclass: #FormSectionDefinition
	instanceVariableNames: 'name label fieldDefinitions gridDefinitions isRepeatable formDefinition parentSection subSections iterationName tabIndex bind'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormSectionDefinition commentStamp: 'BrunoBB 08/06/2014 11:23' prior: 0!
This class represent a definition of a Section in Orbeon Form, the definition is built inside the Orbeon's Form Builder.

name 				- the name of the section definition
label 				- the label of the section
fieldDefinitions	- aDictionary with all field definitions inside this section (fieldName --> aFormFieldDefinition)
gridDefinitions	- aDictionary with all grid defintions of inside this section (gridName --> aFormGridDefinition)
isRepeatable	- whether or not the receiver is repeatable!

!FormSectionDefinition class methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
name: aString definition: anOrbeonFormDefinition

	^super new initialize
		name: aString;
		formDefinition: anOrbeonFormDefinition;
		yourself! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
addFieldDefinition: aFormFieldDefinition

	aFormFieldDefinition tabIndex: (fieldDefinitions size + 1).
	aFormFieldDefinition parentComponent: self.

	^fieldDefinitions at: aFormFieldDefinition name put: aFormFieldDefinition! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
addGridDefinition: aFormGridDefinition

	aFormGridDefinition tabIndex: (gridDefinitions size + 1).

	aFormGridDefinition sectionDefinition: self.

	^gridDefinitions at: aFormGridDefinition name put: aFormGridDefinition! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
addSubSection: aFormSectionDefinition

	aFormSectionDefinition tabIndex: (subSections size + 1).

	aFormSectionDefinition parentSection: self.

	^subSections add: aFormSectionDefinition! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:19'!
bind
	^bind! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:19'!
bind: newValue
	bind := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
classOf: fieldName
	"Answer a subclass of <AppFieldValue> that represent the Orbeon type of the field named <fieldName>."

	^formDefinition classOf: fieldName! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
fieldDefinitions

   "Return the value of the instance variable 'fieldDefinitions'."
   ^fieldDefinitions! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
fieldDefinitions: newValue

   "Modify the value of the instance variable 'fieldDefinitions'."
   fieldDefinitions := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
fieldNames

	^(fieldDefinitions collect: [:each | each name]) asOrderedCollection! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
formDefinition

   "Return the value of the instance variable 'formDefinition'."
   ^formDefinition! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
formDefinition: newValue

   "Modify the value of the instance variable 'formDefinition'."
   formDefinition := newValue! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
getAllFieldDefinitions
	"Answer a collection with all field definitions <FormFieldDefinition> of the receiver.
	Includes fields inside grid definition <FormGridDefinition>"
	| allFields |

	allFields := OrderedCollection new.

	allFields addAll: fieldDefinitions asOrderedCollection.

	gridDefinitions do: [:each | allFields addAll: each fieldDefinitions asOrderedCollection]. 

	^allFields! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
getAllFieldNames
	"Answer a collection with all field names of the receiver.
	Includes fields inside grid definition <FormGridDefinition>"

	^self getAllFieldDefinitions collect: [:each | each name]! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
getAllSectionsIn: anOrderedCollection

	anOrderedCollection addAll: subSections.

	subSections do: [:each | 
		each getAllSectionsIn: anOrderedCollection].! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
getAllSubSectionsOn: anOrderedCollection

	anOrderedCollection add: self.

	subSections do: [:each | each getAllSubSectionsOn: anOrderedCollection].

	^anOrderedCollection! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 14:54'!
getFieldNamed: aString
	"The receiver answer a field definition <FormFieldDefinition> named <aString>.
	NOTE: the receiver does NOT search inside it's grids <FormDefinitionGrid> for the field"

	^self getFieldNamed: aString ifAbsent: [nil]! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 14:54'!
getFieldNamed: aString ifAbsent: aBlock
	"The receiver answer a field definition <FormFieldDefinition> named <aString>.
	NOTE: the receiver does NOT search inside it's grids <FormDefinitionGrid> for the field"

	^fieldDefinitions at: aString ifAbsent: aBlock! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
getGridDefinitionNamed: aString

	^self getGridDefinitionNamed: aString ifNone: [nil]! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
getGridDefinitionNamed: aString ifNone: aBlock

	^gridDefinitions at: aString ifAbsent: aBlock! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
getParentTabIndex
	"Answer the <FormTabIndex> from the parent component"

	^parentSection 
		ifNil: [nil]
		ifNotNil: [parentSection tabIndex]! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
getSubSectionNamed: aString

	^self getSubSectionNamed: aString ifNone: [nil]! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
getSubSectionNamed: aString ifNone: aBlock

	^subSections detect: [:each | each name = aString] ifNone: aBlock! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
getTopIndex
	"Answer the index of the top element in the form tree structure"

	^parentSection 
		ifNil: [tabIndex index]
		ifNotNil: [parentSection getTopIndex]! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
getTreeLevel
	"Answer the tree level of the receiver in the form tree structure"

	^parentSection 
		ifNil: [1]
		ifNotNil: [parentSection getTreeLevel + 1]! !

!FormSectionDefinition methodsFor: 'custom iteration' stamp: 'BrunoBB 08/17/2016 13:41'!
getXpathForNodeContent
	"The receiver answer a xpath string that points to the content node of the receiver "
	| iterationXpath |

	isRepeatable ifFalse: [^name].

	"this only apply to Repeatable sections"
	iterationXpath := iterationName ifNotNil: [iterationName] ifNil: [name, '-iteration']. "does NOT have a Custom Iteration Name"

	^name, '/', iterationXpath! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
gridDefinitions

   "Return the value of the instance variable 'gridDefinitions'."
   ^gridDefinitions! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
gridDefinitions: newValue

   "Modify the value of the instance variable 'gridDefinitions'."
   gridDefinitions := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
gridNames

	^(gridDefinitions collect: [:each | each name]) asOrderedCollection! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
gridSearchFields
	"The receiver answer a collection of search fields in child grids"
	| searchs |

	searchs := OrderedCollection new.
	gridDefinitions do: [:eachGridDef | searchs addAll: eachGridDef searchFields].

	^searchs! !

!FormSectionDefinition methodsFor: 'custom iteration' stamp: 'BrunoBB 08/15/2016 15:15'!
hasCustomIterationName
	"The receiver answer whether or not has a -Custom Iteration Name-"

	^iterationName notNil! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
hasGrids

	^gridDefinitions notEmpty! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
hasSubSections

	subSections ifNil: [subSections := OrderedCollection new.].

	^subSections notEmpty! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
includesFieldDefinition: aFormFieldDefinition

	^fieldDefinitions includes: aFormFieldDefinition! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
includesGridDefinitionNamed: aString

	^gridDefinitions includesKey: aString! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
initialize

	super initialize.

	name := ''.
	label := ''.
	fieldDefinitions := Dictionary new.
	gridDefinitions := Dictionary new.
	isRepeatable := false.
	subSections := OrderedCollection new.
	tabIndex := FormTabIndex for: self withIndex: 0.! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
isMainSection

	^parentSection isNil! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
isRepeatable

   "Return the value of the instance variable 'isRepeatable'."
   ^isRepeatable! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
isRepeatable: newValue

   "Modify the value of the instance variable 'isRepeatable'."
   isRepeatable := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
isSubSection

	^parentSection notNil! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
iterationName

   "Return the value of the instance variable 'iterationName'."
   ^iterationName! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
iterationName: newValue

   "Modify the value of the instance variable 'iterationName'."
   iterationName := newValue! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
label

   "Return the value of the instance variable 'label'."
   ^label! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
label: newValue

   "Modify the value of the instance variable 'label'."
   label := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
levelNumber

	parentSection ifNil: [^1].

	^1 + (parentSection levelNumber)! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
name

   "Return the value of the instance variable 'name'."
   ^name! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
parentName

	parentSection ifNil: [^''].

	^parentSection name! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
parentSection

   "Return the value of the instance variable 'parentSection'."
   ^parentSection! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
parentSection: newValue

   "Modify the value of the instance variable 'parentSection'."
   parentSection := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
parentSectionIsRepeatable
	"Answer whether or not the parent section of the receiver is repeatable"

	parentSection ifNil: [^false].

	^parentSection isRepeatable! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: '[', name, ']'.! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
printStructureOn: aPrintStream count: counter
	"Print the structure of the receiver on stream <aPrintStream>. As FormSectionDefinition is a recursive structure it use a counter <counter> to ident the resulting stream.
	1- Print Field Definitions of the receiver.
	2- Iterate over Grid Definitions of the receiver and print each of them.
	3- Recursive call. Iterate over Sub Section Defnitions of the receiver and print each of them"

	1 to: counter do: [:each | aPrintStream nextPutAll: String tab].

	aPrintStream nextPutAll: '[', name, ']'.
	isRepeatable ifTrue: [aPrintStream nextPutAll: '(R)'] ifFalse: [aPrintStream nextPutAll: '(N)']. "normal or repeat section"
	aPrintStream nextPutAll: String lf.
	
	fieldDefinitions do: [:each | 
		1 to: (counter + 1) do: [:each | aPrintStream nextPutAll: String tab].
		aPrintStream nextPutAll: each name.
		aPrintStream nextPutAll: ', ', each type.
		aPrintStream nextPutAll: String lf.].

	gridDefinitions do: [:each | each printStructureOn: aPrintStream count: counter].

	subSections do: [:each | each printStructureOn: aPrintStream count: counter + 1]! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
printTabIndex

	^self getParentTabIndex 
		ifNil: [tabIndex printIndex] 
		ifNotNil: ['.', parentSection printTabIndex, '.s', tabIndex printIndex ]! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
removeFieldDefinition: aFormFieldDefinition

	^fieldDefinitions removeKey: aFormFieldDefinition name ifAbsent: []! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
removeGridDefinition: aFormGridDefinition

	^gridDefinitions removeKey: aFormGridDefinition name ifAbsent: []! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
removeSubSection: aFormSectionDefinition

	aFormSectionDefinition parentSection: nil.

	^subSections remove: aFormSectionDefinition ifAbsent: []! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
searchFields
	"The receiver answer a collection with all search fields. Including direct fields and fields in child grids "

	^(fieldDefinitions select: [:each | each isSearch]) asOrderedCollection, self gridSearchFields asOrderedCollection! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
sectionDefinitions

	^subSections! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
subSections

   "Return the value of the instance variable 'subSections'."
   ^subSections! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
subSections: newValue

   "Modify the value of the instance variable 'subSections'."
   subSections := newValue! !

!FormSectionDefinition methodsFor: 'other' stamp: 'BrunoBB 08/15/2016 15:15'!
summaryFields

	^fieldDefinitions select: [:each | each isSummary]! !

!FormSectionDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 08/15/2016 15:15'!
tabIndex
	^tabIndex! !

!FormSectionDefinition methodsFor: 'Updating' stamp: 'BrunoBB 08/15/2016 15:15'!
tabIndex: newValue

	tabIndex setTabIndex: newValue.! !

OrbeonObject subclass: #FormSectionInstance
	instanceVariableNames: 'name sectionDefinition formInstance parentSection subSections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormSectionInstance commentStamp: 'BrunoBB 02/17/2015 11:23' prior: 0!
This class represent a concrete instance of a Section in Orbeon Forms.

name 					- section name
sectionDefinition 	- section definition aFormSectionDefinition
iterations				- if the section is repeatable then each instance is aDictionary in this collection
grids					- grids inside the Section aDictionary of FormGridInstance
fields					- a dictionary of fields when the receiver is NOT repeatable!

!FormSectionInstance class methodsFor: 'other' stamp: 'BrunoBB 08/05/2014 07:21'!
name: aString

	^super new initialize
		name: aString;
		yourself! !

!FormSectionInstance class methodsFor: 'other' stamp: 'BrunoBB 08/05/2014 07:21'!
name: aString definition: aFormSectionDefinition

	^super new initialize
		name: aString;
		sectionDefinition: aFormSectionDefinition;
		yourself! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/25/2015 15:26'!
addSubSection: aFormSectionInstance

	aFormSectionInstance parentSection: self.

	^subSections add: aFormSectionInstance! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 09:42'!
anySubSectionHasGrids

	^self subclassResponsibility! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/22/2015 11:40'!
fieldNames

	^sectionDefinition fieldNames! !

!FormSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/22/2015 11:40'!
formInstance

   "Return the value of the instance variable 'formInstance'."
   ^formInstance! !

!FormSectionInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/22/2015 11:40'!
formInstance: newValue

   "Modify the value of the instance variable 'formInstance'."
   formInstance := newValue! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/25/2015 15:06'!
getAllSectionsIn: anOrderedCollection

	^self subclassResponsibility! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/24/2015 14:16'!
hasDefinedGrids

	^sectionDefinition hasGrids! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/24/2015 14:15'!
hasDefinedSubSections

	^sectionDefinition hasSubSections! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/27/2015 13:30'!
id

	formInstance ifNil: [^parentSection id].

	^formInstance id! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/22/2015 11:41'!
initialize

	super initialize.

	name := ''.
	subSections := OrderedCollection new.! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:31'!
isComplex
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has Sub Sections
		* Has NO Grids "

	^false! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:32'!
isComplexWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has Sub Sections
		* Has Grids "

	^false! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/24/2015 14:19'!
isRepeatable

	^self subclassResponsibility! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:36'!
isRepeatableComplex
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has Sub Sections
		* Has NO Grids "

	^false! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:37'!
isRepeatableComplexWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has Sub Sections
		* Has Grids "

	^false! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:33'!
isRepeatableSimple
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has NO Sub Sections
		* Has NO Grids "

	^false! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:34'!
isRepeatableSimpleWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has NO Sub Sections
		* Has Grids "

	^false! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:28'!
isSimple
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has NO Sub Sections
		* Has NO Grids "

	^false! !

!FormSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:29'!
isSimpleWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has NO Sub Sections
		* Has Grids "

	^false! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/25/2015 15:42'!
levelNumber

	^self subclassResponsibility! !

!FormSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 08/05/2014 07:21'!
name

   "Return the value of the instance variable 'name'."
   ^name! !

!FormSectionInstance methodsFor: 'Updating' stamp: 'BrunoBB 08/05/2014 07:21'!
name: newValue

   "Modify the value of the instance variable 'name'."
   name := newValue! !

!FormSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/22/2015 11:40'!
parentSection

   "Return the value of the instance variable 'parentSection'."
   ^parentSection! !

!FormSectionInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/22/2015 11:40'!
parentSection: newValue

   "Modify the value of the instance variable 'parentSection'."
   parentSection := newValue! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/28/2015 09:26'!
printExportType

	^self subclassResponsibility! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/02/2016 16:51'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: '[', name, ']'.! !

!FormSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/22/2015 11:40'!
removeSubSection: aFormSectionDefinition

	aFormSectionDefinition parentSection: nil.

	^subSections remove: aFormSectionDefinition ifAbsent: []! !

!FormSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 08/05/2014 07:21'!
sectionDefinition

   "Return the value of the instance variable 'sectionDefinition'."
   ^sectionDefinition! !

!FormSectionInstance methodsFor: 'Updating' stamp: 'BrunoBB 08/05/2014 07:21'!
sectionDefinition: newValue

   "Modify the value of the instance variable 'sectionDefinition'."
   sectionDefinition := newValue! !

!FormSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/22/2015 11:40'!
subSections

   "Return the value of the instance variable 'subSections'."
   ^subSections! !

!FormSectionInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/22/2015 11:40'!
subSections: newValue

   "Modify the value of the instance variable 'subSections'."
   subSections := newValue! !

FormSectionInstance subclass: #NormalSectionInstance
	instanceVariableNames: 'fields grids nextIteration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!NormalSectionInstance commentStamp: '' prior: 0!
fields 				- aDictionay where each key is the name of the field and the value is the String value of the field.
grids 				- aDictionay where each key is the name of a Grid and the value is an instance of FormGridInstance.
nextIteration	- if the receiver is repeatable then is an instance of NormalSectionInstance if not is nil.!

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
addGrid: aFormGridInstance

	aFormGridInstance section: self.
	^grids at: aFormGridInstance name put: aFormGridInstance! !

!NormalSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 04/03/2016 13:55'!
anySubSectionHasGrids
	"Answer whether or not the receiver or it's subsections has any Grid"

	^subSections anySatisfy: [:each | each hasDefinedGrids or:[each anySubSectionHasGrids]]! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
fieldAt: fieldName put: fieldValue

	^fields at: fieldName put: fieldValue! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
fields

   "Return the value of the instance variable 'fields'."
   ^fields! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
fields: newValue

   "Modify the value of the instance variable 'fields'."
   fields := newValue! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
getAllSectionsIn: anOrderedCollection

	anOrderedCollection add: self.

	subSections do: [:each | 
		each getAllSectionsIn: anOrderedCollection].! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getComplexSections

	^subSections select: [:each | each isComplex]! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getComplexSectionsWithGrids

	^subSections select: [:each | each isComplexWithGrids]! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
getGridNamed: aString

	^grids at: aString ifAbsent: [nil]! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getRepeatableComplexSections

	^subSections select: [:each | each isRepeatableComplex]! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getRepeatableComplexSectionsWithGrids

	^subSections select: [:each | each isRepeatableComplexWithGrids]! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getRepeatableSimpleSections

	^subSections select: [:each | each isRepeatableSimple]! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getRepeatableSimpleSectionsWithGrids

	^subSections select: [:each | each isRepeatableSimpleWithGrids]! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getSectionsToExport

	^self getSimpleSections,
		self getSimpleSectionsWithGrids,
		self getComplexSections,
		self getComplexSectionsWithGrids,
		self getRepeatableSimpleSections,
		self getRepeatableSimpleSectionsWithGrids,
		self getRepeatableComplexSections,
		self getRepeatableComplexSectionsWithGrids! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getSimpleSections

	^subSections select: [:each | each isSimple]! !

!NormalSectionInstance methodsFor: 'get sections' stamp: 'BrunoBB 04/03/2016 13:55'!
getSimpleSectionsWithGrids

	^subSections select: [:each | each isSimpleWithGrids]! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 18:41'!
getSubSectionIndex

	parentSection ifNil: [^formInstance getSubSectionIndexOf: self].

	^((parentSection subSections select: [:each | each name = name]) indexOf: self ifAbsent: [0]) printString! !

!NormalSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 04/03/2016 13:55'!
grids

   "Return the value of the instance variable 'grids'."
   ^grids! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
grids: newValue

   "Modify the value of the instance variable 'grids'."
   grids := newValue! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
includesGridNamed: aString

	^grids includesKey: aString! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
initialize

	super initialize.

	fields := Dictionary new. 
	grids := Dictionary new.! !

!NormalSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 04/03/2016 13:55'!
isComplex
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has Sub Sections
		* Has NO Grids "

	^(self hasDefinedSubSections and: [self hasDefinedGrids not])
		and: [self anySubSectionHasGrids not]! !

!NormalSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 04/03/2016 13:55'!
isComplexWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has Sub Sections
		* Has Grids "

	^self hasDefinedSubSections and: [(self hasDefinedGrids) or: [self anySubSectionHasGrids]]! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
isRepeatable

	^false! !

!NormalSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 04/03/2016 13:55'!
isSimple
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has NO Sub Sections
		* Has NO Grids "

	^self hasDefinedSubSections not and: [self hasDefinedGrids not]! !

!NormalSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 04/03/2016 13:55'!
isSimpleWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is NOT a Repeatable Section
		* Has NO Sub Sections
		* Has Grids "

	^self hasDefinedSubSections not and: [self hasDefinedGrids]! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
levelNumber

	parentSection ifNil: [^1].

	parentSection isRepeatable ifTrue: [^0 + (parentSection levelNumber)].

	^1 + (parentSection levelNumber)! !

!NormalSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 04/03/2016 13:55'!
nextIteration
	^nextIteration! !

!NormalSectionInstance methodsFor: 'Updating' stamp: 'BrunoBB 04/03/2016 13:55'!
nextIteration: newValue
	nextIteration := newValue! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 13:55'!
printExportType

	(parentSection notNil and: [parentSection isRepeatable]) ifTrue: [^'(R)'].

	^'(N)'! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 18:31'!
printStructureOn: aPrintStream count: counter
	"Print the structure of the receiver on stream <aPrintStream>. As FormSectionInstance is a recursive structure it use the counter <counter> to ident the resulting string.
	1- Print Field Instances.
	2- Iterate over Grid Instances and print each of them.
	3- Recursive call. Iterate over Sub Section Instances of the receiver and print each of them"

	1 to: counter do: [:each | aPrintStream nextPutAll: String tab].

	aPrintStream nextPutAll: '[', name, ']'.
	sectionDefinition isRepeatable 
	ifTrue: [aPrintStream nextPutAll: ' - iteration(', self getSubSectionIndex, ')'].
	aPrintStream nextPutAll: String lf.
	
	fields keysAndValuesDo: [:key :value | 
		1 to: (counter + 1) do: [:each | aPrintStream nextPutAll: String tab].
		aPrintStream nextPutAll: key, ': ', (value ifNil: ['nil'] ifNotNil: [:value | value]).
		aPrintStream nextPutAll: String lf.].

	grids do: [:each | each printStructureOn: aPrintStream count: counter].

	subSections do: [:each | 
		each printStructureOn: aPrintStream count: counter + 1]! !

!NormalSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/09/2016 08:32'!
valueAt: fieldName 
	"Answer the string/original value in XML for the field named <fieldName>"

	^fields at: fieldName ifAbsent: nil! !

FormSectionInstance subclass: #RepeatSectionInstance
	instanceVariableNames: 'sectionIterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/25/2015 15:38'!
addSectionIteration: aFormSectionInstance

	aFormSectionInstance parentSection: self.

	^sectionIterations add: aFormSectionInstance! !

!RepeatSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 09:39'!
anySubSectionHasGrids
	"Answer whether or not the receiver or it's subsections has any Grid"

	^sectionIterations anySatisfy: [:each | each anySubSectionHasGrids]! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 05/09/2015 13:10'!
getAllSectionsIn: anOrderedCollection

	"we do NOT include the repeatable section itself but rather the section iterations (instances of NormalSectionInstance)"

	"anOrderedCollection add: self." 

	sectionIterations do: [:each | 
		each getAllSectionsIn: anOrderedCollection].! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/20/2015 04:16'!
initialize

	super initialize.

	sectionIterations := OrderedCollection new.! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/24/2015 14:19'!
isRepeatable

	^true! !

!RepeatSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 09:41'!
isRepeatableComplex
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has Sub Sections
		* Has NO Grids "

	^(self hasDefinedSubSections and: [self hasDefinedGrids not])
		and: [self anySubSectionHasGrids not]! !

!RepeatSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 09:41'!
isRepeatableComplexWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has Sub Sections
		* Has Grids "

	^self hasDefinedSubSections and: [(self hasDefinedGrids) or: [self anySubSectionHasGrids]]! !

!RepeatSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:33'!
isRepeatableSimple
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has NO Sub Sections
		* Has NO Grids "

	^self hasDefinedSubSections not and: [self hasDefinedGrids not]! !

!RepeatSectionInstance methodsFor: 'section type' stamp: 'BrunoBB 02/26/2015 03:34'!
isRepeatableSimpleWithGrids
	"Answer whether the receiver has the following characteristics:
		* Is a Repeatable Section
		* Has NO Sub Sections
		* Has Grids "

	^self hasDefinedGrids and: [self hasDefinedSubSections not]! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/27/2015 05:22'!
iterationsDo: aBlock

	^sectionIterations do: aBlock! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/25/2015 15:41'!
levelNumber

	parentSection ifNil: [^1].

	^1 + (parentSection levelNumber)! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/28/2015 09:25'!
printExportType

	^'(R)'! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 04/02/2016 16:04'!
printStructureOn: aPrintStream count: counter
	"Print the structure of the receiver on stream <aPrintStream> and use the counter <counter> to ident the resulting string.
	Iterate over all section iterations (NormalSectionInstances) to print the structure of each of them"

	sectionIterations do: [:each | each printStructureOn: aPrintStream count: counter].

	subSections do: [:each | each printStructureOn: aPrintStream count: counter + 1].! !

!RepeatSectionInstance methodsFor: 'other' stamp: 'BrunoBB 02/22/2015 11:29'!
removeSectionIteration: aFormSectionInstance

	^sectionIterations remove: aFormSectionInstance ifAbsent: []! !

!RepeatSectionInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/20/2015 04:15'!
sectionIterations

   "Return the value of the instance variable 'sectionIterations'."
   ^sectionIterations! !

!RepeatSectionInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/20/2015 04:15'!
sectionIterations: newValue

   "Modify the value of the instance variable 'sectionIterations'."
   sectionIterations := newValue! !

OrbeonObject subclass: #FormTabIndex
	instanceVariableNames: 'componentDefinition tabIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!FormTabIndex class methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 11:08'!
for: aOrbeonObjectDefinition withIndex: index

	^super new initialize 
		componentDefinition: aOrbeonObjectDefinition;
		setTabIndex: index;
		yourself! !

!FormTabIndex methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 12:15'!
<= aFormTabIndex

	(self getTopIndex < aFormTabIndex getTopIndex) ifTrue: [^true].
	(self getTopIndex > aFormTabIndex getTopIndex) ifTrue: [^false].

	^(self printTabIndex size = aFormTabIndex printTabIndex size)
			ifTrue: [self printTabIndex <= aFormTabIndex printTabIndex]
			ifFalse: [self printTabIndex size < aFormTabIndex printTabIndex size]! !

!FormTabIndex methodsFor: 'Accessing' stamp: 'BrunoBB 07/31/2016 09:50'!
componentDefinition
	^componentDefinition! !

!FormTabIndex methodsFor: 'Updating' stamp: 'BrunoBB 07/31/2016 09:50'!
componentDefinition: newValue
	componentDefinition := newValue! !

!FormTabIndex methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 09:51'!
getParentTabIndex
	"Answer the <FormTabIndex> from the parent component"

	^componentDefinition getParentTabIndex! !

!FormTabIndex methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 12:12'!
getTopIndex

	^componentDefinition getTopIndex! !

!FormTabIndex methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 10:59'!
getTreeLevel
	"Answer the tree level of the receiver in the form tree structure"

	^componentDefinition getTreeLevel! !

!FormTabIndex methodsFor: 'Accessing' stamp: 'BrunoBB 07/31/2016 12:13'!
index
	^tabIndex! !

!FormTabIndex methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 09:50'!
initialize

	super initialize.

	tabIndex := 0.! !

!FormTabIndex methodsFor: 'Accessing' stamp: 'BrunoBB 07/31/2016 10:50'!
printIndex

	^tabIndex printString! !

!FormTabIndex methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 11:45'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: '(', self printTabIndex, ')'! !

!FormTabIndex methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 11:01'!
printTabIndex

	^componentDefinition printTabIndex! !

!FormTabIndex methodsFor: 'Updating' stamp: 'BrunoBB 07/31/2016 09:50'!
setTabIndex: newValue
	tabIndex := newValue! !

!FormTabIndex methodsFor: 'Accessing' stamp: 'BrunoBB 07/31/2016 09:50'!
tabIndex
	^tabIndex! !

OrbeonObject subclass: #OrbeonForm
	instanceVariableNames: 'id createdTime modifiedTime modifiedUser deleted xmlString version properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

OrbeonForm class
	instanceVariableNames: 'cache'!

!OrbeonForm commentStamp: '' prior: 0!
The basic abstract class to represent Orbeon Form using Smalltalk objects.

id 						- id of the form, is identical to the id in MySQL database (mysql).
form 					- name of the form (mysql).
app						- application's name, where the form belongs (mysql).
createdTime 		- time stamp when created (mysql).
modifiedTime 		- time stamp where last modified (mysql).
modifiedUser 		- last modifier user name (mysql).
deleted				- whether the form is deleted or not (mysql).
fieldNames 			- collection of field's names.
xmlString 			- xml that represent the form (mysql).
xmlDocument 		- DOM document to process the form (usually is nil but when processing some value)
version				- version of the form (mysql).!

OrbeonForm subclass: #OrbeonAbstractFormDefinition
	instanceVariableNames: 'definition application formName'
	classVariableNames: 'OrbeonMapDictionary'
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonAbstractFormDefinition class methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 09:53'!
app: app form: form
	"Answer a new instance of the receiver in OrbeonApplication named <app> with form named <form>.
	If the OrbeonApplication does not exist --> a new one is created"

	^super new initialize
		setApplication: app;
		formName: form;
		yourself! !

!OrbeonAbstractFormDefinition class methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 16:26'!
fileOutForm: xmlString to: aPathFile
	| gsFile |

	gsFile := GsFile open: aPathFile mode: 'wb' onClient: false.

	gsFile
		nextPutAll: xmlString;
		flush;
		close.! !

!OrbeonAbstractFormDefinition class methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 15:50'!
initializeMapDictionary

	OrbeonMapDictionary := Dictionary new.
	OrbeonMapDictionary 
		at: 'xf:string' put: AppFieldStringValue;
		at: 'xs:string' put: AppFieldStringValue;
		at: 'xf:boolean' put: AppFieldBooelanValue;
		at: 'xs:boolean' put: AppFieldBooelanValue;
		at: 'xf:integer' put: AppFieldNumberValue;
		at: 'xs:integer' put: AppFieldNumberValue;
		at: 'xf:decimal' put: AppFieldNumberValue;
		at: 'xs:decimal' put: AppFieldNumberValue;
		at: 'xf:double' put: AppFieldNumberValue;
		at: 'xs:double' put: AppFieldNumberValue;
		at: 'xf:positiveInteger' put: AppFieldNumberValue;
		at: 'xs:positiveInteger' put: AppFieldNumberValue;
		at: 'xf:nonPositiveInteger' put: AppFieldNumberValue;
		at: 'xs:nonPositiveInteger' put: AppFieldNumberValue;
		at: 'xf:negativeInteger' put: AppFieldNumberValue;
		at: 'xs:negativeInteger' put: AppFieldNumberValue;
		at: 'xf:nonNegativeInteger' put: AppFieldNumberValue;
		at: 'xs:nonNegativeInteger' put: AppFieldNumberValue;
		at: 'xf:dateTime' put: AppFieldTimeStampValue;
		at: 'xs:dateTime' put: AppFieldTimeStampValue;
		at: 'xf:date' put: AppFieldDateValue;
		at: 'xs:date' put: AppFieldDateValue;
		at: 'xf:time' put: AppFieldTimeValue;
		at: 'xs:time' put: AppFieldTimeValue.! !

!OrbeonAbstractFormDefinition class methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 15:50'!
mapDictionary

	OrbeonMapDictionary ifNil: [self initializeMapDictionary].

	^OrbeonMapDictionary! !

!OrbeonAbstractFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 04/27/2016 15:44'!
application
	^application! !

!OrbeonAbstractFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 04/27/2016 15:44'!
application: newValue
	application := newValue! !

!OrbeonAbstractFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 04/27/2016 15:44'!
definition
	^definition! !

!OrbeonAbstractFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 04/27/2016 15:44'!
definition: newValue
	definition := newValue! !

!OrbeonAbstractFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 04/27/2016 15:44'!
formName
	^formName! !

!OrbeonAbstractFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 04/27/2016 15:44'!
formName: newValue
	formName := newValue! !

!OrbeonAbstractFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:16'!
getApplication

	^application! !

!OrbeonAbstractFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 15:57'!
getApplicationName
	"Answer a string with the name of the receiver's Orbeon Application"

	application ifNil: [^''].

	^application applicationName! !

!OrbeonAbstractFormDefinition methodsFor: 'type-mapping' stamp: 'BrunoBB 04/28/2016 12:26'!
printOn: aStream

	super printOn: aStream.

	aStream nextPutAll: '[', self getApplicationName, '.', formName, ']'! !

!OrbeonAbstractFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 09:58'!
setApplication: appName
	"The receiver set it's application with anOrbeonApplication named <appName>.
	Get the application from the cache or create a new one"

	application := self getOrbeonApplicationClass getExistingApplicationOrCreateNewNamed: appName.! !

!OrbeonAbstractFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 10:30'!
setMetadata
	| reader |

	reader := self newFormReader.
	app := reader getApplicationName.
	form := reader getFormName.
	self setProperty: 'title' with: reader getTitle.
	self setProperty: 'description' with: reader getDescription.! !

OrbeonAbstractFormDefinition subclass: #OrbeonPublishedFormDefinition
	instanceVariableNames: 'formMetadata instances fieldInstances labels permissions fieldsType sectionDefinitions drafts customAttachment attachments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonPublishedFormDefinition class methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
defaultXmlDefinition: formName app: app

	^'<xh:html xmlns:xh="http://www.w3.org/1999/xhtml" xmlns:xf="http://www.w3.org/2002/xforms"
         xmlns:xs="http://www.w3.org/2001/XMLSchema"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xmlns:ev="http://www.w3.org/2001/xml-events"
         xmlns:xi="http://www.w3.org/2001/XInclude"
         xmlns:xxi="http://orbeon.org/oxf/xml/xinclude"
         xmlns:xxf="http://orbeon.org/oxf/xml/xforms"
         xmlns:exf="http://www.exforms.org/exf/1-0"
         xmlns:fr="http://orbeon.org/oxf/xml/form-runner"
         xmlns:saxon="http://saxon.sf.net/"
         xmlns:sql="http://orbeon.org/oxf/xml/sql"
         xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
         xmlns:fb="http://orbeon.org/oxf/xml/form-builder">
    <xh:head>
        <xh:title>Untitled Form</xh:title>
        <xf:model id="fr-form-model" xxf:expose-xpath-types="true">
            <!!-- Main instance -->
            <xf:instance id="fr-form-instance" xxf:exclude-result-prefixes="#all">
                <form>
                    <section-1>
                        <control-1/>
                    </section-1>
                </form>
            </xf:instance>
            <!!-- Bindings -->
            <xf:bind id="fr-form-binds" ref="instance(''fr-form-instance'')">
                <xf:bind id="section-1-bind" name="section-1" ref="section-1">
                    <xf:bind id="control-1-bind" name="control-1" ref="control-1"/>
                </xf:bind>
            </xf:bind>
            <!!-- Metadata -->
            <xf:instance xxf:readonly="true" id="fr-form-metadata" xxf:exclude-result-prefixes="#all">
                <metadata>
                    <application-name>', app,'</application-name>
                    <form-name>', formName, '</form-name>
                    <title xml:lang="en">Untitled Form</title>
                    <description xml:lang="en"/>
                </metadata>
            </xf:instance>
            <!!-- Attachments -->
            <xf:instance id="fr-form-attachments" xxf:exclude-result-prefixes="#all">
                <attachments>
                    <css mediatype="text/css" filename="" size=""/>
                    <pdf mediatype="application/pdf" filename="" size=""/>
                </attachments>
            </xf:instance>
            <!!-- All form resources -->
            <!!-- Don not make readonly by default in case a service modifies the resources -->
            <xf:instance id="fr-form-resources" xxf:readonly="false" xxf:exclude-result-prefixes="#all">
                <resources>
                    <resource xml:lang="en">
                        <section-1>
                            <label>Untitled Section</label>
                        </section-1>
                        <control-1>
                            <label/>
                            <hint/>
                            <alert/>
                        </control-1>
                    </resource>
                </resources>
            </xf:instance>
            <!!-- Utility instances for services -->
            <xf:instance id="fr-service-request-instance" xxf:exclude-result-prefixes="#all">
                <request/>
            </xf:instance>
            <xf:instance id="fr-service-response-instance" xxf:exclude-result-prefixes="#all">
                <response/>
            </xf:instance>
        </xf:model>
    </xh:head>
    <xh:body>
        <fr:view>
            <fr:body xmlns:xbl="http://www.w3.org/ns/xbl" xmlns:p="http://www.orbeon.com/oxf/pipeline"
                     xmlns:oxf="http://www.orbeon.com/oxf/processors">
                <fr:section id="section-1-control" bind="section-1-bind">
                    <xf:label ref="$form-resources/section-1/label"/>
                    <fr:grid>
                        <xh:tr>
                            <xh:td>
                                <xf:input id="control-1-control" bind="control-1-bind">
                                    <xf:label ref="$form-resources/control-1/label"/>
                                    <xf:hint ref="$form-resources/control-1/hint"/>
                                    <xf:alert ref="$fr-resources/detail/labels/alert"/>
                                </xf:input>
                            </xh:td>
                            <xh:td/>
                        </xh:tr>
                    </fr:grid>
                </fr:section>
            </fr:body>
        </fr:view>
    </xh:body>
</xh:html>'! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
addDraft: anOrbeonFormInstance

	anOrbeonFormInstance definition: self.

	(drafts includesKey: anOrbeonFormInstance id) ifFalse: [drafts at: anOrbeonFormInstance id put: OrderedCollection new].

	^(drafts at: anOrbeonFormInstance id) add: anOrbeonFormInstance! !

!OrbeonPublishedFormDefinition methodsFor: 'field instances' stamp: 'BrunoBB 05/03/2016 17:22'!
addFieldInstance: aAppFieldValue
	"Add <aFormFieldValue> to the receiver. Ask for the key <aAppFieldValue name> to get the IdentitySet for that field name.
	#fieldInstances at: fieldName -  answer anIdentitySet with all AppFieldValue for that field name.
	This IdentitySet has an index to sort fields"
	| indexedCollection |

	indexedCollection := (fieldInstances includesKey: aAppFieldValue name) 
		ifFalse: [fieldInstances at: aAppFieldValue name put: RcIdentityBag new]
		ifTrue: [fieldInstances at: aAppFieldValue name].

	^indexedCollection add: aAppFieldValue! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 07/29/2016 09:33'!
addInstance: anOrbeonFormInstance

	anOrbeonFormInstance definition: self.
	^instances at: anOrbeonFormInstance id put: anOrbeonFormInstance! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
addPermission: permissionName to: roleName
	"The receiver add the permission <permissionName> to the role <roleName>"

	^permissions add: permissionName to: roleName! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 07/28/2016 11:04'!
addSectionDefinition: aFormSectionDefinition

	aFormSectionDefinition tabIndex: (sectionDefinitions size + 1).

	^sectionDefinitions at: aFormSectionDefinition name put: aFormSectionDefinition! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
addTestPermissions
	| adminUser superUser user |

	adminUser :=  'orbeon-', self getApplicationName, '-admin'.
	superUser := 'orbeon-', self getApplicationName, '-super'.
	user :=  'orbeon-', self getApplicationName, '-user'.

	permissions := Dictionary new.
	self addPermission: 'create' to: 'all'.
	self addPermission: 'read' to: superUser.
	self addPermission: 'read' to: adminUser.
	self addPermission: 'read' to: user.
	self addPermission: 'update' to: adminUser.
	self addPermission: 'update' to: superUser.
	self addPermission: 'delete' to: adminUser.
	self addPermission: 'export' to: adminUser.
	self addPermission: 'sync' to: adminUser.! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
attachments
	^attachments! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
attachments: newValue
	attachments := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'type-mapping' stamp: 'BrunoBB 05/03/2016 17:22'!
classOf: attributeName
	"Answer a subclass of <AppFieldValue> that represent the Orbeon type of the field named <attributeName>."

	^self mapOrbeonType: 
			(self typeOf: attributeName) "#typeOf: answer <aString> like 'xf:string' - 'xs:integer' - ... "! !

!OrbeonPublishedFormDefinition methodsFor: 'comparision' stamp: 'BrunoBB 05/03/2016 17:22'!
compareWith: anOrbeonFormDefinition
	"The receiver is compared with form definition <anOrbeonFormDefinition>.
	Answer <anOrderedCollection> with the differences"

	^(FormDefinitionComparator formOne: self formTwo: anOrbeonFormDefinition) compare! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 08/15/2016 15:37'!
createFieldDefinitionFor: aFormSectionDefinition reader: anOrbeonFormReader with: xmlSection
	"Populate receiver's section definition <aFormSectionDefinition> with Field Definition using <anOrbeonFormReader> on anXML element <xmlSection>"
	| search summary |

	summary := anOrbeonFormReader summaryFormFieldsNameIn: xmlSection.
	search := anOrbeonFormReader searchFormFieldsNameIn: xmlSection.	
	(anOrbeonFormReader formFieldsIn: xmlSection) do: [:fieldNode | | fieldDefinition fieldName |
		fieldName := anOrbeonFormReader getNameFrom: fieldNode.
		fieldDefinition := FormFieldDefinition name: fieldName.
		fieldDefinition label: (labels at: fieldName ifAbsent: ['']);
			bind: (fieldNode attributeAt: 'bind');
			type: (fieldsType at: fieldName ifAbsent: ['']);
			isSummary: (summary includes: fieldName);
			isSearch: (search includes: fieldName);
			isAttachmentField: (anOrbeonFormReader isAttachmentField: fieldNode).
		aFormSectionDefinition addFieldDefinition: fieldDefinition.
	]! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 08/17/2016 10:23'!
createGridDefinitionFor: aFormSectionDefinition reader: anOrbeonFormReader with: xmlGrid
	"Populate receiver's section definition <aFormSectionDefinition> with Field Definition using <anOrbeonFormReader> on anXML element <xmlGrid>.
	Only get Repeatable Grids"

	(anOrbeonFormReader formGridComponentsIn: xmlGrid) do: [:eachGrid | | gridDefinition gridName |
		gridName := (eachGrid attributeAt: 'bind') allButLast: 5. "remove: -bind"
		gridDefinition := FormGridDefinition name: gridName.
		gridDefinition bind: (xmlGrid attributeAt: 'bind').
		self createGridFieldDefinitionFor: gridDefinition reader: anOrbeonFormReader with: eachGrid.
		aFormSectionDefinition addGridDefinition: gridDefinition.
	]! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 08/15/2016 15:43'!
createGridFieldDefinitionFor: aFormGridDefinition reader: anOrbeonFormReader with: xmlGrid
	"Populate receiver's grid definition <aFormGridDefinition> with Field Definition using <anOrbeonFormReader> on anXML element <xmlGrid>"
	| search summary |

	summary := anOrbeonFormReader summaryFormFieldsNameIn: xmlGrid.
	search := anOrbeonFormReader searchFormFieldsNameIn: xmlGrid.	
	(anOrbeonFormReader formGridFieldsIn: xmlGrid) do: [:fieldNode | | fieldDefinition fieldName |
		fieldName := anOrbeonFormReader getNameFrom: fieldNode.
		fieldDefinition := FormFieldDefinition name: fieldName.
		fieldDefinition label: (labels at: fieldName ifAbsent: ['']);
			bind: (fieldNode attributeAt: 'bind');
			type: (fieldsType at: fieldName ifAbsent: ['']);
			isSummary: (summary includes: fieldName);
			isSearch: (search includes: fieldName);
			isAttachmentField: (anOrbeonFormReader isAttachmentField: fieldNode).
		aFormGridDefinition addFieldDefinition: fieldDefinition.
	]! !

!OrbeonPublishedFormDefinition methodsFor: 'indexes' stamp: 'BrunoBB 05/04/2016 14:06'!
createIndexForFieldNamed: fieldName
	"The receiver create (initialization - executed only once) an Index for the field named <fieldName>.
	The index is created if the field is a search field."

	fieldInstances at: fieldName ifAbsent: [^NoFieldNameException signal: 'There is no field named [', fieldName, '].'].

	(self isSearchField: fieldName)
	ifFalse: [^IsNotSearchFieldException signal: 'The field named [', fieldName, '] is not a Search field --> no index can be created.'].

	self createIndexForFieldNamed: fieldName on: (fieldInstances at: fieldName) "aRcIdentityBag"! !

!OrbeonPublishedFormDefinition methodsFor: 'indexes' stamp: 'BrunoBB 05/04/2016 14:06'!
createIndexForFieldNamed: fieldName on: aRcIdentityBag
	"The receiver create (initialization - executed only once) an Index for the field named <fieldName>"

	(aRcIdentityBag size >= 2000) "see GemStone/S manual on indexes"
	ifFalse: [^CollectionWithout2000ElementsException signal: 'The collection should have at least 2000 elements to create an index on it.']. 

	GsIndexSpec new
		equalityIndex: 'each.value' lastElementClass: (self stTypeOf: fieldName);
		createIndexesOn: aRcIdentityBag.! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 08/16/2016 05:30'!
createSectionDefinitionNamed: sectionName repeatable: aBoolean basedOn: xmlSection with: anOrbeonFormReader
	"Create the receiver FormSectionDefinition using the form reader <anOrbeonFormReader> "
	| section |

	section := FormSectionDefinition name: sectionName definition: self.
	section bind: (xmlSection attributeAt: 'bind');
		isRepeatable: aBoolean.
	aBoolean ifTrue:[section iterationName: (anOrbeonFormReader getCustomIterationNameFor: sectionName bindName: section bind)].
	self createFieldDefinitionFor: section reader: anOrbeonFormReader with: xmlSection.
	self createGridDefinitionFor: section reader: anOrbeonFormReader with: xmlSection.

	^section! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 05/03/2016 17:22'!
createSectionsDefinitionWith: anOrbeonFormReader
	"Create the receiver FormSectionDefinition using the form reader <anOrbeonFormReader> "

	"Normal Sections"
	(anOrbeonFormReader formMainSectionsRepeatable: false) do: [:xmlSection | | sectionName section |

		sectionName := anOrbeonFormReader getNameFrom: xmlSection.
		section := self createSectionDefinitionNamed: sectionName repeatable: false basedOn: xmlSection with: anOrbeonFormReader.
		self addSectionDefinition: section.
		self createSubSectionsFor: section node: xmlSection with: anOrbeonFormReader].

	"Repeatable Sections"
	(anOrbeonFormReader formMainSectionsRepeatable: true) do: [:xmlSection  | | sectionName section | 

		sectionName := anOrbeonFormReader getNameFrom: xmlSection.
		section := self createSectionDefinitionNamed: sectionName repeatable: true basedOn: xmlSection with: anOrbeonFormReader.
		self addSectionDefinition: section.
		self createSubSectionsFor: section node: xmlSection with: anOrbeonFormReader].! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 05/03/2016 17:22'!
createSubSectionsFor: aFormSectionDefinition node: xmlSection with: anOrbeonFormReader
	"The receiver creates subsection of <aFormSectionDefinition> inside xml segment <xmlSection> using the xml reader <anOrbeonFormReader>"

	(anOrbeonFormReader formSubSectionsOf: xmlSection) do: [:xmlSubSection | | sectionName section isRepeatable |
		sectionName := anOrbeonFormReader getNameFrom: xmlSubSection.
		isRepeatable := xmlSubSection includesAttribute: 'repeat'. "set if the sub section is repeatable"
		section := self createSectionDefinitionNamed: sectionName repeatable: isRepeatable basedOn: xmlSubSection with: anOrbeonFormReader.
		aFormSectionDefinition addSubSection: section.
		self createSubSectionsFor: section node: xmlSubSection with: anOrbeonFormReader.]. "recursive call"! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
customAttachment
	^customAttachment! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
customAttachment: newValue
	customAttachment := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
drafts
	^drafts! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
drafts: newValue
	drafts := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldDefinitionDo: aBlock

	^self fieldDefinitions do: aBlock! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldDefinitions
	"The receiver answer a collection of field definitions. Iterate over embedded sections"
	| sections fields |

	fields := OrderedCollection new.

	sections := self getAllSectionsIn: OrderedCollection new.

	sections do: [:each | fields addAll: (each getAllFieldDefinitions)].

	^fields! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 07/31/2016 11:42'!
fieldDefinitionsByTabIndex
	"The receiver answer a collection of field definitions. Iterate over embedded sections"

	^self fieldDefinitions asSortedCollection: [:a :b | a tabIndex <= b tabIndex]! !

!OrbeonPublishedFormDefinition methodsFor: 'test' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldDefinitionSelect: aBlock

	^self fieldDefinitions select: aBlock! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldInstances
	^fieldInstances! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldInstances: newValue
	fieldInstances := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'field instances' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldInstancesOf: fieldName
	"The receiver answer the set collection that contains the FormFieldValue's of the field named <fieldName>"

	^fieldInstances at: fieldName ifAbsent: [IdentitySet new]! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldsType
	^fieldsType! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
fieldsType: newValue
	fieldsType := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
formMetadata
	^formMetadata! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
formMetadata: newValue
	formMetadata := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 05/03/2016 17:22'!
formReader
	"Answer <anOrbeonFormReader> to get values from the XML document of the receiver"

	"^OrbeonFormReader for: xmlString."

	^OrbeonXPathFormReader for: xmlString.! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getAllDraftsFor: user orGroup: group
	"The receiver answer all current saved drafts of with username <user> or groupname <group>"
	| allDrafts |

	allDrafts := OrderedCollection new.

	drafts do: [:collection | allDrafts addAll: (collection select: [:each | each username = user or: [each groupname = group]])].

	^allDrafts! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getAllDraftWithId: aString ifNone: aBlock

	^drafts at: aString ifAbsent: [^aBlock value]! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getAllSectionsIn: anOrderedCollection

	anOrderedCollection addAll: sectionDefinitions.

	sectionDefinitions do: [:each | 
		each getAllSectionsIn: anOrderedCollection].

	^anOrderedCollection! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
getAttachmentFieldNames
	"The receiver answer a collection of <Strings> where each string is the name of a attachment field"

	^self getAttachmentFields collect: [:each | each name]! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
getAttachmentFields
	"The receiver answer a collection of <FormFieldDefinition> where each field is an attachment component"

	^self fieldDefinitions select: [:each | each isAttachmentField]! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getAttachmentIdFor: fieldName

	^attachments at: fieldName ifAbsent: [nil]! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getDraftWithId: aString

	^(drafts at: aString ifAbsent: [^nil]) last! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getDraftWithId: aString ifNone: aBlock
	| currentDrafts |

	currentDrafts := drafts at: aString ifAbsent: [^aBlock value].

	currentDrafts isEmpty ifTrue: [^aBlock value].

	^currentDrafts last! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
getFieldNamed: fieldName

	^self getFieldNamed: fieldName ifNone: [nil]! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getFieldNamed: aString ifNone: aBlock

	^self fieldDefinitions detect: [:each | each name = aString] ifNone: aBlock! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 07/31/2016 12:55'!
getFieldsNames
	"The receiver answer a collection of field definitions. Iterate over embedded sections"

	^self fieldDefinitionsByTabIndex collect: [:each | each name]! !

!OrbeonPublishedFormDefinition methodsFor: 'test' stamp: 'BrunoBB 05/03/2016 17:22'!
getFieldsWithHyphen
	"Answer a collection with field that it's name includes '-' (Hyphen character).
	This is used for the Form Workflow when evaluating a expression. 
	If the form name includes '-' (Hyphen character) the evaluation will fail.

	Example (without hyphen): [:form | form age > 20]
	
	Example (with hyphen): [:form | form person-age > 20] - this will fail because '-' is a unary message, the correct is:

	[:form | (form perform: 'person-age') > 20]

	When importing Form Definitions from Orbeon if any field name has a hyphen --> we must show a warning to the user"


	^self fieldDefinitionSelect: [:each | each name includes: $-]! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 09/23/2016 05:34'!
getFieldTypesPercentages
	"Answer aDictionary where the key is the name of the field and the value is percentage of that type of field"
	| total qNumber qBoolean qString qDate qTime qTimeStamp groupedDict |

	total := fieldsType size.
	qNumber := qBoolean := qString := qDate := qTime := qTimeStamp := 0.

	groupedDict := fieldsType groupBy: [:fType | self mapOrbeonType: fType] having: [:group | true].

	groupedDict keysAndValuesDo: [:key :collection | 
		(key stType = Number) ifTrue: [qNumber := collection size].
		(key stType = Boolean) ifTrue: [qBoolean := collection size].
		(key stType = String) ifTrue: [qString := collection size].
		(key stType = Date) ifTrue: [qDate := collection size].
		(key stType = Time) ifTrue: [qTime := collection size].
		(key stType = TimeStamp) ifTrue: [qTimeStamp := collection size]. 
	].

	^OrderPreservingDictionary new
		at: 'String' put: (qString * 100 / total);
		at: 'Number' put: (qNumber * 100 / total);
		at: 'Date' put: (qDate * 100 / total);
		at: 'Time' put: (qTime * 100 / total);
		at: 'TimeStamp' put: (qTimeStamp * 100 / total);
		at: 'Boolean' put: (qBoolean * 100 / total);
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 09/24/2016 08:47'!
getFieldTypesQuantity
	"Answer aDictionary where the key is the name of the field and the value is percentage of that type of field"
	| total qNumber qBoolean qString qDate qTime qTimeStamp groupedDict |

	total := fieldsType size.
	qNumber := qBoolean := qString := qDate := qTime := qTimeStamp := 0.

	groupedDict := fieldsType groupBy: [:fType | self mapOrbeonType: fType] having: [:group | true].

	groupedDict keysAndValuesDo: [:key :collection | 
		(key stType = Number) ifTrue: [qNumber := collection size].
		(key stType = Boolean) ifTrue: [qBoolean := collection size].
		(key stType = String) ifTrue: [qString := collection size].
		(key stType = Date) ifTrue: [qDate := collection size].
		(key stType = Time) ifTrue: [qTime := collection size].
		(key stType = TimeStamp) ifTrue: [qTimeStamp := collection size]. 
	].

	^OrderPreservingDictionary new
		at: 'String' put: qString;
		at: 'Number' put: qNumber;
		at: 'Date' put: qDate;
		at: 'Time' put: qTime;
		at: 'TimeStamp' put: qTimeStamp;
		at: 'Boolean' put: qBoolean;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getFirstInstance

	instances isEmpty ifTrue: [^nil].

	^self getInstanceWithId: instances keys asOrderedCollection first! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/04/2016 13:45'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate

	^self getInstancesDayDistributionQuantityUsing: #createdTime from: startDate to: endDate! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 12:23'!
getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock

	^self getInstancesDayDistributionQuantityUsing: #createdTime from: startDate to: endDate satisfying: formConditionBlock! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 11:46'!
getInstancesCreationDistributionByMonthFrom: startDate to: endDate
	| distribution newDistribution |

	distribution := self getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate.
	newDistribution := OrderPreservingDictionary new.

	#('January' 'February' 'March' 'April' 'May' 'June' 'July' 'August' 'September' 'October' 'November' 'December')
		do: [:each | newDistribution at: each put: 0]. 

	distribution keysAndValuesDo: [:key :value | | currentDate currentValue |
		currentDate := startDate addDays: key.
		currentValue := newDistribution at: currentDate monthName ifAbsent: [0].
		newDistribution at: currentDate monthName put: (currentValue + value).
	].

	^newDistribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 17:58'!
getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution newDistribution |

	distribution := self getInstancesCreationDayDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock.
	newDistribution := OrderPreservingDictionary new.

	#('January' 'February' 'March' 'April' 'May' 'June' 'July' 'August' 'September' 'October' 'November' 'December')
		do: [:each | newDistribution at: each put: 0]. 

	distribution keysAndValuesDo: [:key :value | | currentDate currentValue |
		currentDate := startDate addDays: key.
		currentValue := newDistribution at: currentDate monthName ifAbsent: [0].
		newDistribution at: currentDate monthName put: (currentValue + value).
	].

	^newDistribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/04/2016 13:45'!
getInstancesDayDistributionQuantityUsing: sortSelector from: startDate to: endDate
	| distribution |

	distribution := OrderPreservingDictionary new.

	(instances select: [:each | | value |
		value := each perform: sortSelector.
		startDate <= value  and: [value <= endDate]])
			do: [:each | | key value |
				value := each perform: sortSelector.
				key := value date asDays - startDate asDays.
				(distribution includesKey: key) 
					ifFalse: [distribution at: key put: 1] 
					ifTrue: [distribution at: key put: ((distribution at: (key)) + 1)]].

	^distribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 07:01'!
getInstancesDayDistributionQuantityUsing: sortSelector from: startDate to: endDate satisfying: formConditionBlock
	| distribution |

	distribution := OrderPreservingDictionary new.

	(instances select: [:each | | value |
		value := each perform: sortSelector.
		startDate <= value  and: [(value <= endDate) and: [formConditionBlock value: each]]])
			do: [:each | | key value |
				value := each perform: sortSelector.
				key := value date asDays - startDate asDays.
				(distribution includesKey: key) 
					ifFalse: [distribution at: key put: 1] 
					ifTrue: [distribution at: key put: ((distribution at: (key)) + 1)]].

	^distribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/04/2016 13:46'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate

	^self getInstancesDayDistributionQuantityUsing: #modifiedTime from: startDate to: endDate! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 09/29/2016 10:19'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate groupedBy: anInteger
	| distribution numberOfDays dayInterval |

	numberOfDays := endDate asDays - startDate asDays.
	dayInterval := numberOfDays // anInteger.
	distribution := OrderPreservingDictionary new.

	((instances select: [:each | startDate <= each modifiedTime and: [each modifiedTime <= endDate]]) asSortedCollection: [:a :b | a modifiedTime <= b modifiedTime]) 
		do: [:each | | currentDate key |
			currentDate := each modifiedTime date.
			key := (currentDate asDays - startDate asDays) // anInteger.
			(distribution includesKey: key)
				ifFalse: [distribution at: key put: 1] 
				ifTrue: [distribution at: key ifAbsentPut: [1].
					distribution at: key put: ((distribution at: (key)) + 1)].	
		].

	^distribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 12:23'!
getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock

	^self getInstancesDayDistributionQuantityUsing: #modifiedTime from: startDate to: endDate satisfying: formConditionBlock! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/02/2016 10:52'!
getInstancesModificationDistributionByMonthFrom: startDate to: endDate
	| distribution newDistribution |

	distribution := self getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate.
	newDistribution := OrderPreservingDictionary new.

	#('January' 'February' 'March' 'April' 'May' 'June' 'July' 'August' 'September' 'October' 'November' 'December')
		do: [:each | newDistribution at: each put: 0]. 

	distribution keysAndValuesDo: [:key :value | | currentDate currentValue |
		currentDate := startDate addDays: key.
		currentValue := newDistribution at: currentDate monthName ifAbsent: [0].
		newDistribution at: currentDate monthName put: (currentValue + value).
	].

	^newDistribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 17:58'!
getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution newDistribution |

	distribution := self getInstancesModificationDayDistributionQuantityFrom: startDate to: endDate satisfying: formConditionBlock.
	newDistribution := OrderPreservingDictionary new.

	#('January' 'February' 'March' 'April' 'May' 'June' 'July' 'August' 'September' 'October' 'November' 'December')
		do: [:each | newDistribution at: each put: 0]. 

	distribution keysAndValuesDo: [:key :value | | currentDate currentValue |
		currentDate := startDate addDays: key.
		currentValue := newDistribution at: currentDate monthName ifAbsent: [0].
		newDistribution at: currentDate monthName put: (currentValue + value).
	].

	^newDistribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/07/2016 12:25'!
getInstancesQuantityFrom: startDate to: endDate

	^(instances select: [:each | startDate <= createdTime and: [createdTime <= endDate]]) size! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution' stamp: 'BrunoBB 10/19/2016 07:04'!
getInstancesQuantityFrom: startDate to: endDate satisfying: formConditionBlock

	^(instances select: [:each | startDate <= createdTime and: [(createdTime <= endDate) and: [formConditionBlock value: each]]]) size! !

!OrbeonPublishedFormDefinition methodsFor: 'field instances' stamp: 'BrunoBB 05/03/2016 17:22'!
getInstancesSatisfying: aBlock in: fieldName

	^((fieldInstances at: fieldName ifAbsent: [^OrderedCollection new]) select: aBlock) asOrderedCollection collect: [:each | each parentObject]! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getInstanceWithId: aString

	^instances at: aString ifAbsent: [nil]! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getInstanceWithId: aString ifNone: aBlock

	^instances at: aString ifAbsent: aBlock! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
getMandatoryFieldsMissing
	"Answer a collection of mandatory fields missing needed to use Orbeon Form Processes.
	All these fields must be included in the Orbeon Form Definition in order the Form Worflow work correctly"
	| missingManFields |

	missingManFields := OrderedCollection new.

	self getFieldNamed: 's' ifNone: [missingManFields add: 's'].
	self getFieldNamed: 'k' ifNone: [missingManFields add: 'k'].
	self getFieldNamed: 'oop' ifNone: [missingManFields add: 'oop'].
	self getFieldNamed: 'uuid' ifNone: [missingManFields add: 'uuid'].
	self getFieldNamed: 'assignmentOop' ifNone: [missingManFields add: 'assignmentOop'].
	self getFieldNamed: 'username' ifNone: [missingManFields add: 'username'].
	self getFieldNamed: 'rol' ifNone: [missingManFields add: 'rol'].
	self getFieldNamed: 'formId' ifNone: [missingManFields add: 'formId'].

	^missingManFields! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/25/2016 04:30'!
getPercentageDistributionForFieldNamed: fieldName
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	| distributionValue distributionPercentage totalValue |

	distributionValue := self getQuantityDistributionForFieldNamed: fieldName.

	totalValue := distributionValue inject: 0 into: [:sum :each | sum + each].

	distributionValue keysAndValuesDo: [:key :value | 
		distributionPercentage at: key put: (100 * value // totalValue).
	]. 

	^distributionPercentage! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution with range' stamp: 'BrunoBB 10/12/2016 05:12'!
getPercentageDistributionForFieldNamed: fieldName range: range
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	| distributionValue distributionPercentage maxValue startValue endValue |

	distributionValue := OrderPreservingDictionary new.
	distributionPercentage := OrderPreservingDictionary new.
	maxValue := (self fieldInstancesOf: fieldName) size.
	startValue := range first.
	endValue := range second.

	(self fieldInstancesOf: fieldName) do: [:fieldInst |  | fieldValue count |
		fieldValue := (fieldInst value class = String) ifTrue: [fieldInst value] ifFalse: [fieldInst value printString].
		(fieldInst value notNil and: [(startValue <= fieldInst value) and: [fieldInst value <= endValue]])
		ifTrue: [
			(distributionValue includesKey: fieldValue) ifFalse: [distributionValue at: fieldValue put: 0].
			count := (distributionValue at: fieldValue) + 1.
			distributionValue at: fieldValue put: count.
			distributionPercentage at: fieldValue put: (count * 100 / maxValue)]
	].

	^distributionPercentage! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution with range' stamp: 'BrunoBB 10/15/2016 05:29'!
getQuantityDistributionForFieldCollection: fieldInstancesCollection range: range
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole.
	Where the values of the fields are in the range <range>.
	Note: range value must be compatible with fields values "
	| distributionValue startValue endValue |

	distributionValue := OrderPreservingDictionary new.
	startValue := range first.
	endValue := range second.

	fieldInstancesCollection do: [:fieldInst |  | fieldValue count |
		fieldValue := (fieldInst value class = String) ifTrue: [fieldInst value] ifFalse: [fieldInst value printString].
		(fieldInst value notNil and: [(startValue <= fieldInst value) and: [fieldInst value <= endValue]])
		ifTrue: [
			(distributionValue includesKey: fieldValue) ifFalse: [distributionValue at: fieldValue put: 0].
			count := (distributionValue at: fieldValue) + 1.
			distributionValue at: fieldValue put: count]
		ifFalse: [
			(distributionValue includesKey: 'other') ifFalse: [distributionValue at: 'other' put: 0].
			count := (distributionValue at: 'other') + 1.
			distributionValue at: 'other' put: count.
		] 
	].

	^distributionValue! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/16/2016 11:29'!
getQuantityDistributionForFieldNamed: fieldName
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"

	^self getValueDistributionForFieldCollection: (self fieldInstancesOf: fieldName)! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/16/2016 11:30'!
getQuantityDistributionForFieldNamed: fieldName from: startDate to: endDate
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	| dateFilteredCollection |

	dateFilteredCollection := (self fieldInstancesOf: fieldName) select: [:each | startDate <= each parentObject createdTime and: (each parentObject createdTime <= endDate)].

	^self getValueDistributionForFieldCollection: dateFilteredCollection! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution with range' stamp: 'BrunoBB 10/15/2016 05:29'!
getQuantityDistributionForFieldNamed: fieldName range: range
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole.
	Where the values of the fields are in the range <range>.
	Note: range value must be compatible with fields values "

	^self getQuantityDistributionForFieldCollection: (self fieldInstancesOf: fieldName) range: range! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution with range' stamp: 'BrunoBB 10/15/2016 05:32'!
getQuantityDistributionForFieldNamed: fieldName range: range from: startDate to: endDate
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole.
	Where the values of the fields are in the range <range>.
	Note: range value must be compatible with fields values "
	| fieldCollection |

	fieldCollection := (self fieldInstancesOf: fieldName) select: [:each | startDate <= each parentObject createdTime and: [each parentObject createdTime <= endDate]].

	^self getQuantityDistributionForFieldCollection: fieldCollection range: range! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution with range' stamp: 'BrunoBB 10/15/2016 05:37'!
getQuantityDistributionForFieldNamed: fieldName range: range from: startDate to: endDate satisfying: formConditionBlock
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole.
	Where the values of the fields are in the range <range>.
	Note: range value must be compatible with fields values "
	| fieldCollection |

	fieldCollection := (self fieldInstancesOf: fieldName) select: [:each | (startDate <= each parentObject createdTime 
																								and: [each parentObject createdTime <= endDate]) 
																									and: [formConditionBlock value: each parentObject]].

	^self getQuantityDistributionForFieldCollection: fieldCollection range: range! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/16/2016 11:30'!
getQuantityDistributionForFieldNamed: fieldName satisfying: formConditionBlock
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	| blockFilteredCollection |

	blockFilteredCollection := (self fieldInstancesOf: fieldName) select: [:fieldInst | formConditionBlock value: fieldInst parentObject].

	^self getValueDistributionForFieldCollection: blockFilteredCollection! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 07/24/2016 10:21'!
getSectionNamed: aString

	^sectionDefinitions at: aString ifAbsent: [nil]! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/25/2016 04:15'!
getValueDistributionForFieldCollection: fieldInstancesCollection
	"Answer a Dictionary where the key is a value of field <fieldName> and value is a quantity in the whole collection <fieldInstancesCollection>"
	| distribution |
 
	distribution := OrderPreservingDictionary new.

	fieldInstancesCollection do: [:fieldInst |  | fieldValue count |
		fieldValue := (fieldInst value class = String) ifTrue: [fieldInst value] ifFalse: [fieldInst value printString] . 
		(distribution includesKey: fieldValue) ifFalse: [distribution at: fieldValue put: 0].
		count := (distribution at: fieldValue) + 1.
		distribution at: fieldValue put: count.
	].

	^distribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/27/2016 04:38'!
getValueDistributionForFieldName: fieldName secondaryField: secondaryField
     "The receiver answer anArray with two Dictionaries <OrderPreservingDictionary> with receiver's field instances value distribution.
     The first dictionary is the main distribution for the field named <fieldName>.
     The second dictionary is the second distribution for the field named <secondaryField>."
     | distribution orderedDistribution secondaryDistribution orderedSecondaryDistribution mainGroupedDictionary |

     distribution := Dictionary new.
     secondaryDistribution := Dictionary new.
     orderedSecondaryDistribution := OrderPreservingDictionary new.
     orderedDistribution := OrderPreservingDictionary new.

     (self fieldInstancesOf: fieldName) do: [:fieldInst |  | fieldValue count secondaryKey secondaryValue secondaryCount |
         fieldValue := (fieldInst value class = String) ifTrue:[fieldInst value] ifFalse: [fieldInst value printString].
         (distribution includesKey: fieldValue) ifFalse: [distribution at: fieldValue put: 0].
         count := (distribution at: fieldValue) + 1.
         distribution at: fieldValue put: count. "main distribution"

         secondaryValue := fieldInst parentObject fieldValueAt: secondaryField.
         secondaryKey := fieldValue, '.', secondaryValue.
         (secondaryDistribution includesKey: secondaryKey) ifFalse: [secondaryDistribution at: secondaryKey put: 0].
         secondaryCount := (secondaryDistribution at: secondaryKey) + 1.
         secondaryDistribution at: secondaryKey put: secondaryCount. "secondary distribution"
     ].

    distribution keys asSortedCollection do: [:key| orderedDistribution at: key put: (distribution at: key)].

    mainGroupedDictionary := secondaryDistribution keys asSortedCollection groupBy: [:each | (each subStrings: '.') first] having: [:e | true].

    mainGroupedDictionary keys asSortedCollection do: [:key | | keyCollection groupedDictionary |
        keyCollection := mainGroupedDictionary at: key.
         groupedDictionary := Dictionary new.
         keyCollection do: [:eachKey | groupedDictionary at: eachKey put: (secondaryDistribution at: eachKey).
         (groupedDictionary associations asSortedCollection: [:a :b | a value >= b value]) do: [:eachAssoc |
            orderedSecondaryDistribution at: eachAssoc key put: eachAssoc value].
         ].
     ] .

     ^Array with: orderedDistribution with: orderedSecondaryDistribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/27/2016 05:27'!
getValueDistributionForFieldName: fieldName secondaryFields: fieldNamesCollection
	""! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/27/2016 10:43'!
getValueDistributionForFieldName2: fieldName secondaryField: secondaryField
     "The receiver answer anArray with two Dictionaries <OrderPreservingDictionary> with receiver's field instances value distribution.
     The first dictionary is the main distribution for the field named <fieldName>.
     The second dictionary is the second distribution for the field named <secondaryField>."
     | distribution secondaryDistribution  |

     distribution := OrderPreservingDictionary new.
     secondaryDistribution := OrderPreservingDictionary new.

     (self fieldInstancesOf: fieldName) do: [:fieldInst |  | fieldValue count secondaryValue secondaryCount subValueDictionary |
        fieldValue := (fieldInst value class = String) ifTrue:[fieldInst value] ifFalse: [fieldInst value printString].
        (distribution includesKey: fieldValue) ifFalse: [distribution at: fieldValue put: 0].
        count := (distribution at: fieldValue) + 1.
        distribution at: fieldValue put: count. "main distribution"

        secondaryValue := fieldInst parentObject fieldValueAt: secondaryField.

        (secondaryDistribution includesKey: fieldValue) ifFalse: [secondaryDistribution at: fieldValue put: Dictionary new].
		subValueDictionary := secondaryDistribution at: fieldValue.
        secondaryCount := (subValueDictionary at: secondaryValue ifAbsent: [0]) + 1.
		subValueDictionary at: secondaryValue put: secondaryCount.
     ].

     ^Array with: distribution with: secondaryDistribution! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution by fields' stamp: 'BrunoBB 10/28/2016 08:27'!
getValueDistributionForFieldName2: fieldName secondaryField: secondaryField satisfying: formConditionBlock
     "The receiver answer anArray with two Dictionaries <OrderPreservingDictionary> with receiver's field instances value distribution.
     The first dictionary is the main distribution for the field named <fieldName>.
     The second dictionary is the second distribution for the field named <secondaryField>."
     | distribution secondaryDistribution blockFilteredCollection |

	blockFilteredCollection := (self fieldInstancesOf: fieldName) select: [:fieldInst | formConditionBlock value: fieldInst parentObject].

   distribution := OrderPreservingDictionary new.
   secondaryDistribution := OrderPreservingDictionary new.

   blockFilteredCollection do: [:fieldInst |  | fieldValue count secondaryValue secondaryCount subValueDictionary |
		fieldValue := (fieldInst value class = String) ifTrue:[fieldInst value] ifFalse: [fieldInst value printString].
        (distribution includesKey: fieldValue) ifFalse: [distribution at: fieldValue put: 0].
        count := (distribution at: fieldValue) + 1.
        distribution at: fieldValue put: count. "main distribution"

        secondaryValue := fieldInst parentObject fieldValueAt: secondaryField.

        (secondaryDistribution includesKey: fieldValue) ifFalse: [secondaryDistribution at: fieldValue put: Dictionary new].
		subValueDictionary := secondaryDistribution at: fieldValue.
        secondaryCount := (subValueDictionary at: secondaryValue ifAbsent: [0]) + 1.
		subValueDictionary at: secondaryValue put: secondaryCount.
    ].

     ^Array with: distribution with: secondaryDistribution! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
gridDefinitions
	"Answer a collection with all grid definitions <FormGridDefinition> of the receiver. It collect all grids from Section Definitions"
	| grids |

	grids := OrderedCollection new.

	sectionDefinitions do: [:each | grids addAll: each gridDefinitions].

	^grids! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
gridDefinitionsNames
	"Answer a collection with all grid definitions names of the receiver."

	^self gridDefinitions collect: [:each | each name]! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
hasCreatePermission: roleName

	^self hasPermission: 'create' for: roleName! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
hasDeletePermission: roleName

	^self hasPermission: 'delete' for: roleName! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
hasInstances

	^true! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
hasPermission: permissionName for: roleName
	"Answer whether or not the role named <roleName> has the permission named <permissionName> in the receiver"
	
	^permissions has: roleName permission: permissionName! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
hasRangeSearch: attributeName
	| type |

	type := fieldsType at: attributeName ifAbsent: [^false].

	(type = 'xf:string') ifTrue: [^false].
	(type = 'xf:email') ifTrue: [^false].
	(type = 'xf:boolean') ifTrue: [^false].

	^true! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
hasReadPermission: roleName

	^self hasPermission: 'read' for: roleName! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
hasUpdatePermission: roleName

	^self hasPermission: 'update' for: roleName! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
includesSectionDefinition: aFormSectionDefinition

	^sectionDefinitions includes: aFormSectionDefinition! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 07/28/2016 11:02'!
initialize

	super initialize.

	instances := RcKeyValueDictionary new. "hold instances by <id>, more easy to perform the synchronization"
	drafts := RcKeyValueDictionary new. 
	fieldInstances := RcKeyValueDictionary new. "a rc dictionary (by field name) of RcIdentityBag"
	fieldsType := Dictionary new.
	permissions := Dictionary new.
	sectionDefinitions := Dictionary new.
	attachments := Dictionary new.

	self isAvailable: false.
	self isSingleton: false.! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
instanceDefinitionId

	^id! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
instances
	^instances! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
instances: newValue
	instances := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'properties' stamp: 'BrunoBB 05/03/2016 17:30'!
isAvailable
	"Answer whether or not the receiver is available (published) to enter new forms"

	^self getProperty: 'isAvailable'! !

!OrbeonPublishedFormDefinition methodsFor: 'properties' stamp: 'BrunoBB 05/03/2016 17:31'!
isAvailable: value
	"The receiver set the boolean value of property <isAvailable> with value <value>"

	^self setProperty: 'isAvailable' with: value! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
isSearchField: fieldName
	"Answer whether or not the field named <fieldName> is a search field"

	^self searchFields includes: fieldName! !

!OrbeonPublishedFormDefinition methodsFor: 'properties' stamp: 'BrunoBB 05/03/2016 17:31'!
isSingleton
	"Answer whether or not the receiver is a singleton form"

	^self getProperty: 'isSingleton'! !

!OrbeonPublishedFormDefinition methodsFor: 'properties' stamp: 'BrunoBB 05/03/2016 17:31'!
isSingleton: value
	"The receiver set the boolean value of property <isSingleton> with value <value>"

	^self setProperty: 'isSingleton' with: value! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
labelAt: key
	"The receiver answer the label associated with field name <key>"

	^labels at: key ifAbsent: [^'No Label'].! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
labels
	^labels! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
labels: newValue
	labels := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'type-mapping' stamp: 'BrunoBB 05/03/2016 17:22'!
mapOrbeonType: typeString
	"The receiver map the Orbeon type string <typeString> to a GemStone class subclass of <FormFieldValue>"

	^self class mapDictionary at: typeString ifAbsent: [AppFieldStringValue]! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
migratePermissions
	"This method was used to migrate the old permission structure to the new permissions structure (class OrbeonPermissions)"
	| newPermissions |

	newPermissions := OrbeonPermissions new.

	permissions keysAndValuesDo: [:keyStringPermission :arrayRole | 
		newPermissions addAll: (keyStringPermission subStrings: ' ') toAll: arrayRole.
	].

	"permissions := newPermissions. "

	^newPermissions! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
newFormReader
	"Answer <anOrbeonFormReader> to get values from the XML document of the receiver"

	"^OrbeonFormReader for: xmlString."

	^OrbeonXPathFormReader for: xmlString.! !

!OrbeonPublishedFormDefinition methodsFor: 'field instances' stamp: 'BrunoBB 05/03/2016 17:22'!
numberOfFieldInstances

	^fieldInstances inject: 0 into: [:sum :each | sum + each size]! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 05/03/2016 17:22'!
numberOfInstances

	^instances size! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
permissions
	^permissions! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
permissions: newValue
	permissions := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
permissionsCollection
	| permissionCollection |

	permissionCollection  := OrderedCollection new.
	permissions keysAndValuesDo: [:key :value | | grantString |
		grantString := key, ' >> '.
		value do: [:each | grantString := grantString, each, ', '].
		permissionCollection add: (grantString allButLast: 2).
	].

	^permissionCollection! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
printCompleteCustomAttachmentId

		^id, '-', self printCustomAttachmentId! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
printCustomAttachmentId

		^(customAttachment subStrings: '/') last! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 09/23/2016 09:19'!
printNumberOfInstances

	^self numberOfInstances printString! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
printStructure
	| stream |

	stream := PrintStream on: String new.

	self printStructureOn: stream.

	^stream contents! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
printStructureOn: aPrintStream
	"Print the structure of the receiver on stream <aPrintStream>. Iterate over receiver's section definitions"

	aPrintStream nextPutAll: '<', self getApplicationName, '.', formName,'>'.
	aPrintStream nextPutAll: String lf.

	sectionDefinitions do: [:eachSection | 	eachSection printStructureOn: aPrintStream count: 1].! !

!OrbeonPublishedFormDefinition methodsFor: 'process xml' stamp: 'BrunoBB 08/16/2016 05:22'!
processXMLDefinition
	"This is a time consuming operation. Uses a XML Parser to get data from <xmlString> variable"
	| reader |

	reader := self newFormReader.
	labels := reader formFieldsLabels. 						"answer: aDictionary (key: field name value: label)"
	permissions := reader getPermissions. 						"answer: aDictionary (key: permission name value: collection of rols)"
	fieldsType := reader getFieldTypes.						"answer: aDictionary (key: field name value: field type)"
	self isSingleton: reader isSingleton.

	customAttachment := reader getCustomAttachmentId. "set the id of the custom pdf of the receiver"
	self createSectionsDefinitionWith: reader.

	self attachments: (reader getAttachmentIdsFrom: self).
	^self! !

!OrbeonPublishedFormDefinition methodsFor: 'publish' stamp: 'BrunoBB 05/03/2016 17:28'!
publish

	self isAvailable: true.

	definition draft: 'N'.
	definition version: version.
	definition xmlString: xmlString.
	definition definition: self.! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
removeDraft: anOrbeonFormInstance

	"anOrbeonFormInstance definition: nil."

	(drafts includesKey: anOrbeonFormInstance id) ifFalse: [^anOrbeonFormInstance].

	(drafts at: anOrbeonFormInstance id) remove: anOrbeonFormInstance ifAbsent: [].

	(drafts at: anOrbeonFormInstance id) isEmpty ifTrue: [drafts removeKey: anOrbeonFormInstance id].

	^anOrbeonFormInstance! !

!OrbeonPublishedFormDefinition methodsFor: 'field instances' stamp: 'BrunoBB 05/03/2016 17:22'!
removeFieldInstance: aAppFieldValue

	(fieldInstances includesKey: aAppFieldValue name) ifFalse: [^aAppFieldValue].

	^(fieldInstances at: aAppFieldValue name) remove: aAppFieldValue ifAbsent: [nil]! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
removeInstance: anOrbeonFormInstance

	anOrbeonFormInstance definition: nil.
	^instances removeKey: anOrbeonFormInstance id! !

!OrbeonPublishedFormDefinition methodsFor: 'permissions' stamp: 'BrunoBB 05/03/2016 17:22'!
removePermission: permissionName to: roleName
	"The receiver removes the permission <permissionName> from the role <roleName>"

	^permissions remove: permissionName from: roleName! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
removeSectionDefinition: aFormSectionDefinition

	^sectionDefinitions remove: aFormSectionDefinition ifAbsent: []! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/17/2016 13:58'!
reportInstancesCreationDistributionByMonthFrom: startDate to: endDate
	| distribution secondsToRun |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesCreationDistributionByMonthFrom: startDate to: endDate].

	^(OrbeonPublishedFormDistributionReport applicationName: self applicationName formName: formName version: self version distribution: distribution)
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/19/2016 06:35'!
reportInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution secondsToRun |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesCreationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock].

	^(OrbeonPublishedFormDistributionReport applicationName: self applicationName formName: formName version: self version distribution: distribution)
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/19/2016 05:58'!
reportInstancesDayDistributionUsing: sortSelector from: startDate to: endDate satisfying: formConditionBlock
	| distribution secondsToRun |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesDayDistributionQuantityUsing: sortSelector from: startDate to: endDate satisfying: formConditionBlock].

	^(OrbeonPublishedFormDistributionReport applicationName: self applicationName formName: formName version: self version distribution: distribution)
		startDate: startDate;
		endDate: endDate;
		conditionStringBlock: formConditionBlock blockSource;
		distributionMethod: sortSelector;
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/17/2016 13:52'!
reportInstancesModificationDistributionByMonthFrom: startDate to: endDate
	| distribution secondsToRun |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesModificationDistributionByMonthFrom: startDate to: endDate].

	^(OrbeonPublishedFormDistributionReport applicationName: self applicationName formName: formName version: self version distribution: distribution)
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/19/2016 06:35'!
reportInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock
	| distribution secondsToRun |

	secondsToRun := Time secondsElapsedTime: [distribution := self getInstancesModificationDistributionByMonthFrom: startDate to: endDate satisfying: formConditionBlock].

	^(OrbeonPublishedFormDistributionReport applicationName: self applicationName formName: formName version: self version distribution: distribution)
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/16/2016 07:12'!
reportTotalValueDistributionFor: fieldName
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	
	^self reportValueDistributionForFieldCollection: (self fieldInstancesOf: fieldName)! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/28/2016 09:08'!
reportValueDistributionFor: fieldName from: startDate to: endDate
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	| secondsToRun dateFilteredCollection |

	secondsToRun := Time secondsElapsedTime: [dateFilteredCollection := (self fieldInstancesOf: fieldName) select: [:each | startDate <= each parentObject createdTime and: (each parentObject createdTime <= endDate)]].

	^(self reportValueDistributionForFieldCollection: dateFilteredCollection)
		startDate: startDate;
		endDate: endDate;
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/17/2016 10:02'!
reportValueDistributionFor: fieldName from: startDate to: endDate satisfying: formConditionBlock
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	| secondsToRun filteredCollection |

	secondsToRun := Time secondsElapsedTime: [
									filteredCollection := (self fieldInstancesOf: fieldName) 
											select: [:fieldInst | (startDate <= fieldInst parentObject createdTime) 
																	and: [(fieldInst parentObject createdTime <= endDate)
																and: [formConditionBlock value: fieldInst parentObject]]
											]
							].

	^(self reportValueDistributionForFieldCollection: filteredCollection)
		startDate: startDate;
		endDate: endDate;
		conditionStringBlock: formConditionBlock blockSource;
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/17/2016 10:02'!
reportValueDistributionFor: fieldName satisfying: formConditionBlock
	""
	| secondsToRun blockFilteredCollection |

	secondsToRun := Time secondsElapsedTime: [blockFilteredCollection := (self fieldInstancesOf: fieldName) select: [:fieldInst | formConditionBlock value: fieldInst parentObject]].

	^(self reportValueDistributionForFieldCollection: blockFilteredCollection)
		conditionStringBlock: blockFilteredCollection blockSource;
		sumSecondsToConstruct: secondsToRun;
		yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/19/2016 07:09'!
reportValueDistributionForFieldCollection: fieldInstancesCollection
	"Answer a Dictionary where the key is a value of field <fieldName> and value is the percentage of value in the whole"
	| secondsToRun distribution fieldName |

	fieldName := fieldInstancesCollection ifEmpty: ['no-instances'] ifNotEmpty: [(fieldInstancesCollection at: 1) name].

	secondsToRun := Time secondsElapsedTime: [distribution := self getValueDistributionForFieldCollection: fieldInstancesCollection].
	
	^(OrbeonFieldDistributionReport applicationName: self getApplicationName formName: self formName version: self version fieldName: fieldName distribution: distribution)
			sumSecondsToConstruct: secondsToRun;
			yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/27/2016 05:51'!
reportValueDistributionForFieldName: fieldName secondaryField: secondaryField
	""
	| secondsToRun distributionArray |

	secondsToRun := Time secondsElapsedTime: [distributionArray := self getValueDistributionForFieldName2: fieldName secondaryField: secondaryField].
	
	^(OrbeonRecursiveFieldDistributionReport applicationName: self getApplicationName formName: self formName version: self version fieldName: fieldName distribution: distributionArray first)
			sumSecondsToConstruct: secondsToRun;
			addSubDistribution: distributionArray second;
			yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'distribution report' stamp: 'BrunoBB 10/28/2016 08:30'!
reportValueDistributionForFieldName: fieldName secondaryField: secondaryField satisfying: formConditionBlock
	""
	| secondsToRun distributionArray |

	secondsToRun := Time secondsElapsedTime: [distributionArray := self getValueDistributionForFieldName2: fieldName secondaryField: secondaryField satisfying: formConditionBlock].
	
	^(OrbeonRecursiveFieldDistributionReport applicationName: self getApplicationName formName: self formName version: self version fieldName: fieldName distribution: distributionArray first)
			sumSecondsToConstruct: secondsToRun;
			addSubDistribution: distributionArray second;
			yourself! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
resetDrafts

	drafts := RcKeyValueDictionary new.! !

!OrbeonPublishedFormDefinition methodsFor: 'field instances' stamp: 'BrunoBB 05/03/2016 17:22'!
resetInstances

		instances := RcKeyValueDictionary new.
		fieldInstances := Dictionary new.! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
searchFields
	"Answer a collection with the search fields of the receiver"
	| searchs |

	searchs := OrderedCollection new.
	sectionDefinitions do: [:eachDef | searchs addAll: (eachDef searchFields collect: [:each | each name])].

	^searchs! !

!OrbeonPublishedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 05/03/2016 17:22'!
sectionDefinitions
	^sectionDefinitions! !

!OrbeonPublishedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 05/03/2016 17:22'!
sectionDefinitions: newValue
	sectionDefinitions := newValue! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
selectFieldDefinition: aBlock

	^self fieldDefinitions select: aBlock! !

!OrbeonPublishedFormDefinition methodsFor: 'publish' stamp: 'BrunoBB 05/03/2016 17:22'!
setAvailableNodeWith: aBooleanString
	| xmlDoc metadata available newXML |

	xmlDoc := XMLDOMParser parse: xmlString.
	metadata := xmlDoc allElementsNamed: 'metadata'.
	metadata isEmpty ifTrue: [^self].
	metadata := metadata last.

	metadata removeNodes: (metadata allElementsNamed: 'available').

	available := XMLElement named: 'available'.
	available addContent: aBooleanString.
	metadata addNode: available.

	newXML := String new.
	xmlDoc printOn: (PrintStream on: newXML).
	xmlString := newXML.! !

!OrbeonPublishedFormDefinition methodsFor: 'type-mapping' stamp: 'BrunoBB 05/03/2016 17:22'!
stTypeOf: attributeName
	"Answer the type (Smalltalk class) of the for the field named <attributeName> if field name is absent answer nil"

	^(self classOf: attributeName) stType! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
summaryFields
	"Answer a collection with all field names that are the summary of the receiver"
	| summaries |

	summaries := OrderedCollection new.
	sectionDefinitions do: [:eachDef | summaries addAll: (eachDef summaryFields collect: [:each | each name])].

	^summaries! !

!OrbeonPublishedFormDefinition methodsFor: 'publish' stamp: 'BrunoBB 05/03/2016 17:26'!
switchAvailable

	self isAvailable
	ifTrue: [self unpublish.
		^self isAvailable: false].

	self setAvailableNodeWith: 'true'.
	^self isAvailable: true! !

!OrbeonPublishedFormDefinition methodsFor: 'type-mapping' stamp: 'BrunoBB 05/03/2016 17:22'!
typeOf: attributeName
	"Answer the type <aString> of the field named <attributeName> if field name is absent answer nil.
	<aString> is like: 'xs:string' - 'xs:integer - ''xs:boolean' - ...  "

	^fieldsType at: attributeName ifAbsent: [nil]! !

!OrbeonPublishedFormDefinition methodsFor: 'publish' stamp: 'BrunoBB 05/03/2016 17:24'!
unpublish

	self isAvailable: false.

	^self setAvailableNodeWith: 'false'! !

!OrbeonPublishedFormDefinition methodsFor: 'field instances' stamp: 'BrunoBB 05/03/2016 17:22'!
updateFieldInstancesFrom: aFieldValueHolder
	"The receiver update the field's values <FormFieldValue> in the holder <aFieldValueHolder> and mark it as processed.
	The collections <aFieldValueHolder toAdd> and <aFieldValueHolder toRemove> are complemtary, so if one has elements then the other is empty and viceversa. 
	<aFieldValueHolder> is used for repeatable fields with indexes. If a value is removed from the repeatable field --> must be removed from the index collection. <FieldValueHolder> class track these changes"

	aFieldValueHolder alreadyAddedToFormDefinition 
	ifFalse: [aFieldValueHolder alreadyAddedToFormDefinition: true.
		^aFieldValueHolder currentValues do: [:each | self addFieldInstance: each]]. "executed only once"

	"values has been added to the field after a edit operation"
	aFieldValueHolder toAdd do: [:each | self addFieldInstance: each].

	"values has been removed to the field after a edit operation"
	aFieldValueHolder toRemove do: [:each | self removeFieldInstance: each]. 

"	aFieldValueHolder alreadyAddedToFormDefinition: true."! !

!OrbeonPublishedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 05/03/2016 17:22'!
updateFrom: dbArray
	"The receiver update it's self using an update row <dbArray> of the database "

	createdTime := TimeStamp fromOrbeonString: dbArray first trimLeadingSeparators.
	modifiedTime := TimeStamp fromOrbeonString: dbArray second trimLeadingSeparators.
	modifiedUser := dbArray third.
	deleted := dbArray at: 6.
	version := Integer fromString: (dbArray at: 7).
	xmlString := dbArray at: 8.

	^self! !

OrbeonAbstractFormDefinition subclass: #OrbeonSavedFormDefinition
	instanceVariableNames: 'draft username groupname'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 16:28'!
attachments

	definition ifNil: [^#()].

	^definition attachments! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 12:08'!
createPublishedFormDefinition

	^OrbeonPublishedFormDefinition new
		id: id;
		application: application;
		formName: formName;
		xmlString: xmlString;
		title: self title;
		description: self description;
		yourself! !

!OrbeonSavedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 04/27/2016 15:43'!
draft
	^draft! !

!OrbeonSavedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 04/27/2016 15:43'!
draft: newValue
	draft := newValue! !

!OrbeonSavedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 04/27/2016 15:43'!
groupname
	^groupname! !

!OrbeonSavedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 04/27/2016 15:43'!
groupname: newValue
	groupname := newValue! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 16:29'!
hasInstances

	^false! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 15:46'!
initialize

	super initialize.

	draft := 'N'.! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 07/03/2016 13:09'!
labelAt: key
	"The receiver answer the label associated with field name <key>"

	^definition labelAt: key! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 16:29'!
newFormReader
	"Answer <anOrbeonFormReader> to get values from the XML document of the receiver"

	^OrbeonXPathFormReader for: definition xmlString with: xmlString! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 16:29'!
printDraft

	(draft = 'N') ifTrue: [^'false'].

	^'true'! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 16:29'!
processXMLDefinition

	definition ifNil: [^self].

	definition processXMLDefinition! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/30/2016 08:52'!
setXmlString: aString
	"The receiver set the instance variable <xmlString> with <aString> and create the related <OrbeonPublishedFormDefinition> if does not exist"

	xmlString := aString.

	definition 
		ifNil: [definition := self createPublishedFormDefinition.
				definition definition: self].

	definition xmlString: xmlString.! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 07/03/2016 13:08'!
summaryFields

	^definition summaryFields! !

!OrbeonSavedFormDefinition methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 16:29'!
updateFrom: dbArray
	"The receiver update it's self using an update row <dbArray> of the database "

	createdTime := TimeStamp fromOrbeonString: dbArray first.
	modifiedTime := TimeStamp fromOrbeonString: dbArray second.
	id := dbArray third.
	modifiedUser := dbArray fourth notEmpty ifTrue:[dbArray fourth allButFirst] ifFalse: [''].
	username := dbArray fifth notEmpty ifTrue:[dbArray fifth allButFirst] ifFalse: [''].
	groupname := (dbArray at: 6) notEmpty ifTrue:[(dbArray at: 6) allButFirst] ifFalse: [''].
	xmlString := dbArray at: 7.

	^self! !

!OrbeonSavedFormDefinition methodsFor: 'Accessing' stamp: 'BrunoBB 04/27/2016 15:43'!
username
	^username! !

!OrbeonSavedFormDefinition methodsFor: 'Updating' stamp: 'BrunoBB 04/27/2016 15:43'!
username: newValue
	username := newValue! !

!OrbeonForm class methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
cache

	cache ifNil: [self initializeCache].

	^cache! !

!OrbeonForm class methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
initializeCache
	"self initializeCache"
	cache := OrderedCollection new.! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
applicationName

	^self getApplication applicationName! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
available

	^self getProperty: 'available'! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
cloneForAudit
	"The receiver answer a new cloned instance of it's self for audit purpose.
	The section structure is NOT cloned but can be reconstructed from <xmlString>"
	| clone |

	clone := self class new initialize.
	clone id: id;
	"	form: form;
		app: app;"
		createdTime: createdTime;
		modifiedTime: modifiedTime;
		modifiedUser: modifiedUser;
		deleted: deleted;
		xmlString: xmlString;
		version: version.

	^clone! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
createdTime

   "Return the value of the instance variable 'createdTime'."
   ^createdTime! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
createdTime: newValue

   "Modify the value of the instance variable 'createdTime'."
   createdTime := newValue! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
deleted

   "Return the value of the instance variable 'deleted'."
   ^deleted! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
deleted: newValue

   "Modify the value of the instance variable 'deleted'."
   deleted := newValue! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
description

	^self getProperty: 'description'! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
description: aString

	^self setProperty: 'description' with: aString! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
getApplication

	self subclassResponsibility! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
getId

	id ifNil: [^'nil'].

	^id! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
getObjectName
	"Answer the name of the receiver to be used as a key to store fields values"

	^self formName! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
getProperty: propertieName

	^self getProperty: propertieName ifNone: [nil]! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
getProperty: propertieName ifNone: aBlock

	^properties at: propertieName ifAbsent: aBlock! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 07/31/2016 10:59'!
getTreeLevel
	"Answer the tree level of the receiver in the form tree structure.
	The form is the top node of the structure. "

	^0! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
id

   "Return the value of the instance variable 'id'."
   ^id! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
id: newValue

   "Modify the value of the instance variable 'id'."
   id := newValue! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
includesProperty: propertieName

	^properties includesKey: propertieName! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 09/30/2016 10:57'!
initialize

	super initialize.

	createdTime := TimeStamp now.
	deleted := 'N'.
	modifiedTime := TimeStamp now.
	modifiedUser := ''.
	properties := Dictionary new.
	version := 1.! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
logicalDelete

	deleted := 'Y'! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
modifiedTime

   "Return the value of the instance variable 'modifiedTime'."
   ^modifiedTime! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
modifiedTime: newValue

   "Modify the value of the instance variable 'modifiedTime'."
   modifiedTime := newValue! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
modifiedUser

   "Return the value of the instance variable 'modifiedUser'."
   ^modifiedUser! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
modifiedUser: newValue

   "Modify the value of the instance variable 'modifiedUser'."
   modifiedUser := newValue! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
newFormReader
	"Answer <anOrbeonFormReader> to get values from the XML document of the receiver"


	^self subclassResponsibility! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 09/20/2016 16:45'!
printShortId

	id ifNil: [^''].

	^(id size >= 5) ifTrue: [id first: 5] ifFalse: [id]! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
properties
	^properties! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
properties: newValue
	properties := newValue! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
registry
	| application |

	application := self getApplication.
	application ifNil: [^nil].

	^application registryForFormNamed: self formName ifNone: [nil]! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
setProperty: propertieName with: value

	^properties at: propertieName put: value! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
shortId

	^id size > 10 ifTrue: [id first: 10] ifFalse: [id]! !

!OrbeonForm methodsFor: 'binary store' stamp: 'BrunoBB 07/19/2016 12:04'!
storeXmlDecodedOn: pathFile
	"The receiver destructively store on file <pathFile> the content of inst var <xmlString> "

	(FileStream write: pathFile)
		nextPutAll: ((GRCodec forEncoding: 'utf-8') decode: xmlString);
		flush;
		close! !

!OrbeonForm methodsFor: 'binary store' stamp: 'BrunoBB 07/19/2016 12:05'!
storeXmlEncodedOn: pathFile
	"The receiver destructively store on file <pathFile> the content of inst var <xmlString> "

	(FileStream write: pathFile)
		nextPutAll: ((GRCodec forEncoding: 'utf-8') encode: xmlString);
		flush;
		close! !

!OrbeonForm methodsFor: 'binary store' stamp: 'BrunoBB 07/19/2016 12:00'!
storeXmlOn: pathFile
	"The receiver destructively store on file <pathFile> the content of inst var <xmlString> "

	(FileStream write: pathFile)
		nextPutAll: xmlString;
		flush;
		close! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
title

	^self getProperty: 'title'! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
title: aString

	^self setProperty: 'title' with: aString! !

!OrbeonForm methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:18'!
updateLastModifiedTime

	modifiedTime := TimeStamp now.! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
version

   "Return the value of the instance variable 'version'."
   ^version! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
version: newValue

   "Modify the value of the instance variable 'version'."
   version := newValue! !

!OrbeonForm methodsFor: 'Accessing' stamp: 'BrunoBB 04/28/2016 11:18'!
xmlString

   "Return the value of the instance variable 'xmlString'."
   ^xmlString! !

!OrbeonForm methodsFor: 'Updating' stamp: 'BrunoBB 04/28/2016 11:18'!
xmlString: newValue

   "Modify the value of the instance variable 'xmlString'."
   xmlString := newValue! !

OrbeonForm subclass: #OrbeonFormInstance
	instanceVariableNames: 'draft username groupname fields definition fieldsValues repeatFieldsValues sections externalOwner isValid assignment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonFormInstance commentStamp: 'BrunoBB 11/24/2014 05:12' prior: 0!
This class has Orbeon Form Instance. Orbeon Form Instances are always created in the NEW link of Orbeon Form.

Instances of this class represent Forms being filled by users and sent to be processed.

username 				- username of the user that filled the form (mysql).
groupname 				- groupname of the user (mysql).
fields              			- aDictionary where each key is a field name and each value is a string value of the field.
definition 				- the OrbeonFormDefinition of the receiver.
fieldsValues 			- aDictionary where each key is the field name and each value is a AppFieldValueHolder.
repeatFieldsValues 	- aDictionary with the repeats components of the form and it's values.
sections					- aDictionary with the section components of the form and it's values.
externalOwner		- an external owner to be used by third parties packages as Orbeon Form Processes where the <externalOwner> is an OrbeonFormTransition. 

externalOwner:
Using this variable third party packages can use the service layer of Orbeon Form to query forms with some values and after the result (collection of OrbeonFormInstance) collect <externalOwner> variable to access to the specific domain object.!

!OrbeonFormInstance class methodsFor: 'other' stamp: 'BrunoBB 04/29/2016 14:48'!
newFor: anOrbeonFormDefinition

	^super new initialize
		definition: anOrbeonFormDefinition;
		yourself! !

!OrbeonFormInstance class methodsFor: 'other' stamp: 'BrunoBB 04/29/2016 14:44'!
newFor: anOrbeonFormDefinition withId: id

	^super new initialize
		definition: anOrbeonFormDefinition;
		id: id;
		yourself! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
addToDefinition

	^definition addInstance: self! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
assignment
	^assignment! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
assignment: newValue
	assignment := newValue! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/17/2016 12:41'!
classOf: attributeName
	"Answer a Smalltalk class that represent the Orbeon type of the field named <attributeName>.
	1. Get the type of the attribute named <attributeName>
	2. Map the type of 1 with a Smalltalk class"

	^definition classOf: attributeName! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 08/22/2016 14:03'!
cloneForAudit
	"The receiver answer a new cloned instance of it's self for audit purpose.
	The section structure is NOT cloned but can be reconstructed from <xmlString>"
"#depreceted"
	| clone |

	clone := super cloneForAudit.
	clone username: username;
		groupname: groupname;
		draft: draft;
		isValid: isValid.

	^clone! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
definition

   "Return the value of the instance variable 'definition'."
   ^definition! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
definition: newValue

   "Modify the value of the instance variable 'definition'."
   definition := newValue! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
definitionId
	"Answer the id of the form definition"

	^definition id! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 10/07/2016 14:38'!
doesNotUnderstand: aMessageDescriptor
	"The receiver overide this method in order to respond to messages that are equal in name to any of it's field name"

	^(self hasFieldNamed: aMessageDescriptor selector)
	ifTrue: [(self fieldValueAt: aMessageDescriptor selector asString) value]
	ifFalse: [super doesNotUnderstand: aMessageDescriptor]! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
draft
	^draft! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
draft: newValue
	draft := newValue! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
externalOwner

   "Return the value of the instance variable 'externalOwner'."
   ^externalOwner! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
externalOwner: newValue

   "Modify the value of the instance variable 'externalOwner'."
   externalOwner := newValue! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 07/29/2016 08:27'!
fieldDefinitionsByTabIndex
	"The receiver answer a collection of field definitions. Iterate over embedded sections"

	^definition fieldDefinitionsByTabIndex! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
fields

   "Return the value of the instance variable 'fields'."
   ^fields! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
fields: newValue

   "Modify the value of the instance variable 'fields'."
   fields := newValue! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
fieldsValues

   "Return the value of the instance variable 'fieldsValues'."
   ^fieldsValues! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
fieldsValues: newValue

   "Modify the value of the instance variable 'fieldsValues'."
   fieldsValues := newValue! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 04/09/2016 10:22'!
fieldValueAt: fieldName
	"Answer the converted value (a Smalltalk object) for the field named <fieldName>"
	| valueHolder appFieldValue |

	valueHolder := fieldsValues at: fieldName ifAbsent: [^nil].

	valueHolder currentValues ifEmpty: [^nil].

	appFieldValue := valueHolder currentValues last.

	^appFieldValue value! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/11/2016 11:56'!
fieldValuesDo: aBlock
	"The receivet iterate over the fields collection <fieldsValues> and apply the one argument <aBlock> in each iteration"

	^fieldsValues do: aBlock! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 10:43'!
formName

	^definition formName! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 04/14/2016 13:14'!
generateFormFieldValueSet
	"The receiver create an IdentitySet <fieldsValues> of form field values <FormFieldValue>. 
	It creates one element <instance subclass of FormFieldValue> per field in the original XML form."

	"#getAllFields answer aDictionary where each key is the name of the field and each value is a collection of string values"
	self getAllFields keysAndValuesDo: [:key :valuesCollection | | valueHolder |
			valueHolder := (fieldsValues includesKey: key) "<fieldsValues at:> answer aFieldValueHolder"
								ifFalse: [fieldsValues at: key put: (OrbeonFieldValueHolder forFieldNamed: key)]
								ifTrue: [fieldsValues at: key].
			"convert string values to real type values or update added/removed values"
			valueHolder processFieldValueCollection: valuesCollection withFieldClass: (self classOf: key) in: self.
			definition updateFieldInstancesFrom: valueHolder. "update general index collection of the OrbeonFormDefinition"

	].

	^fieldsValues! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/11/2016 11:56'!
getAllFields
	"The receiver answer a Dictionary with ALL fields (including nested sections) where the key is the field name and the value is the collection of fields value. 
	If collection size = 0 --> empty collection, 
	if the collection size = 1 --> simple field, 
	otherwise a field inside a repeat component"
	| allFields |

	allFields := Dictionary new.

	self getAllSections do: [:eachSection | 
		eachSection fields keysAndValuesDo: [:fieldName :fieldValue |
			(allFields includesKey: fieldName)
			ifTrue: [(allFields at: fieldName) add: fieldValue]
			ifFalse: [allFields at: fieldName put: (OrderedCollection with: fieldValue)]].
		eachSection grids do: [:eachGrid | 
			eachGrid gridIterations do: [:dictIteration | 
				dictIteration keysAndValuesDo: [:fieldName :fieldValue |
					(allFields includesKey: fieldName)
					ifTrue: [(allFields at: fieldName) add: fieldValue]
					ifFalse: [allFields at: fieldName put: (OrderedCollection with: fieldValue)]
					]]]].
	^allFields! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/11/2016 11:56'!
getAllGrids
	"The receiver answer an OrderedCollection with ALL Grids (including nested sections) where each element is a Dictionary.
	Each Dictionary represent a Grid Iteration, where the key is the field name and the value is the field value"
	| allGrids |

	allGrids := OrderedCollection new.

	self getAllSections do: [:eachSection | 
		eachSection grids do: [:eachGrid | allGrids addAll: eachGrid gridIterations].].

	^allGrids! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/11/2016 11:56'!
getAllSections
	"Answer all sections <FormSectionInstance> of the receiver "

	^self getAllSectionsIn: OrderedCollection new! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/11/2016 11:56'!
getAllSectionsIn: anOrderedCollection
	"Answer all sections of the receiver in parameter <anOrderedCollection>"

	sections do: [:each | 
		each getAllSectionsIn: anOrderedCollection].

	^anOrderedCollection! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 04/28/2016 11:15'!
getApplication

	^definition application! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 15:01'!
getApplicationName
	"Answer a string with the name of the receiver's Orbeon Application"

	^definition getApplicationName! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
getAttachmentFieldNames
	"Answer a collection of names where each name is a <FormFieldDefinition> that represent an attachment component"

	^definition getAttachmentFieldNames! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
getAttachmentIds
	"Answer a collection with all IDs of receiver's attachments.
	The attachment's names: <instanceId>-<fileId> "

	^((self getAttachmentFieldNames collect: [:each | self valueAt: each]) reject: [:each | each isNil])
			collect: [:each | id, '-', (each subStrings: '/') last] "<instanceId>-<fileId> "! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getComplexSections

	^sections select: [:each | each isComplex]! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getComplexSectionsWithGrids

	^sections select: [:each | each isComplexWithGrids]! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/11/2016 11:56'!
getFields
	"This method does NOT includes repeat values and repeat sections, for that see #getAllFields in this class
	The receiver answer a Dictionary with fields (including nested sections) where the key is the field name and the value is the field value."
	| allFields |

	allFields := Dictionary new.

	self getAllSections do: [:each | allFields addAll: each fields].

	^allFields! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 04/27/2016 15:03'!
getFormName
	"Answer aString with the name of the receiver's definition name"

	^definition formName! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
getPermissionsForUser: user inGroup: group with: formDefinition
	"The receiver answer a collection of permissions (a string separated by blanks) for the user <username> and group <group> using the form definition <formDefinition> to get the current permissions to apply.
	To calculate permissions we use the last version of OrbeonFormDefinition and NOT the <formInstance definition>.
	If not --> is not possible to change permission of a given instance if the definition version is higher that the instance version.
	Using the last version of OrbeonFormDefinition to calculate permissions --> all instance are affected by the change"
	| printStream |

	printStream := PrintStream on: String new.

	((self hasUser: user createAccessUsing: formDefinition permissions) 
	or: [self hasRole: group createAccessUsing: formDefinition permissions])
		ifTrue: [printStream nextPutAll: 'create '].

	((self hasUser: user readAccessUsing: formDefinition permissions) 
	or: [self hasRole: group readAccessUsing: formDefinition permissions])
		ifTrue: [printStream nextPutAll: 'read '].

	((self hasUser: user updateAccessUsing: formDefinition permissions) 
	or: [self hasRole: group updateAccessUsing: formDefinition permissions])
		ifTrue: [printStream nextPutAll: 'update '].

	((self hasUser: user deleteAccessUsing: formDefinition permissions) 
	or: [self hasRole: group deleteAccessUsing: formDefinition permissions])
		ifTrue: [printStream nextPutAll: 'delete '].

	^printStream contents! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getRepeatableComplexSections

	^sections select: [:each | each isRepeatableComplex]! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getRepeatableComplexSectionsWithGrids

	^sections select: [:each | each isRepeatableComplexWithGrids]! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getRepeatableSimpleSections

	^sections select: [:each | each isRepeatableSimple]! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getRepeatableSimpleSectionsWithGrids

	^sections select: [:each | each isRepeatableSimpleWithGrids]! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getSectionsToExport

	^self getSimpleSections,
		self getSimpleSectionsWithGrids,
		self getComplexSections,
		self getComplexSectionsWithGrids,
		self getRepeatableSimpleSections,
		self getRepeatableSimpleSectionsWithGrids,
		self getRepeatableComplexSections,
		self getRepeatableComplexSectionsWithGrids! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getSimpleSections

	^sections select: [:each | each isSimple]! !

!OrbeonFormInstance methodsFor: 'get sections' stamp: 'BrunoBB 02/11/2016 11:56'!
getSimpleSectionsWithGrids

	^sections select: [:each | each isSimpleWithGrids]! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 04/03/2016 18:42'!
getSubSectionIndexOf: aFormSectionInstance

	^((sections select: [:each | each name = aFormSectionInstance name]) indexOf: aFormSectionInstance ifAbsent: [0]) printString! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 04/15/2016 13:13'!
getValueOfFields: collectionOfFieldNames

	^collectionOfFieldNames collect: [:each | self valueAt: each]! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
groupname

   "Return the value of the instance variable 'groupname'."
   ^groupname! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
groupname: newValue

   "Modify the value of the instance variable 'groupname'."
   groupname := newValue! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasExternalOwner

	^externalOwner notNil! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasFieldNamed: aSymbol
	"Answer whether or not the receiver has a field named <aSymbol>"

	^(self getFields includesKey: aSymbol asString) 
		or: [self getAllGrids anySatisfy: [:each | each includesKey: aSymbol asString] ]! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasRole: aRoleName createAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	(groupname = aRoleName) ifFalse: [^anOrbeonPermissions hasCreateAccess: aRoleName].

	^anOrbeonPermissions hasCreateAccess: 'group-member'! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasRole: aRoleName deleteAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	(groupname = aRoleName) ifFalse: [^anOrbeonPermissions hasDeleteAccess: aRoleName].

	^anOrbeonPermissions hasDeleteAccess: 'group-member'! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasRole: aRoleName readAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	(groupname = aRoleName) ifFalse: [^anOrbeonPermissions hasReadAccess: aRoleName].

	^anOrbeonPermissions hasReadAccess: 'group-member'! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasRole: aRoleName updateAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	(groupname = aRoleName) ifFalse: [^anOrbeonPermissions hasUpdateAccess: aRoleName].

	^anOrbeonPermissions hasUpdateAccess: 'group-member'! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasUser: aUserName createAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	^(aUserName = username) and: [anOrbeonPermissions hasCreateAccess: 'owner']! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasUser: aUserName deleteAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	^(aUserName = username) and: [anOrbeonPermissions hasDeleteAccess: 'owner']! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasUser: aUserName readAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	^(aUserName = username) and: [anOrbeonPermissions hasReadAccess: 'owner']! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
hasUser: aUserName updateAccessUsing: anOrbeonPermissions

	anOrbeonPermissions isEnabled ifFalse: [^true]. "if permissions are NOT enabled then it has ALL permissions"

	^(aUserName = username) and: [anOrbeonPermissions hasUpdateAccess: 'owner']! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 07/29/2016 09:00'!
holderValueAt: fieldName
	"Answer the <AppFieldValueHolder> for the field named <fieldName>"

	^fieldsValues at: fieldName ifAbsent: [nil]! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 07/27/2016 11:37'!
initialize

	super initialize.

	draft  := 'N'.
	username := ''.
	groupname := ''.
	fields := Dictionary new.
	fieldsValues := Dictionary new.
	repeatFieldsValues := IdentitySet new. 
"<repeatFieldsValues> is an IdentitySet of where each key is the name of the repeat component and the value is anOrderedCollection of Dictionaries. Where each Dictonary holds the data of each individual repeat "
	sections := OrderedCollection new.
	isValid := false.! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
isAttachmentField: fieldName
	"Answer whether or not the field named <fieldName> is an attachment field"

	^(definition getFieldNamed: fieldName) isAttachmentField! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
isValid

   "Return the value of the instance variable 'isValid'."
   ^isValid! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
isValid: newValue

   "Modify the value of the instance variable 'isValid'."
	newValue isString ifTrue: [^isValid := self evaluate: newValue].
 
	isValid := newValue! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
labelAt: key
	"Answer the label of field named <key> if none answer 'No Label' "

	^definition labelAt: key! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 03/23/2016 08:14'!
newFormReader
	"Answer <anOrbeonFormReader> to get values from the XML document of the receiver"

	^OrbeonXPathFormReader for: definition xmlString with: xmlString! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
printDraft

	(draft = 'N') ifTrue: [^'false'].

	^'true'! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
printStructure
	| stream |

	stream := PrintStream on: String new.

	self printStructureOn: stream.

	^stream contents! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 08/16/2016 10:58'!
printStructureOn: aPrintStream
	"Print the structure of the receiver on stream <aPrintStream>. Iterate over receiver's section instances"

	aPrintStream nextPutAll: '<', self getApplicationName, '.', self getFormName,'>'.
	aPrintStream nextPutAll: String lf.

	sections do: [:eachSection | eachSection printStructureOn: aPrintStream count: 1].! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
process

	^externalOwner process! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 02/23/2016 08:34'!
processXML
	"Set the values of receiver field's from <xmlString> using an orbeon form reader.
	The receiver has a dictionary (fieldName>value) to hold fields."

	self processXMLSections. "Create the GS Sections from the XML string"

	definition addInstance: self.

	self generateFormFieldValueSet. "store field values as instance of <FormFieldValue>"

	^fields! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 04/02/2016 10:06'!
processXMLSections
	"Set the values of receiver field's from <xmlString> using an orbeon form reader."

	sections := self newFormReader createSectionsInstancesWith: definition.
	
	sections do: [:each | each formInstance: self].! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
repeatFieldsValues

   "Return the value of the instance variable 'repeatFieldsValues'."
   ^repeatFieldsValues! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
repeatFieldsValues: newValue

   "Modify the value of the instance variable 'repeatFieldsValues'."
   repeatFieldsValues := newValue! !

!OrbeonFormInstance methodsFor: 'other' stamp: 'BrunoBB 02/11/2016 11:56'!
satisfyQuery: collectionOfOrbeonQueryNode
	"Answer whether or not the receiver satisfy the query node <OrbeonQueryNode> in collection <collectionOfOrbeonQueryNode>.
	The search is AND"
	| queryFields |

	"collect the values of each field contained in <collectionOfOrbeonQueryNode>"
	queryFields := self getValueOfFields: (collectionOfOrbeonQueryNode collect: [:each | each name]).

	"all satisfy node value = field value (AND)"
	collectionOfOrbeonQueryNode keysAndValuesDo: [:index :node |
		(node value = (queryFields at: index)) ifFalse: [^false]].


	^true! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
sections

   "Return the value of the instance variable 'sections'."
   ^sections! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
sections: newValue

   "Modify the value of the instance variable 'sections'."
   sections := newValue! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 04/16/2016 16:11'!
typeOf: attributeName
	"Answer the type <aString> of the field named <attributeName> if field name is absent answer nil.
	<aString> is like: 'xs:string' - 'xs:integer - ''xs:boolean' - ...  "

	^definition typeOf: attributeName! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 04/21/2016 11:04'!
updateFrom: dbArray
	"The receiver update it's self using an update row <dbArray> of the database "

	createdTime := TimeStamp fromOrbeonString: dbArray first trimLeadingSeparators.
	modifiedTime := TimeStamp fromOrbeonString: dbArray second trimLeadingSeparators.
	modifiedUser := dbArray third.
	id := dbArray at: 6.
	deleted := dbArray at: 7.
	xmlString :=  dbArray at: 8.
	username :=  dbArray at: 9.
	groupname :=  dbArray at: 10.
	version := Integer fromString: (dbArray at: 11).
	^self! !

!OrbeonFormInstance methodsFor: 'Accessing' stamp: 'BrunoBB 02/11/2016 11:56'!
username

   "Return the value of the instance variable 'username'."
   ^username! !

!OrbeonFormInstance methodsFor: 'Updating' stamp: 'BrunoBB 02/11/2016 11:56'!
username: newValue

   "Modify the value of the instance variable 'username'."
   username := newValue! !

!OrbeonFormInstance methodsFor: 'form-field-value' stamp: 'BrunoBB 04/15/2016 13:52'!
valueAt: fieldName
	"The receiver answer the value of the field named <fieldName> if none answer nil.
	The value is the original string value in the XML"
	| value |

	self getAllSections do: [:each | value := each valueAt: fieldName.
		value ifNotNil: [^value]].

	^nil! !

OrbeonObject subclass: #OrbeonFormUserOptions
	instanceVariableNames: 'orbeonWebIsLinked languageName initialTransitionDisplayMode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonFormUserOptions methodsFor: 'other' stamp: 'BrunoBB 10/22/2015 18:46'!
initialize

	super initialize.

	"true - WALinkedAssignmentManager, false - WAEmbeddedAssignmentManager"
	orbeonWebIsLinked := true.
	languageName := #english.
	initialTransitionDisplayMode := #expanded.! !

!OrbeonFormUserOptions methodsFor: 'Accessing' stamp: 'BrunoBB 06/23/2015 12:23'!
languageName
	^languageName! !

!OrbeonFormUserOptions methodsFor: 'Updating' stamp: 'BrunoBB 06/23/2015 12:23'!
languageName: newValue
	languageName := newValue! !

!OrbeonFormUserOptions methodsFor: 'Accessing' stamp: 'BrunoBB 11/05/2014 03:48'!
orbeonWebIsLinked

   "Return the value of the instance variable 'orbeonWebIsLinked'."
   ^orbeonWebIsLinked! !

!OrbeonFormUserOptions methodsFor: 'Updating' stamp: 'BrunoBB 11/05/2014 03:48'!
orbeonWebIsLinked: newValue

   "Modify the value of the instance variable 'orbeonWebIsLinked'."
   orbeonWebIsLinked := newValue! !

!OrbeonFormUserOptions methodsFor: 'other' stamp: 'BrunoBB 10/23/2015 11:43'!
setTransitionsDisplayMode: aBoolean

	aBoolean ifTrue: [^initialTransitionDisplayMode := #expanded].

	initialTransitionDisplayMode := #collapsed.! !

!OrbeonFormUserOptions methodsFor: 'other' stamp: 'BrunoBB 10/22/2015 18:46'!
switchTransitionDisplayMode

	initialTransitionDisplayMode := (initialTransitionDisplayMode = #expanded) ifTrue: [#collapsed] ifFalse: [#expanded].! !

!OrbeonFormUserOptions methodsFor: 'other' stamp: 'BrunoBB 10/22/2015 18:46'!
transitionDisplayModeIsExpanded

	^initialTransitionDisplayMode = #expanded! !

!OrbeonObject class methodsFor: 'other' stamp: 'BrunoBB 12/02/2013 09:16'!
new

	^super new initialize! !

!OrbeonObject methodsFor: 'other' stamp: 'BrunoBB 11/30/2013 13:55'!
addToCache

	^self class cache add: self! !

!OrbeonObject methodsFor: 'other' stamp: 'BrunoBB 04/16/2015 06:28'!
isGateway

	^false! !

!OrbeonObject methodsFor: 'other' stamp: 'BrunoBB 08/08/2014 07:41'!
removeFromCache

	^self class cache remove: self ifAbsent: []! !

Object subclass: #OrbeonPermissions
	instanceVariableNames: 'isEnabled permissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/14/2015 13:13'!
add: permissionName to: role

	^(permissions at: role ifAbsentPut: [OrderedCollection new]) add: permissionName! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/25/2015 14:13'!
add: permissionName toAll: arrayRole

	arrayRole do: [:eachRole | self add: permissionName to: eachRole]! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/25/2015 14:22'!
addAll: permissionNameArray toAll: arrayRole

	permissionNameArray do: [:eachPermission | 
		arrayRole do: [:eachRole | | newRoleName |
			newRoleName := (eachRole = 'all') ifTrue: ['anyone'] ifFalse: [eachRole].
			self add: eachPermission to: newRoleName]]! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/14/2015 14:48'!
has: roleName permission: permissionName
	"Answer whether or not the receiver has the permission <permissionName> for the role <roleName>.
	First check for the permission collection of -anyone- then check in the specific collection of the role <roleName>"

	^permissions at: 'anyone' ifPresent: [:permissionsCollection | 
		(permissionsCollection includes: permissionName) 	
		ifFalse: [(permissions at: roleName ifAbsent: [^false]) includes: permissionName]
		ifTrue: [true]].! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/15/2015 13:48'!
hasCreateAccess: roleName

	^self has: roleName permission: 'create'! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/15/2015 13:48'!
hasDeleteAccess: roleName

	^self has: roleName permission: 'delete'! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/15/2015 13:47'!
hasReadAccess: roleName

	^self has: roleName permission: 'read'! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/15/2015 13:48'!
hasUpdateAccess: roleName

	^self has: roleName permission: 'update'! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/14/2015 13:11'!
initialize

	super initialize.

	isEnabled := false.
	permissions := Dictionary new.! !

!OrbeonPermissions methodsFor: 'Accessing' stamp: 'BrunoBB 08/14/2015 13:10'!
isEnabled
	^isEnabled! !

!OrbeonPermissions methodsFor: 'Updating' stamp: 'BrunoBB 08/14/2015 13:10'!
isEnabled: newValue
	isEnabled := newValue! !

!OrbeonPermissions methodsFor: 'Accessing' stamp: 'BrunoBB 08/14/2015 13:10'!
permissions
	^permissions! !

!OrbeonPermissions methodsFor: 'Updating' stamp: 'BrunoBB 08/14/2015 13:10'!
permissions: newValue
	permissions := newValue! !

!OrbeonPermissions methodsFor: 'other' stamp: 'BrunoBB 08/14/2015 13:14'!
remove: permissionName from: role

	^(permissions at: role ifAbsent: [^nil]) remove: permissionName ifAbsent: [nil]! !

Object subclass: #OrbeonUser
	instanceVariableNames: 'centralUser name password rols managerRole languageName'
	classVariableNames: 'Cache'
	poolDictionaries: ''
	category: 'Orbeon'!

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 02/04/2016 10:11'!
addAdminUser

	^self newManagerNamed: 'admin' password: 'admin'! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
allRols

	^(self cache inject: OrderedCollection new into: [:all :each | all addAll: each rols. all]) asSet asSortedCollection! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
cache

	Cache ifNil: [self initializeCache].

	^Cache! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
cache: aCollection

	^Cache := aCollection! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 06/29/2016 07:34'!
getExistingOrNewUserNamed: aString
	"The receiver answer a user named <aString>.
	If the user does not exist creates a new one with password <password>"

	^self getExistingOrNewUserNamed: aString password: ''! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 06/29/2016 07:33'!
getExistingOrNewUserNamed: aString password: password
	"The receiver answer a user named <aString>.
	If the user does not exist creates a new one with password <password>"

	(self getUserNamed: aString) ifNotNil: [:user | ^user].

	^self newUsername: aString password: password! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 06/29/2016 07:34'!
getUserNamed: aString

	^self cache at: aString ifAbsent: [nil]! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
initializeCache
	"self initializeCache"
	Cache := Dictionary new.

	self addAdminUser.! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:44'!
migrateToApplicationUser

	self cache do: [:eachUser | 
		eachUser class migrateTo: OrbeonUser.
		eachUser migrate]! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
named: aString

	^self cache detect: [:each | each name = aString] ifNone: [nil]! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 02/04/2016 10:11'!
newManagerNamed: aString password: password

	^(self newNamed: aString password: password)
			managerRole: OrbeonManagerUserRole new;
			yourself! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 02/04/2016 10:11'!
newNamed: aString password: password
	| user centralUser |

	centralUser := ApplicationUser getExistingOrNewUserNamed: aString.
	centralUser password: password.

	user := self new initialize.
	user centralUser: centralUser.

	user addToCache.

	^user! !

!OrbeonUser class methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
transformCache
	| newCache |

	newCache := Dictionary new.

	self cache do: [:each | newCache at: each name put: each].

	^newCache! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
addRol: rolName

	^rols add: rolName! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:46'!
addToCache

	^self class cache at: self name put: self! !

!OrbeonUser methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2015 11:23'!
centralUser
	^centralUser! !

!OrbeonUser methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2015 11:23'!
centralUser: newValue
	centralUser := newValue! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasCreatePermissionOn: anObeonFormDefinition
	"Answer whether the receiver has permission <permissionName> on Orbeon Form Definition <anObeonFormDefinition>"

	^self hasPermission: 'create' on: anObeonFormDefinition! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasDeletePermissionOn: anObeonFormDefinition
	"Answer whether the receiver has permission <permissionName> on Orbeon Form Definition <anObeonFormDefinition>"

	^self hasPermission: 'delete' on: anObeonFormDefinition! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasExportPermissionOn: anObeonFormDefinition
	"Answer whether the receiver has permission <permissionName> on Orbeon Form Definition <anObeonFormDefinition>"

	^self hasPermission: 'export' on: anObeonFormDefinition! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasPermission: permissionName on: anObeonFormDefinition
	"Answer whether the receiver has permission <permissionName> on Orbeon Form Definition <anObeonFormDefinition>"

	^true
	"^rols anySatisfy: [:each | anObeonFormDefinition hasPermission: permissionName for: each]"! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasReadPermissionOn: anObeonFormDefinition
	"Answer whether the receiver has permission <permissionName> on Orbeon Form Definition <anObeonFormDefinition>"

	^self hasPermission: 'read' on: anObeonFormDefinition! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasRol: rolName

	^rols includes: rolName! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasSynchronizePermissionOn: anObeonFormDefinition
	"Answer whether the receiver has permission <permissionName> on Orbeon Form Definition <anObeonFormDefinition>"

	^self hasPermission: 'sync' on: anObeonFormDefinition! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
hasUpdatePermissionOn: anObeonFormDefinition
	"Answer whether the receiver has permission <permissionName> on Orbeon Form Definition <anObeonFormDefinition>"

	^self hasPermission: 'update' on: anObeonFormDefinition! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
initialize

	super initialize.

	rols := OrderedCollection new.
	managerRole := OrbeonManagerUserRole new.
	languageName := #english! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
isAdmin

	^managerRole isAdmin! !

!OrbeonUser methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2015 11:23'!
languageName
	^languageName! !

!OrbeonUser methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2015 11:23'!
languageName: newValue
	languageName := newValue! !

!OrbeonUser methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2015 11:23'!
managerRole

   "Return the value of the instance variable 'managerRole'."
   ^managerRole! !

!OrbeonUser methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2015 11:23'!
managerRole: newValue

   "Modify the value of the instance variable 'managerRole'."
   managerRole := newValue! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
managerRolName

	^managerRole roleName! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:35'!
migrateFrom: oldUser instVarMap: aMap

	super migrateFrom: oldUser instVarMap: aMap.

	centralUser := ApplicationUser getExistingOrNewUserNamed: name.
	centralUser password: password.! !

!OrbeonUser methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2015 11:46'!
name

   "Return the value of the instance variable 'name'."
   ^centralUser username! !

!OrbeonUser methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2015 11:46'!
name: newValue
   "Modify the value of the instance variable 'name'."

   centralUser username: newValue! !

!OrbeonUser methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2015 11:47'!
password

   "Return the value of the instance variable 'password'."
   ^centralUser password! !

!OrbeonUser methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2015 11:47'!
password: newValue

   "Modify the value of the instance variable 'password'."
   centralUser password: newValue! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:46'!
removeFromCache

	^self class cache removeKey: self name ifAbsent: [nil]! !

!OrbeonUser methodsFor: 'other' stamp: 'BrunoBB 12/20/2015 11:23'!
removeRol: rolName

	^rols remove: rolName ifAbsent: []! !

!OrbeonUser methodsFor: 'Accessing' stamp: 'BrunoBB 12/20/2015 11:23'!
rols

   "Return the value of the instance variable 'rols'."
   ^rols! !

!OrbeonUser methodsFor: 'Updating' stamp: 'BrunoBB 12/20/2015 11:23'!
rols: newValue

   "Modify the value of the instance variable 'rols'."
   rols := newValue! !

!OrbeonUser methodsFor: 'Accessing' stamp: 'BrunoBB 07/03/2016 08:56'!
username
	^centralUser username! !

!Boolean class methodsFor: '*orbeon' stamp: 'BrunoBB 08/21/2016 14:30'!
fromOrbeonString: aString

	^self fromString: aString! !

!Number methodsFor: '*orbeon' stamp: 'BrunoBB 07/02/2014 18:14'!
printMySqlString

	^self printString! !

!Number methodsFor: '*orbeon' stamp: 'BrunoBB 04/09/2016 13:39'!
printOrbeonString

	^self printString! !

!FalseClass methodsFor: '*orbeon' stamp: 'BrunoBB 04/09/2016 13:40'!
printOrbeonString

	^'false'! !

!Time class methodsFor: '*orbeon' stamp: 'BrunoBB 08/21/2016 14:30'!
fromOrbeonString: aString

	^self fromString: aString! !

!Time methodsFor: '*orbeon' stamp: 'BrunoBB 07/02/2014 18:16'!
printMySqlString

	^self hours printString, ':', self minutes printString! !

!Time methodsFor: '*orbeon' stamp: 'BrunoBB 04/09/2016 13:39'!
printOrbeonString

	^self hours printString, ':', self minutes printString! !

!DateTime class methodsFor: '*orbeon' stamp: 'BrunoBB 04/07/2016 10:44'!
fromOrbeonString: aString
	"Answer an instance of the receiver based on time stamp string <aString>.
	The supported TimeStamp format is: Date + Time. Where Date format is defined in Date fromOrbeonString:.
	Time part support p.m. or pm"

	^self fromOrbeonString: aString 
			dateFormat: self getOrbeonWebServerConfiguration dateFormat asUppercase! !

!DateTime class methodsFor: '*orbeon' stamp: 'BrunoBB 04/07/2016 11:37'!
fromOrbeonString: aString dateFormat: stringFormat
	"Answer an instance of the receiver based on time stamp string <aString>.
	The supported TimeStamp format is: Date + Time. Where Date format is defined in Date fromOrbeonString:.
	Time part support p.m. or pm"
	| timeStampArray dateString timeString |

	aString isEmpty ifTrue: [^nil].
	timeStampArray := ((aString findString: 'T') = 0) ifTrue: [aString subStrings: ' '] ifFalse: [aString subStrings: 'T'].

	((timeStampArray size = 2) or: [timeStampArray size = 3]) 
	ifFalse: [OrbeonTimeStampMaskException signal: 'Incorrect time stamp format'].
	"size = 3 --> must includes pm or p.m. or am or a.m."
	(timeStampArray size = 3 and: [(timeStampArray third asUppercase findString: 'M') = 0])
	ifTrue:  [OrbeonTimeStampMaskException signal: 'Incorrect time stamp format'].

	dateString := timeStampArray first.
	timeString := timeStampArray second.
	(timeStampArray size = 3) ifTrue: [timeString := timeString, timeStampArray third].

	^self newWithDate: (Date fromOrbeonString: dateString format: stringFormat) time: (Time fromString: timeString)! !

!DateTime methodsFor: '*orbeon' stamp: 'BrunoBB 08/06/2015 10:12'!
asOrbeonDateTimeString
	"yyyy-mm-ddThh:mm:ss+zzzz"

	| string offset offsetHours offsetMinutes |
	string := self asStringUsingFormat: #(3 2 1 $- 1 1 $: true true false).
	string at: 11 put: $T.
	offset := timeZone offsetAtLocal: self.
	string add: $..
	offset := offset abs // 60.
	offsetHours := offset // 60.
	offsetMinutes := offset \\ 60.
	offsetHours < 10 ifTrue: [string add: $0].
	string addAll: offsetHours printString.
	offsetMinutes < 10 ifTrue: [string add: $0].
	string addAll: offsetMinutes printString.
	^string.! !

!DateTime methodsFor: '*orbeon' stamp: 'BrunoBB 10/15/2015 14:02'!
printMySqlString

	"^self asDate printMySqlString, ' ', self asTime printMySqlString"

	^self asDateAndTime asTimeStamp printMySqlString! !
